c*************************************************************************  
c     This file contains subroutines which call the different
c     functions and subroutines to calculate matrix elements squared.
c
c*************************************************************************  
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     DOUBLE PRECISION FUNCTION Amplitude(rand, p, x, v, nlo, ps_number)
c     SUBROUTINE RE_Amplitude(rand, p, x, v, lokt, xuz, m2s, ps_number)
c     SUBROUTINE QED_Amplitude(rand, p, x, v, lokt, xuz, m2s, ps_number)
c
c*************************************************************************  

c*************************************************************************
      DOUBLE PRECISION FUNCTION Amplitude(rand, p, x, v, nlo, ps_number)
c*************************************************************************
c     This is the manager of all matrixelement routines and the (only) 
c     interface of those to the main program.  Depending on the process ID
c     the right matrix element is called with the correct calling 
c     convention, which might well be individually different.
c     Once many processes are included in the program, this subroutine
c     will be quite complex, but it should actually not be much more than a
c     big select-case statement.
c
c*************************************************************************
c     INPUT
c     rand      : Array with random numbers driven by monaco. 
c     p         : 4-momenta of the partons involved in the process.
c                 The first 2 entries are the incoming partons.
c     x         : The feynman x of the incoming partons 
c     v         : The 4-momenta of the other particles
c     nlo       : switch for NLO processes
c     ps_number : Number specifying the desired matrix element squared
c     
c     OUTPUT
c     Amplitude : The amplitude square as returned by the matrix element code.
c*************************************************************************
      implicit none

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/process.inc"

      double precision x(nx) 
      double precision rand(max_PS_dim)
      double precision p(0:3,max_p,max_kin)
      double precision v(0:3,max_v,max_kin)
      double precision rn, rn2(2), xuz(2,2:max_kin), m2s(0:max_kin)
      double precision m2s_dec,dump
      logical lokt(max_kin)
      integer NLO,i, ps_number

c     to convert "NLO" to box- and pentagon call conventions
      integer VBFNLOswitch(0:5)
      data VBFNLOswitch / 0, 4,-4,-5,-6,-8 /

c     masses and widths
      double precision XM2s(6),XMGs(6)
      COMMON /BKOPOUshort/ XM2s,XMGs
      double precision BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM, BHGAMZ, XGW, XGZ, XGH, GAMT
      COMMON /BRANCH/ BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM, BHGAMZ, XGW, XGZ, XGH, GAMT


c     external matrixelement functions
      double precision m2s_ubtd
      double precision m2s_wbfvv
      double precision m2s_wbfw
      double precision m2s_ggfh
      double precision RandomNumber
      double precision m2s_qqVV
      double precision m2s_qqZZ
      double precision m2s_qqZA
      double precision m2s_qqAA
      double precision m2s_qqWWZ
      double precision m2s_qqZZZ
      double precision m2s_qqZZW
      double precision m2s_qqWWW
      double precision m2s_qqWWA
      double precision m2s_qqZZA
      double precision m2s_qqWZA
      double precision m2s_qqWAA
      double precision m2s_qqZAA
      double precision m2s_qqAAA
      double precision m2s_qqWAAj2
      double precision m2s_qqwgaglu_hel
      double precision m2s_qqwzglu_hel 
      double precision m2s_qqwpgaglu_hel
      double precision m2s_qqwpzglu_hel
      double precision m2s_qqWZ
      double precision m2s_qqWA
      double precision m2s_wbfwpp


      integer mu
      external m2s_ubtd
      external m2s_wbfvv
      external m2s_wbfw
      external m2s_ggfh
      external RandomNumber
      external m2s_qqVV
      external m2s_qqZZ
      external m2s_qqZA
      external m2s_qqAA
      external m2s_qqWWZ
      external m2s_qqZZZ
      external m2s_qqZZW
      external m2s_qqWWW
      external m2s_qqWWA
      external m2s_qqZZA
      external m2s_qqWZA
      external m2s_qqWAA
      external m2s_qqZAA
      external m2s_qqAAA
      external m2s_qqWAAj2
      external m2s_qqwgaglu_hel
      external m2s_qqwzglu_hel 
      external m2s_qqwpgaglu_hel
      external m2s_qqwpzglu_hel
      external m2s_qqWZ
      external m2s_qqWA
      external m2s_wbfwpp

c     auxiliary 
      double precision amp, Q(0:3)
      integer lflavr(2)
      double precision mass2
      external mass2

c initialize
      do mu=1,2
        do i=2,max_kin
          xuz(mu,i) = 0d0
        enddo
      enddo

      SELECT CASE(process)

#ifdef WITH_VBF
**********************
*
*     VBF : Hjj
*
**********************
c     pp -> Hjj
      CASE(Hjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> Hjj -> gamma gamma jj
      CASE(Hjj_AA)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         AMplitude = Amplitude*BHGAM
c     pp -> Hjj -> mu+mu- jj
      CASE(Hjj_mu)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHMU
c     pp -> Hjj -> tau+tau- jj
      CASE(Hjj_tau)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjj -> b bbar jj
      CASE(Hjj_bbar)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = AMplitude*BHBB
c     pp -> Hjj -> W W jj
      CASE(Hjj_WW)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),3,4,1,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(Hjj_WPhadWM, Hjj_WPWMhad)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         if (process.eq.Hjj_WPhadWM) then
            call m2s_VVsum_had(v(0,1,1),3,4,3,2,m2s_dec)
         else
            call m2s_VVsum_had(v(0,1,1),3,4,1,4,m2s_dec)
         endif
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjj -> Z Z jj 
      CASE(Hjj_ZZ_ll)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(Hjj_ZZhad)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum_had(v(0,1,1),2,2,5,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(Hjj_ZZ_lnu)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec  

**********************
*
*     VBF : Hjjj
*
**********************
c     pp -> Hjjj
      CASE(HjjjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> Hjjj -> gamma gamma jjj
      CASE(HjjjLO_AA)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHGAM
c     pp -> spin-2 jjj -> gamma gamma jjj
      CASE(AAjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 11, VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> Hjjj -> mu+mu- jjj
      CASE(HjjjLO_mu)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHMU
c     pp -> Hjjj -> tau+tau- jjj
      CASE(HjjjLO_tau)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjjj -> b bbar jjj
      CASE(HjjjLO_bbar)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = AMplitude*BHBB
c     pp -> Hjjj -> W W jjj
      CASE(HjjjLO_WW)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),3,4,1,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(HjjjLO_WPhadWM, HjjjLO_WPWMhad)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         if (process.eq.HjjjLO_WPhadWM) then
            call m2s_VVsum_had(v(0,1,1),3,4,3,2,m2s_dec)
         else
            call m2s_VVsum_had(v(0,1,1),3,4,1,4,m2s_dec)
         endif
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjjj -> Z Z jjj
      CASE(HjjjLO_ZZ_ll)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(HjjjLO_ZZhad)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum_had(v(0,1,1),2,2,5,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(HjjjLO_ZZ_lnu)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
#endif

#ifdef WITH_HJJJ
********************
*
*     VBF : Hjjj
*
********************
c     pp -> Hjjj
      CASE(Hjjj)
         rn = RandomNumber()
         call m2s_wbfhj(6, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
c         pp -> Hjjj -> gamma gamma jjj
      CASE(Hjjj_AA)
         rn = RandomNumber()
         call m2s_wbfhj(6, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         Amplitude = Amplitude*BHGAM
c     pp -> Hjjj -> mu+mu- jjj
      CASE(Hjjj_mu)
         rn = RandomNumber()
         call m2s_wbfhj(6, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         Amplitude = Amplitude*BHMU
c     pp -> Hjjj -> tau+tau- jjj
      CASE(Hjjj_tau)
         rn = RandomNumber()
         call m2s_wbfhj(6, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjjj -> b bbar jjj
      CASE(Hjjj_bbar)
         rn = RandomNumber()
         call m2s_wbfhj(6, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         Amplitude = AMplitude*BHBB
c     pp -> Hjjj -> W W jjj
      CASE(Hjjj_WW)
         rn = RandomNumber()
         call m2s_wbfhj(6, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjjj -> Z Z jjj 
      CASE(Hjjj_ZZ_ll)
         rn = RandomNumber()
         call m2s_wbfhj(6, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(Hjjj_ZZ_lnu)
         rn = RandomNumber()
         call m2s_wbfhj(6, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec

*********************
*
*     VBF : Hjjjj
*
*********************
c     pp -> Hjjjj
      CASE(HjjjjLO)
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0)
c     pp -> Hjjjj-> gamma gamma jjjj
      CASE(HjjjjLO_AA)
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0) 
         Amplitude = Amplitude*BHGAM
c     pp -> Hjjjj -> mu+mu- jjjj
      CASE(HjjjjLO_mu)
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHMU
c     pp -> Hjjjj -> tau+tau- jjjj
      CASE(HjjjjLO_tau)
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjjjj -> b bbar jjjj
      CASE(HjjjjLO_bbar)
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHBB
c     pp -> Hjjjj -> W W jjjj
      CASE(HjjjjLO_WW)
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjjjj -> Z Z jjjj 
      CASE(HjjjjLO_ZZ_ll)
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(HjjjjLO_ZZ_lnu)
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec 
         
#endif

#ifdef WITH_VBF
***********************
*
*     VBF : Wjj, Zjj, Ajj
*
***********************
c     pp -> Zjj
      CASE(Zjj_l, Zjj_nu) 
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(2,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> W+jj
      CASE(WPjj) 
         rn = RandomNumber()
         Amplitude = m2s_wbfw(3,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> W-jj
      CASE(WMjj) 
         rn = RandomNumber()
         Amplitude = m2s_wbfw(4,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> Ajj
      CASE(Ajj)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(1,nlo,x,p(0,1,1),v(0,1,1),rn)

**************************
*
*     VBF : Wjjj, Zjjj, Ajjj
*
**************************
c     pp -> Zjjj
      CASE(ZjjjLO_l, ZjjjLO_nu) 
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 2, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W+jjj
      CASE(WPjjjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 3, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W-jjj
      CASE(WMjjjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 4, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp ->Ajjj
      CASE(AjjjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 1, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)

******************************
*
*     VBF : WWjj, ZZjj, WZjj
*
******************************
c     pp -> WWjj 
      CASE(WPWMjj, WPhadWMjj, WPWMhadjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(34,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> W+W+jj 
      CASE(WPWPjj,WMWMjj, WPhadWPjj, WMhadWMjj)  ! note that W-W-jj is obtained from W+W+jj
         rn = RandomNumber()
         Amplitude = m2s_wbfwpp(33,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> ZZjj 
      CASE(ZZjj_ll, ZZhadjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(22,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
      CASE(ZZjj_lnu)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(21,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> W+Zjj
      CASE(WPZjj, WPhadZjj, WPZhadjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfw(32,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> W-Zjj
      CASE(WMZjj, WMhadZjj, WMZhadjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfw(42,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> spin-2 (or Higgs) jj -> aajj
      CASE(AAjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(11,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)

**********************************
*
*     VBF : WWjjj, ZZjjj, WZjjj
*
**********************************
c     pp -> WW jjj
      CASE(WPWMjjjLO, WPhadWMjjjLO, WPWMhadjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 34, VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> ZZ jjj
      CASE(ZZjjjLO_ll, ZZhadjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 22, VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
      CASE(ZZjjjLO_lnu)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 21, VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W+Z jjj
      CASE(WPZjjjLO, WPhadZjjjLO, WPZhadjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 32,0,  lokt, 
     1          x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W-Z jjj
      CASE(WMZjjjLO, WMhadZjjjLO, WMZhadjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 42,0, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W+W+ jjj
      CASE(WPWPjjjLO, WMWMjjjLO, WPhadWPjjjLO, WMhadWMjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfwppg( 33,0, lokt, 
!         call m2s_wbfwppg( 33,VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
         Amplitude=m2s(0)

***********************
*
*     VBF : HAjj
*
***********************
c     pp -> HAjj
      CASE(HAjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(61,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> HAjj -> A gamma gamma jj
      CASE(HAjj_AA)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(61,nlo,x,p(0,1,1),v(0,1,1),rn)
         AMplitude = Amplitude*BHGAM
c     pp -> HAjj -> A mu+mu- jj
      CASE(HAjj_mu)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(61,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHMU
c     pp -> HAjj -> A tau+tau- jj
      CASE(HAjj_tau)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(61,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjj -> A b bbar jj
      CASE(HAjj_bbar)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(61,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHBB
c     pp -> HAjj -> A W W jj
      CASE(HAjj_WW)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(61,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> HAjj -> A Z Z jj 
      CASE(HAjj_ZZ_ll)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(61,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(HAjj_ZZ_lnu)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(61,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec  


***********************
*
*     VBF : HAjjj
*
***********************
c     pp -> HAjjj
      CASE(HAjjjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 16, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> HAjjjLO -> A gamma gamma jj
      CASE(HAjjjLO_AA)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 16, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         AMplitude = Amplitude*BHGAM
c     pp -> HAjjjLO -> A mu+mu- jj
      CASE(HAjjjLO_mu)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 16, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHMU
c     pp -> HAjjjLO -> A tau+tau- jj
      CASE(HAjjjLO_tau)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 16, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjj -> A b bbar jj
      CASE(HAjjjLO_bbar)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 16, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHBB
c     pp -> HAjjjLO -> A W W jj
      CASE(HAjjjLO_WW)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 16, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> HAjjjLO -> A Z Z jj 
      CASE(HAjjjLO_ZZ_ll)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 16, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
      CASE(HAjjjLO_ZZ_lnu)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 16, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         Amplitude=m2s(0)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec  

#endif

#ifdef WITH_DIBOSON
*********************
*
*     DIBOSON : WW, WZ, ZZ, WA, ZA, AA
*
*********************
c pp -> W+ W- -> ve e+ e- ve~
      CASE(WW,WPhadWMlep,WPlepWMhad)
         rn = RandomNumber()
         Amplitude = m2s_qqvv( 34, VBFNLOswitch(nlo), x, p, v, rn)
      CASE(GFWW,GFWPhadWMlep,GFWPlepWMhad)
         rn = RandomNumber()
         Amplitude = m2s_qqvv( 34, -8, x, p, v, rn)
c pp -> W+ Z -> ve e+ mu- mu+

      CASE(WPZ,WPhadZ,WPZhad)
         rn = RandomNumber()
         Amplitude = m2s_qqWZ( 32, nlo, x, p, v, rn)
c pp -> W- Z -> ve e- mu- mu+

      CASE(WMZ,WMhadZ,WMZhad)
         rn = RandomNumber()
         Amplitude = m2s_qqWZ( 42, nlo, x, p, v, rn)

c pp -> Z Z -> e+ e- mu+ mu-
      CASE(ZZ,ZZhad)
         rn = RandomNumber()
         Amplitude = m2s_qqzz( 22, VBFNLOswitch(nlo), x, p, v, rn)
      CASE(GFZZ,GFZZhad)
         rn = RandomNumber()
         Amplitude = m2s_qqzz( 22, -8, x, p, v, rn)

c pp -> W+ A -> ve e+ mu- mu+

      CASE(WPA)
         rn = RandomNumber()
         Amplitude = m2s_qqWA( 31, nlo, x, p, v, rn)

c pp -> W- A -> ve e+ mu- mu+

      CASE(WMA)
         rn = RandomNumber()
         Amplitude = m2s_qqWA( 41, nlo, x, p, v, rn)

c pp -> Z A -> e+ e- A
      CASE(ZA)
         rn = RandomNumber()
         Amplitude = m2s_qqza( 21, VBFNLOswitch(nlo), x, p, v, rn)
      CASE(GFZA)
         rn = RandomNumber()
         Amplitude = m2s_qqza( 21, -8, x, p, v, rn)

c pp -> A A 
      CASE(AA)
         rn = RandomNumber()
         Amplitude = m2s_qqAA( 11, VBFNLOswitch(nlo), x, p, v, rn)
      CASE(GFAA)
         rn = RandomNumber()
         Amplitude = m2s_qqAA( 11, -8, x, p, v, rn)


**********************
*
*     DIBOSON : WWj, WZj, ZZj, WAj, ZAj, AAj
*
**********************
c pp -> W+ W- j -> ve e+ e- ve~ j
      CASE(WWjLO,WPhadWMlepjLO,WPlepWMhadjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqVVj( 34, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)
       CASE(WPZjLOb,WPhadZjLO,WPZhadjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWZj( 32, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)
      CASE(WMZjLOb,WMhadZjLO,WMZhadjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWZj( 42, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)

c pp -> Z Z j -> e- e+ mu- mu+ j
      CASE(ZZjLO,ZZhadjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZZj( 22, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)

      CASE(WPAjLOb)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWAj( 31, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)
      CASE(WMAjLOb)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWAj( 41, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)      

c pp -> Z A j -> e- e+ A j
      CASE(ZAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZAj( 21, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)

c pp -> A A j 
      CASE(AAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         do i=2,n_kin
           lokt(i) = .false.
         enddo
         call m2s_qqAAj( 11, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)

#endif          

#ifdef WITH_DIBOSONJET
      CASE(WMAJ)
         if (nlo.eq.0) then
            Amplitude = m2s_qqwgaglu_hel(x,p(0,1,1),v(0,1,1),nlo,.false.)
         else 
#ifdef WITH_NLO
            Call m2s_ppwgajvirt(x,rn,.false.,p(0,1,1),v(0,1,1), m2s(0),nlo)
            Amplitude=m2s(0)
#endif
         endif
      CASE(WPAJ)
         if (nlo.eq.0) then
            Amplitude = m2s_qqwpgaglu_hel(x,p(0,1,1),v(0,1,1),nlo,.true.)
         else 
#ifdef WITH_NLO
            Call m2s_ppwpgajvirt(x,rn,.true.,p(0,1,1),v(0,1,1), m2s(0),nlo)
            Amplitude=m2s(0)
#endif
         endif
      CASE(WPZJ, WPhadZJ, WPZhadJ)
         if (nlo.eq.0) then
            Amplitude = m2s_qqwpzglu_hel(x,p(0,1,1),v(0,1,1),nlo,.true.)
         else 
#ifdef WITH_NLO
            Call m2s_ppwpzjvirt(x,rn,.true.,p(0,1,1),v(0,1,1), m2s(0),nlo)
            Amplitude=m2s(0)
#endif
         endif
      CASE(WMZJ, WMhadZJ, WMZhadJ)
         if (nlo.eq.0) then
            Amplitude = m2s_qqwzglu_hel(x,p(0,1,1),v(0,1,1),nlo,.false.)
         else 
#ifdef WITH_NLO
            Call m2s_ppwzjvirt(x,rn,.false.,p(0,1,1),v(0,1,1), m2s(0),nlo)
            Amplitude=m2s(0)
#endif
         endif
      CASE(WMAJLO)
         lokt(1) = .true.
         do i=2,max_kin
         lokt(i) = .false.
         enddo
         CALL m2s_qqwgagluj(xuz,0,.false.,lokt,x,p,v,m2s)
         Amplitude = m2s(0)
      CASE(WPAJLO)
         lokt(1) = .true.
         do i=2,max_kin
         lokt(i) = .false.
         enddo
         CALL m2s_qqwpgagluj(xuz,0,.true.,lokt,x,p,v,m2s)
         Amplitude = m2s(0)
      CASE(WMZJLO, WMhadZJjLO, WMZhadJjLO)
         lokt(1) = .true.
         do i=2,max_kin
         lokt(i) = .false.
         enddo
         CALL m2s_qqwzgluj(xuz,0,.false.,lokt,x,p,v,m2s)
         Amplitude = m2s(0)
      CASE(WPZJLO, WPhadZJjLO, WPZhadJjLO)
         lokt(1) = .true.
         do i=2,max_kin
         lokt(i) = .false.
         enddo
         CALL m2s_qqwpzgluj(xuz,0,.true.,lokt,x,p,v,m2s)
         Amplitude = m2s(0)
#endif

#ifdef WITH_TRIBOSON
**********************************************************************
*
*     TRIBOSON : WWZ, ZZZ, ZZW, WWW, WWA, ZZA, WZA, WAA, ZAA, AAA
*
**********************************************************************	 
c pp -> W+ W- Z -> ve e+ e- ve~ e- e+
      CASE(WWZ, WPhadWMZ, WPWMhadZ, WWZhad)
         rn = RandomNumber()
         Amplitude = m2s_qqWWZ(342,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> Z Z Z -> e- e+ e- e+ e- e+
      CASE(ZZZ, ZZZhad)
         rn = RandomNumber()
         Amplitude = m2s_qqZZZ(222,VBFNLOswitch(nlo),x,p,v,rn,
     1                                       narrowwidth,ps_number)

c pp -> Z Z W+ -> e- e+ e- e+ ve e+
      CASE(ZZWP, ZZhadWP, ZZWPhad)
         rn = RandomNumber()
         Amplitude = m2s_qqZZW(223,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> Z Z W- -> e- e+ e- e+ e- ve~
      CASE(ZZWM, ZZhadWM, ZZWMhad)
         rn = RandomNumber()
         Amplitude = m2s_qqZZW(224,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> W+W-W+ -> ve e+ e- ve~ ve e+
      CASE(WWWP, WPhadWMWP, WPWMhadWP)
         rn = RandomNumber()
         Amplitude = m2s_qqWWW( 343,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> W+W-W- -> e- ve~ ve e+ e- ve~
      CASE(WWWM, WMhadWPWM, WMWPhadWM)
         rn = RandomNumber()
         Amplitude = m2s_qqWWW( 434,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> W+ W- A -> ve e+ e- ve~ A
      CASE(WWA)
         rn = RandomNumber()
         Amplitude = m2s_qqWWA( 341, VBFNLOswitch(nlo), x, p, v, rn)

c pp -> Z Z A -> e- e+ e- e+ A
      CASE(ZZA)
         rn = RandomNumber()
         Amplitude = m2s_qqZZA( 221, VBFNLOswitch(nlo), x, p, v, rn)

c pp -> W+ Z A -> e+ ve e- e+ A
      CASE(WPZA)
         rn = RandomNumber()
         Amplitude = m2s_qqWZA( 321, VBFNLOswitch(nlo), x, p, v, rn, ps_number)

c pp -> W- Z A -> e- ve~ e- e+ A
      CASE(WMZA)
         rn = RandomNumber()
         Amplitude = m2s_qqWZA( 421, VBFNLOswitch(nlo), x, p, v, rn, ps_number)

c pp -> W+ A A -> e+ ve A A
      CASE(WPAA)
         rn = RandomNumber()
         Amplitude = m2s_qqWAA( 311, VBFNLOswitch(nlo), x, p, v, rn, 
     &        ps_number, narrowwidth)

c pp -> W- A A -> e- ve~ A A
      CASE(WMAA)
         rn = RandomNumber()
         Amplitude = m2s_qqWAA( 411, VBFNLOswitch(nlo), x, p, v, rn, 
     &        ps_number, narrowwidth)

c pp -> Z A A -> e+ e- A A
      CASE(ZAA)
         rn = RandomNumber()
         Amplitude = m2s_qqZAA( 211, VBFNLOswitch(nlo), x, p, v, rn, 
     &        ps_number, narrowwidth)

c pp -> Z A A -> ve~ ve A A
      CASE(ZnAA)
         rn = RandomNumber()
         Amplitude = m2s_qqZAA( 511, VBFNLOswitch(nlo), x, p, v, rn, 
     &        ps_number, narrowwidth)

c pp -> A A A 
      CASE(AAA)
         rn = RandomNumber()
         Amplitude = m2s_qqAAA( 111, VBFNLOswitch(nlo), x, p, v, rn, ps_number)

*****************************************************************************
*
*     TRIBOSON : WWZj, ZZZj, ZZWj, WWWj, WWAj, ZZAj, WZAj, WAAj, ZAAj, AAAj
*
*****************************************************************************
c pp -> W+ W- Z j -> ve e+ e- ve~ e- e+ j
      CASE(WWZjLO, WPhadWMZjLO, WPWMhadZjLO, WWZhadjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWWZj(342,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> Z Z Z j -> e- e+ e- e+ e- e+ j
      CASE(ZZZjLO, ZZZhadjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZZZj(222, 0, lokt, x, p, v, rn, xuz(1,2), m2s,
     1                       narrowwidth, ps_number)
         Amplitude = m2s(0)

c pp -> Z Z W+ j -> e- e+ e- e+ ve e+ j
      CASE(ZZWPjLO, ZZhadWPjLO, ZZWPhadjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZZWj(223,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> Z Z W- j -> e- e+ e- e+ e- ve~ j
      CASE(ZZWMjLO, ZZhadWMjLO, ZZWMhadjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZZWj(224,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> W+W-W+j -> ve e+ e- ve~ vmu mu+ j
      CASE(WWWPjLO, WPhadWMWPjLO, WPWMhadWPjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWWWj(343,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> W+W-W-j -> e- ve~ ve e+ e- ve~ j
      CASE(WWWMjLO, WMhadWPWMjLO, WMWPhadWMjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWWWj(434,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> W+ W- A j -> ve e+ e- ve~ A j
      CASE(WWAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqVVV1j( 341, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)

c pp -> Z Z A j -> e- e+ e- e+ A j
      CASE(ZZAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZZAj( 221, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)

c pp -> W+ Z A j -> e+ ve e- e+ A j
      CASE(WPZAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWZAj( 321, 0, lokt,x, p, v, rn, xuz(1,2), m2s, ps_number)
         Amplitude = m2s(0)

c pp -> W- Z A j -> e- ve~ e- e+ A j
      CASE(WMZAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWZAj( 421, 0, lokt,x, p, v, rn, xuz(1,2), m2s, ps_number)
         Amplitude = m2s(0)

c pp -> W+ A A j -> e+ ve A A j
      CASE(WPAAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         do i = 2, 11
            lokt(i) = .false.
         end do
         call m2s_qqWAAj( 311, 0, lokt,x, p, v, rn, xuz(1,2), m2s, 
     -        ps_number, narrowwidth)
         Amplitude = m2s(0)

c pp -> W- A A j -> e- ve~ A A j
      CASE(WMAAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         do i = 2, 11
            lokt(i) = .false.
         end do
         call m2s_qqWAAj( 411, 0, lokt,x, p, v, rn, xuz(1,2), m2s, 
     -        ps_number, narrowwidth)
         Amplitude = m2s(0)

c pp -> Z A A j -> e+ e- A A j
      CASE(ZAAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZAAj( 211, 0, lokt,x, p, v, rn, xuz(1,2), m2s, 
     -        ps_number, narrowwidth)
         Amplitude = m2s(0)

c pp -> Z A A j -> ve~ ve A A j
      CASE(ZnAAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZAAj( 511, 0, lokt,x, p, v, rn, xuz(1,2), m2s, 
     &        ps_number, narrowwidth)
         Amplitude = m2s(0)

c pp -> A A A j 
      CASE(AAAjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqAAAj( 111, 0, lokt,x, p, v, rn, xuz(1,2), m2s, ps_number)
         Amplitude = m2s(0)

#endif

**************************************
*
*     VVVj 
*
**************************************
#ifdef WITH_TRIBOSONJET
c pp -> W+ A A j -> e+ ve A A j
      CASE(WPAAj)
         rn = RandomNumber()
         Amplitude = m2s_qqWAAj2( 311, VBFNLOswitch(nlo), x, p, v, rn, 
     &        ps_number, narrowwidth)

c pp -> W- A A j -> e- ve~ A A j
      CASE(WMAAj)
         rn = RandomNumber()
         Amplitude = m2s_qqWAAj2( 411, VBFNLOswitch(nlo), x, p, v, rn,
     &        ps_number, narrowwidth)

c pp -> W+ A A j j -> e+ ve A A j j
      CASE(WPAAjjLO)
         rn = RandomNumber()
         lokt( 1) = .true.
         do i = 2, 11
            lokt(i) = .false.
         end do
         call m2s_qqWAAjj( 311, 0, lokt,x, p, v, rn, xuz(1,2), 
     &        ps_number, narrowwidth, m2s)
         Amplitude = m2s(0)

c pp -> W- A A j j -> e- ve~ A A j j
      CASE(WMAAjjLO)
         rn = RandomNumber()
         lokt( 1) = .true.
         do i = 2, 11
            lokt(i) = .false.
         end do
         call m2s_qqWAAjj( 411, 0, lokt,x, p, v, rn, xuz(1,2), 
     &        ps_number, narrowwidth, m2s)
         Amplitude = m2s(0)
#endif


#ifdef WITH_GGF
**************************************
*
*     GF : Hjj 
*
**************************************
c     GF pp -> Hjj
      CASE(GFHjj)
         do mu = 0,3 !     sum the 2 lepton momenta up to Higgs momentum
            q(mu) = v(mu,1,1)+v(mu,2,1)
         enddo
         Amplitude = m2s_ggfh( x, p(0,1,1),q,rand )
*SEGorig         Amplitude = m2s_ggfh( x, p(0,1,1),v(0,1,1),rand )
c     GF pp -> Hjj -> gamma gamma jj
      CASE(GFHjj_AA)
         do mu = 0,3 !     sum the 2 lepton momenta up to Higgs momentum
            q(mu) = v(mu,1,1)+v(mu,2,1)
         enddo
         m2s_dec = q(0)*q(0) - q(1)*q(1) - q(2)*q(2) - q(3)*q(3)
         amplitude = m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
         AMplitude = Amplitude*BHGAM
         Amplitude = Amplitude*16D0*pi*XMGS(6)/
     -        ((m2s_dec - XM2S(6))**2 + XMGS(6)**2)
c     GF pp -> Hjj -> mu+ mu- jj
      CASE(GFHjj_mu)  
         do mu = 0,3 !     sum the 2 lepton momenta up to Higgs momentum
            q(mu) = v(mu,1,1)+v(mu,2,1)
         enddo
         m2s_dec = q(0)*q(0) - q(1)*q(1) - q(2)*q(2) - q(3)*q(3)
         amplitude = m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
         Amplitude = Amplitude*BHMU
         Amplitude = Amplitude*16D0*pi*XMGS(6)/
     -        ((m2s_dec - XM2S(6))**2 + XMGS(6)**2)
c     GF pp -> Hjj -> tau+ tau- jj
      CASE(GFHjj_tau)
         do mu = 0,3 !     sum the 2 lepton momenta up to Higgs momentum
            q(mu) = v(mu,1,1)+v(mu,2,1)
         enddo
         m2s_dec = q(0)*q(0) - q(1)*q(1) - q(2)*q(2) - q(3)*q(3)
         amplitude = m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
         Amplitude = Amplitude*BHTAU
         Amplitude = Amplitude*16D0*pi*XMGS(6)/
     -        ((m2s_dec - XM2S(6))**2 + XMGS(6)**2)
c     GF pp -> Hjj -> b bbar jj
      CASE(GFHjj_bbar)
         do mu = 0,3 !     sum the 2 lepton momenta up to Higgs momentum
            q(mu) = v(mu,1,1)+v(mu,2,1)
         enddo
         m2s_dec = q(0)*q(0) - q(1)*q(1) - q(2)*q(2) - q(3)*q(3)
         amplitude = m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
         Amplitude = amplitude*BHBB
         Amplitude = Amplitude*16D0*pi*XMGS(6)/
     -        ((m2s_dec - XM2S(6))**2 + XMGS(6)**2)
c     GF pp -> Hjj -> WWjj -> llnunu jj
      CASE(GFHjj_WW)
         do mu = 0,3 !     sum the 4 lepton momenta up to Higgs momentum
            q(mu) = v(mu,1,1)+v(mu,2,1)+v(mu,3,1)+v(mu,4,1)
         enddo
         call m2s_VVsum(v(0,1,1),3,4,2,1,amp)
         amp = amp / ( (xm2s(6) - mass2(q) )**2  +  xmgs(6)**2 ) 
         Amplitude = amp * m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
c     GF pp -> Hjj -> Z Z jj 
      CASE(GFHjj_ZZ_ll)
         do mu = 0,3 !     sum the 4 lepton momenta up to Higgs momentum
            q(mu) = v(mu,1,1)+v(mu,2,1)+v(mu,3,1)+v(mu,4,1)
         enddo
         call m2s_VVsum(v(0,1,1),2,2,2,2,amp)
         amp = amp / ( (xm2s(6) - mass2(q) )**2  +  xmgs(6)**2 ) 
         Amplitude = amp * m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
      CASE(GFHjj_ZZ_lnu)
         do mu = 0,3 !     sum the 4 lepton momenta up to Higgs momentum
            q(mu) = v(mu,1,1)+v(mu,2,1)+v(mu,3,1)+v(mu,4,1)
         enddo
         call m2s_VVsum(v(0,1,1),2,2,2,1,amp)
         amp = amp / ( (xm2s(6) - mass2(q) )**2  +  xmgs(6)**2 ) 
         Amplitude = amp * m2s_ggfh( x, p(0,1,1),q,rand ) ! pp-> Hjj
#endif

      CASE DEFAULT
         print *,"Error: Amplitude for process not implemented, ID = ",procID
         print *,"Check process-ID and compiler flags"
         STOP
      END SELECT


      RETURN
      END
       


#ifdef WITH_NLO
c*************************************************************************
      SUBROUTINE RE_Amplitude(rand, p, x, v, lokt, xuz, m2s, ps_number)
c*************************************************************************
c     Amplitude for real emissions.
c*************************************************************************
      implicit none

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/process.inc"

      double precision x(nx) 
      double precision rand(max_PS_dim)
      double precision p(0:3,max_p,max_kin)
      double precision v(0:3,max_v,max_kin)
      double precision rn, xuz(2,2:max_kin), m2s(0:max_kin), m2s_dec
      logical lokt(max_kin)
      integer L, ps_number

c     masses and widths
      double precision XM2s(6),XMGs(6)
      COMMON /BKOPOUshort/ XM2s,XMGs
      double precision BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM, BHGAMZ, XGW, XGZ, XGH, GAMT
      COMMON /BRANCH/ BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM, BHGAMZ, XGW, XGZ, XGH, GAMT

      double precision RandomNumber
      external RandomNumber
   
      double complex ahvv(3,4,4), ahvvL(3,4,4)
      common/tensorhvv/ ahvv, ahvvL


      SELECT CASE(process)

#ifdef WITH_VBF
**************************************
*
*     VBF : Hjjj, WWjjj, ZZjjj, WZjjj
*
**************************************
c     pp -> Hjjj
      CASE(Hjj)
         rn = RandomNumber()
         call m2s_wbfzhg( 6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> Hjjj -> gamma gamma jjj
      CASE(Hjj_AA)
         rn = RandomNumber()
         call m2s_wbfzhg( 6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHGAM
         enddo
c     pp -> spin-2 jjj -> gamma gamma jjj
      CASE(AAjj)
         rn = RandomNumber()
         call m2s_wbfzhg( 11, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
c     pp -> Hjjj -> mu+mu- jjj
      CASE(Hjj_mu)
         rn = RandomNumber()
         call m2s_wbfzhg( 6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHMU
         enddo
c     pp -> Hjjj -> tau+tau- jjj
      CASE(Hjj_tau)
         rn = RandomNumber()
         call m2s_wbfzhg( 6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHTAU
         enddo
c     pp -> Hjjj -> b bbar jjj
      CASE(Hjj_bbar)
         rn = RandomNumber()
         call m2s_wbfzhg( 6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHBB
         enddo
c     pp -> Hjjj -> W W jjj
      CASE(Hjj_WW)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),3,4,1,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
      CASE(Hjj_WPhadWM, Hjj_WPWMhad)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         if (process.eq.Hjj_WPhadWM) then
            call m2s_VVsum_had(v(0,1,1),3,4,3,2,m2s_dec)
         else
            call m2s_VVsum_had(v(0,1,1),3,4,1,4,m2s_dec)
         endif
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> Hjjj -> Z Z jjj
      CASE(Hjj_ZZ_ll)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
      CASE(Hjj_ZZhad)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum_had(v(0,1,1),2,2,5,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
      CASE(Hjj_ZZ_lnu)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
#endif

#ifdef WITH_HJJJ
**************************************
*
*     VBF : Hjjjj
*
**************************************
c     pp -> Hjjjj
      CASE(Hjjj)
         call m2s_wbfh4j(6, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
c     pp -> Hjjjj -> gamma gamma jjjj
      CASE(Hjjj_AA)
         call m2s_wbfh4j(6, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         do L = 0,n_kin
            m2s(L) = m2s(L)*BHGAM
         enddo
c     pp -> Hjjjj -> mu+mu- jjjj
      CASE(Hjjj_mu)
         call m2s_wbfh4j(6, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         do L = 0,n_kin
            m2s(L) = m2s(L)*BHMU
         enddo
c     pp -> Hjjjj -> tau+tau- jjjj
      CASE(Hjjj_tau)
         call m2s_wbfh4j(6, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         do L = 0,n_kin
            m2s(L) = m2s(L)*BHTAU
         enddo
c     pp -> Hjjjj -> b bbar jjjj
      CASE(Hjjj_bbar)
         call m2s_wbfh4j(6, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         do L = 0,n_kin
            m2s(L) = m2s(L)*BHBB
         enddo
c     pp -> Hjjjj -> W W jjjj
      CASE(Hjjj_WW)
         call m2s_wbfh4j(6, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_kin
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> Hjjjj -> Z Z jjjj
      CASE(Hjjj_ZZ_ll)
         call m2s_wbfh4j(6, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_kin
            m2s(L) = m2s(L)*m2s_dec
         enddo

      CASE(Hjjj_ZZ_lnu)
         call m2s_wbfh4j(6, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_kin
            m2s(L) = m2s(L)*m2s_dec
         enddo
#endif 

#ifdef WITH_VBF
**************************************
*
*     VBF : Wjjj, Zjjj
*
**************************************
c     pp -> Ajjj 
      CASE(Ajj)
         rn = RandomNumber()
         call m2s_wbfzhg( 1, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> Zjjj 
      CASE(Zjj_l, Zjj_nu) 
         rn = RandomNumber()
         call m2s_wbfzhg( 2, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> W+ jjj
      CASE(WPjj) 
         rn = RandomNumber()
         call m2s_wbfw3j( 3, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> W- jjj
      CASE(WMjj)
         rn = RandomNumber()
         call m2s_wbfw3j( 4, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)


**************************************
*
*     VBF : WWjjj, ZZjjj, WZjjj
*
**************************************
c     pp -> WW jjj
      CASE(WPWMjj, WPhadWMjj, WPWMhadjj)
         rn = RandomNumber()
         call m2s_wbfzhg( 34, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
c     pp -> ZZ jjj
      CASE(ZZjj_ll, ZZhadjj)
         rn = RandomNumber()
         call m2s_wbfzhg( 22, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
      CASE(ZZjj_lnu)
         rn = RandomNumber()
         call m2s_wbfzhg( 21, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
c     pp -> W+Z jjj
      CASE(WPZjj, WPhadZjj, WPZhadjj) 
         rn = RandomNumber()
         call m2s_wbfw3j( 32, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> W-Z jjj
      CASE(WMZjj, WMhadZjj, WMZhadjj) 
         rn = RandomNumber()
         call m2s_wbfw3j( 42, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
      CASE(WPWPjj, WMWMjj, WPhadWPjj, WMhadWMjj)
         rn = RandomNumber()
         call m2s_wbfwppg( 33,1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 

**************************************
*
*     VBF : HAjjj
*
**************************************
c     pp -> HAjjj
      CASE(HAjj)
         rn = RandomNumber()
         call m2s_wbfzhg( 61, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> HAjjj -> A gamma gamma jjj
      CASE(HAjj_AA)
         rn = RandomNumber()
         call m2s_wbfzhg( 61, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHGAM
         enddo
c     pp -> HAjjj -> A mu+mu- jjj
      CASE(HAjj_mu)
         rn = RandomNumber()
         call m2s_wbfzhg( 61, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHMU
         enddo
c     pp -> HAjjj -> A tau+tau- jjj
      CASE(HAjj_tau)
         rn = RandomNumber()
         call m2s_wbfzhg( 61, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHTAU
         enddo
c     pp -> HAjjj -> A b bbar jjj
      CASE(HAjj_bbar)
         rn = RandomNumber()
         call m2s_wbfzhg( 61, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHBB
         enddo
c     pp -> HAjjj -> A W W jjj
      CASE(HAjj_WW)
         rn = RandomNumber()
         call m2s_wbfzhg(61, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> HAjjj -> A Z Z jjj
      CASE(HAjj_ZZ_ll)
         rn = RandomNumber()
         call m2s_wbfzhg(61, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
      CASE(HAjj_ZZ_lnu)
         rn = RandomNumber()
         call m2s_wbfzhg(61, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
#endif


#ifdef WITH_DIBOSON
**************************************
*
*     DIBOSON : WWj, WZj, ZZj, WAj, ZAj, AAj
*
**************************************	 
c pp -> W+ W- (j) -> ve e+ e- ve~ (j)
      CASE(WW,WPhadWMlep,WPlepWMhad)
         rn = RandomNumber()
         call m2s_qqVVj( 34, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
      CASE(WPZ,WPhadZ,WPZhad)
         rn = RandomNumber()
         call m2s_qqWZj( 32, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
      CASE(WMZ,WMhadZ,WMZhad)
         rn = RandomNumber()
         call m2s_qqWZj( 42, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
c pp -> Z Z (j) -> e- e+ mu- mu+ (j)
      CASE(ZZ,ZZhad)
         rn = RandomNumber()
         call m2s_qqZZj( 22, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
      CASE(WPA)
         rn = RandomNumber()
         call m2s_qqWAj( 31, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
      CASE(WMA)
         rn = RandomNumber()
         call m2s_qqWAj( 41, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
c pp -> Z A (j) -> e- e+ A (j)
      CASE(ZA)
         rn = RandomNumber()
         call m2s_qqZAj( 21, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
c pp -> A A (j) 
      CASE(AA)
         rn = RandomNumber()
         call m2s_qqAAj( 11, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
#endif


#ifdef WITH_DIBOSONJET
**************************************
*
*     DIBOSON PLUS JET : WZjj
*
**************************************	 
c pp -> W+-A (jj) -> ve e+- a (jj)
      CASE(WMAJ)
         call m2s_qqwgagluj(xuz,1,.false.,lokt,x,p,v,m2s)
      CASE(WPAJ)
         call m2s_qqwpgagluj(xuz,1,.true.,lokt,x,p,v,m2s)
c pp -> W+-Z (jj) -> ve e+- mu+ mu- (jj)
      CASE(WMZJ, WMhadZJ, WMZhadJ)
         call m2s_qqwzgluj(xuz,1,.false.,lokt,x,p,v,m2s)
      CASE(WPZJ, WPhadZJ, WPZhadJ)
         call m2s_qqwpzgluj(xuz,1,.true.,lokt,x,p,v,m2s)       
#endif


#ifdef WITH_TRIBOSON
*****************************************************************************
*
*     TRIBOSON : WWZj, ZZZj, ZZWj, WWWj, WWAj, ZZAj, WZAj, WAAj, ZAAj, AAAj
*
*****************************************************************************
c pp -> W+ W- Z -> ve e+ e- ve~ e- e+
      CASE(WWZ, WPhadWMZ, WPWMhadZ, WWZhad)
         rn = RandomNumber()
         call m2s_qqWWZj(342,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> Z Z Z -> e- e+ e- e+ e- e+
      CASE(ZZZ, ZZZhad)
         rn = RandomNumber()
         call m2s_qqZZZj(222, 1, lokt, x, p, v, rn, xuz(1,2), m2s,
     1                                       narrowwidth, ps_number)

c pp -> Z Z W+ -> e- e+ e- e+ ve e+
      CASE(ZZWP, ZZhadWP, ZZWPhad)
         rn = RandomNumber()
         call m2s_qqZZWj(223,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> Z Z W- -> e- e+ e- e+ e- ve~
      CASE(ZZWM, ZZhadWM, ZZWMhad)
         rn = RandomNumber()
         call m2s_qqZZWj(224,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> W+ W- W+ -> ve e+ e- ve~ ve e+
      CASE(WWWP, WPhadWMWP, WPWMhadWP)
         rn = RandomNumber()
         call m2s_qqWWWj(343,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> W- W+ W- -> e- ve~ ve e+ e- ve~
      CASE(WWWM, WMhadWPWM, WMWPhadWM)
         rn = RandomNumber()
         call m2s_qqWWWj(434,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> W+ W- A -> ve e+ e- ve~ A
      CASE(WWA)
         rn = RandomNumber()
         call m2s_qqVVV1j( 341, 1, lokt, x, p, v, rn, xuz(1,2), m2s)

c pp -> Z Z A -> e- e+ e- e+ A
      CASE(ZZA)
         rn = RandomNumber()
         call m2s_qqZZAj( 221, 1, lokt, x, p, v, rn, xuz(1,2), m2s)

c pp -> W+ Z A -> e+ ve e- e+ A
      CASE(WPZA)
         rn = RandomNumber()
         call m2s_qqWZAj( 321, 1, lokt, x, p, v, rn, xuz(1,2), m2s, ps_number)

c pp -> W- Z A -> e- ve~ e- e+ A
      CASE(WMZA)
         rn = RandomNumber()
         call m2s_qqWZAj( 421, 1, lokt, x, p, v, rn, xuz(1,2), m2s, ps_number)

c pp -> W+ A A -> e+ ve A A
      CASE(WPAA)
         rn = RandomNumber()
         call m2s_qqWAAj( 311, 1, lokt, x, p, v, rn, xuz(1,2), m2s, 
     -        ps_number, narrowwidth)

c pp -> W- A A -> e- ve~ A A
      CASE(WMAA)
         rn = RandomNumber()
         call m2s_qqWAAj( 411, 1, lokt, x, p, v, rn, xuz(1,2), m2s, 
     -        ps_number, narrowwidth)

c pp -> Z A A -> e+ e- A A
      CASE(ZAA)
         rn = RandomNumber()
         call m2s_qqZAAj( 211, 1, lokt, x, p, v, rn, xuz(1,2), m2s, 
     &        ps_number, narrowwidth)

c pp -> Z A A -> ve~ ve A A
      CASE(ZnAA)
         rn = RandomNumber()
         call m2s_qqZAAj( 511, 1, lokt, x, p, v, rn, xuz(1,2), m2s, 
     &        ps_number, narrowwidth)

c pp -> A A A 
      CASE(AAA)
         rn = RandomNumber()
         call m2s_qqAAAj( 111, 1, lokt, x, p, v, rn, xuz(1,2), m2s, 
     &        ps_number)

#endif

#ifdef WITH_TRIBOSONJET
*****************************************************************************
*
*     TRIBOSON PLUS JET: WAAjj
*
*****************************************************************************
c pp -> W+ A A j -> e+ ve A A j
      CASE(WPAAj)
         rn = RandomNumber()
         call m2s_qqWAAjj( 311, 1, lokt, x, p, v, rn, xuz(1,2), ps_number
     &        ,narrowwidth, m2s)

c pp -> W- A A j -> e- ve~ A A j
      CASE(WMAAj)
         rn = RandomNumber()
         call m2s_qqWAAjj( 411, 1, lokt, x, p, v, rn, xuz(1,2), ps_number
     &        ,narrowwidth , m2s)
#endif

      CASE DEFAULT
         print *,"Error: Real emissions Amplitude ",
     &           "for process not implemented, ID = ",procID
         STOP
      END SELECT

      RETURN
      END


c*************************************************************************
      SUBROUTINE QED_Amplitude(rand, p, x, v, lokt, xuz, m2s, ps_number)
c*************************************************************************
c     Amplitude for real qed emissions.
c*************************************************************************
      implicit none

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/process.inc"

      double precision x(nx) 
      double precision rand(max_PS_dim)
      double precision p(0:3,max_p,max_kin)
      double precision v(0:3,max_v,max_kin)
      double precision rn, xuz(2,2:max_kin), m2s(0:max_kin), m2s_dec
      logical lokt(max_kin)
      integer L, ps_number

c     masses and widths
      double precision XM2s(6),XMGs(6)
      COMMON /BKOPOUshort/ XM2s,XMGs
      double precision BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM, BHGAMZ, XGW, XGZ, XGH, GAMT
      COMMON /BRANCH/ BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM, BHGAMZ, XGW, XGZ, XGH, GAMT

      double precision RandomNumber
      external RandomNumber

      SELECT CASE(process)

#ifdef WITH_VBF
**************************************
*
*     VBF : HjjA
*
**************************************
c     pp -> HjjA
      CASE(Hjj)
         rn = RandomNumber()
         call m2s_wbfHA( 6,1,lokt, x, p, v, xuz, m2s)
c     pp -> HjjA -> gamma gamma jjA
      CASE(Hjj_AA)
         rn = RandomNumber()
         call m2s_wbfHA( 6,1,lokt, x, p, v, xuz, m2s)
         do L = 0,n_qed
            m2s(L) = m2s(L)*BHGAM
         enddo
c     pp -> HjjA -> mu+mu- jjA
      CASE(Hjj_mu)
         rn = RandomNumber()
         call m2s_wbfHA( 6,1,lokt, x, p, v, xuz, m2s)
         do L = 0,n_qed
            m2s(L) = m2s(L)*BHMU
         enddo
c     pp -> HjjA -> tau+tau- jjA
      CASE(Hjj_tau)
         rn = RandomNumber()
         call m2s_wbfHA( 6,1,lokt, x, p, v, xuz, m2s)
         do L = 0,n_qed
            m2s(L) = m2s(L)*BHTAU
         enddo
c     pp -> HjjA -> b bbar jjA
      CASE(Hjj_bbar)
         rn = RandomNumber()
         call m2s_wbfHA( 6,1,lokt, x, p, v, xuz, m2s)
         do L = 0,n_qed
            m2s(L) = m2s(L)*BHBB
         enddo
c     pp -> HjjA -> W W jjA
      CASE(Hjj_WW)
         rn = RandomNumber()
         call m2s_wbfHA( 6,1,lokt, x, p, v, xuz, m2s)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_qed
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> HjjA -> Z Z jjA
      CASE(Hjj_ZZ_ll)
         rn = RandomNumber()
         call m2s_wbfHA( 6,1,lokt, x, p, v, xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_qed
            m2s(L) = m2s(L)*m2s_dec
         enddo
      CASE(Hjj_ZZ_lnu)
         rn = RandomNumber()
         call m2s_wbfHA( 6,1,lokt, x, p, v, xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_qed
            m2s(L) = m2s(L)*m2s_dec
         enddo
#endif
      CASE DEFAULT
         print *,"Error: QED emissions Amplitude ",
     &           "for process not implemented, ID = ",procID
         STOP
      END SELECT

      RETURN
      END

c with_nlo
#endif

