***************************************************************************
*
*  This is the VBFNLO main program. 
*
****************************************************************************

****************************************************************************
*  Last modified: 03.05.2011
****************************************************************************

      subroutine dovbfnlo
      implicit none

c include global variables via include-files, e.g. "procID"
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/tau_hel.inc"
#include "VBFNLO/utilities/process.inc"
#include "VBFNLO/utilities/scales.inc"

c define local variables
c--------------------------------------------------------
c     variables for the integration loops
      integer ps_number
      integer iteration, seed
      integer*8 Ncall, i, two
      parameter (two = 2)
      common/Ncall1/Ncall,iteration

c     digit characters for numbering the grid output file
      character*1 digit(61)
      data digit /'1','2','3','4','5','6','7','8','9','A',
     1            'B','C','D','E','F','G','H','I','J','K',
     1            'L','M','N','O','P','Q','R','S','T','U',
     1            'V','W','X','Y','Z','a','b','c','d','e',
     1            'f','g','h','i','j','k','l','m','n','o',
     1            'p','q','r','s','t','u','v','w','x','y',
     1            'z'/
c     the filenames for the grid output
      character*250 file_name_out, file_name_in

c     random number array controlled by monaco
      real*8 rand(max_PS_dim),rn

c     4-momenta of the partons involved in the basic QCD 2->2(+1)+X process.
c     The partons are the incoming quarks/gluons + the 2 tagging jets + 
c     the real emission (for NLO only)
      real*8 p(0:3,max_p,max_kin) 

c     Additional particles like a higgs boson, or any decay products 
c     (also hadronic ones). in short: everything which is not a parton 
c     involved in the basic QCD 2->2(+1)+X process.
      real*8 v(0:3,max_v,max_kin) 

c     Jets after jet-defining function
      real*8 jets(0:7,max_jets,max_kin) 
      integer n_jets(max_kin)
c     Charged leptons, i.e. electrons, muons, taus but not neutrinos
      real*8 leptons(0:8,max_v,max_kin)
      integer n_leptons(max_kin)
c     photons
      real*8 photons(0:7,max_v,max_kin)
      integer n_photons(max_kin)
c     invisible particles, i.e. neutrinos, LSP
      real*8 invisible(0:8,max_v,max_kin)
      integer n_invisible(max_kin)          
      
c     event weights, jacobi-factors, amplitude-squares
      real*8 weight, dps, m2s(0:max_kin), dsig(0:max_kin)

c     the result of the integration: crossection, error, chisquare
      real*8 chi2
      real*8 NLOxsection(-2:5), NLOsdev(-2:5), NLOchi2(-2:5)
c     crossection, error, chisquare for  real photon emission
      real*8 PHTNxsection, PHTNsdev, PHTNchi2
      real*8 ps_xsec(5), ps_sdev(5), sub_xsec(5), sub_sdev(5)
      common/finalPr/ ps_xsec, ps_sdev,sub_xsec, sub_sdev

c     other variables
      real*8 x(nx)   ! feynman x parameters
      logical cut_ok ! whether event passes cuts
      logical ps_ok  ! phase space generator is suitable for this ps point
      real*8 xuz(2,2:max_kin)
      integer gnlo, L, mu, nd
      logical lokt(max_kin),realcont
      common /nlovariable/ gnlo
      logical noEventOutput

c     external functions
      logical Cuts       ! general cut-function
      logical Choose_PS  ! general PS-function
      real*8 amplitude   ! general squared-amplitude function
      external Cuts, amplitude, Choose_PS

      real*8 RandomNumber
      external RandomNumber     

** Functions and variables to fully define the input gridname
      character*250 GetInputPath
      character*250 path
      external GetInputPath

c     debugging and testing variables and switches
      logical showtiming
      parameter (showtiming=.false.)

      real*8 dotrr
      external dotrr
      logical Finalprinting
      parameter(Finalprinting=.true.)

      write(*,*)'  '
      write(*,*)'-----------------------------------------------------'
      write(*,*)'VBFNLO, version 2.7.0 beta 3'
      write(*,*)'arXiV: 1107.4038, arXiV: 1207.4975'
      write(*,*)'http://www-itp.particle.uni-karlsruhe.de/~vbfnloweb'
      write(*,*)'-----------------------------------------------------'
      write(*,*)'  '
      write(*,*)'  '


** LoopTools initialisation
#ifdef WITH_LT
      call initLT      
#endif

      if (showtiming) call mtime

c     first, do all the initialization
    
      call InitGlobalParameters

      call PrintProcInfo

      call InitPDFs(1)
      
      call InitCouplings

*      call InitScales

      call InitProcess

      call InitCuts
      
      call InitPhaseSpace
    
      call InitRandomNumbers(seed)

      call InitHistograms


c make grid directory
      IF((Loops_sub_LO.gt.1).or.(Loops_sub_NLO.gt.1)) then
      CALL system("mkdir -p GRID")
      endif


** Setting input path
      path = GetInputPath()

c...Les Houches interface
      noEventOutput = .false.
      if ((lha.or.hepmc).and..not.doNLO) then 
         SELECT CASE (process)
         CASE(WMAJ, WPAJ, WPZJ, WMZJ, WPAAJ, WMAAJ,
     -        WMAJLO, WPAJLO, WPZJLO, WMZJLO )
            lha=.false.
            hepmc=.false.
            noEventOutput = .true.
            print*,''
            print*,'Sorry, event file is not available for the process',
     -           procID
            print*,''
         CASE DEFAULT
            call lha_file(-1)
         END SELECT
      endif
      
      boxcountm  = 0
      boxcount2m = 0
      box2countm  = 0
      box2count2m = 0
      pentcountm  = 0
      pentcount2m = 0
      hexcountm  = 0
      hexcount2m = 0
c==========================
c    The LO Calculation
c==========================
    
      do i=1,5
         ps_xsec(i)=0d0
         ps_sdev(i)=0
      enddo
      PHTNxsection=0d0
      PHTNsdev=0d0


      if (doBorn) then

*         call InitPDFs(0)
         gnlo = 0

         do ps_number = 1,PS_Loops !multi-channel
            if (PS_Loops.gt.1) then
               write(*,*)" "
               write(*,*)"PHASE SPACE", ps_number
               write(*,*)" "
            endif
            sub_xsec(1)=0d0
            sub_sdev(1)=0d0


            do sub_number = 1, Loops_sub_LO ! multi-processes


               if (Loops_sub_LO.gt.1) then
                  write(*,*)" "
                  write(*,*)"Subprocess", sub_number
                  write(*,*)" "
               endif

               if (Loops_sub_LO.gt.1) then
                  file_name_in  = path(1:len_trim(path))//"/"//
     &                 gridname1(ps_number)
     &                 (1:INDEX(gridname1(ps_number),'  ')-1)
                  file_name_in = file_name_in(1:len_trim(file_name_in))
     &                 //'_'//digit(sub_number)
               else
                  file_name_in  = path(1:len_trim(path))//"/"//
     &                 gridname1(ps_number)
     &                 (1:INDEX(gridname1(ps_number),'  ')-1)
                  file_name_in = file_name_in(1:len_trim(file_name_in))
               end if

               xsection=0d0
               sdev=0d0

               do iteration = 1, N_iterations1

cFC                  print*,"Proc", procID
cFC                  print*, "ps_number",ps_number
cFC                  PRINT*, "Loops_sub_LO",Loops_sub_LO
cFC                  PRINT*, "n_CALL", NCALL
cFC                  print*, "ps_loops",ps_loops
                  
c Adjust statistics for different phase space 
                  call statistics_proc(ps_number)
         
cFC                PRINT*, "n_CALL", NCALL

                  if (Loops_sub_LO.gt.1) then
                     file_name_out = 'GRID/'//gridname1(ps_number)
     &                    (1:INDEX(gridname1(ps_number),'  ')-1)//
     &                    '_'//digit(sub_number)//'.out.'//digit(iteration)
                  else
                     file_name_out = gridname1(ps_number)
     &                    (1:INDEX(gridname1(ps_number),'  ')-1)//
     &                    '.out.'//digit(iteration)
                  end if

                  if ( iteration .eq. 1 ) then
                     call monaco_init( PS_dimension, Ncall, seed )
                     call monaco_read(file_name_in)
                  else
                     call monaco_init2( PS_dimension, Ncall )
                  end if       
                  if (iteration.eq.N_iterations1) then
                     call monran_set(1) ! save random number info
                  endif
                  call ResetWeights(ps_number)

                  do i = 1, Ncall
c     retrieve a set of random numbers and the corresponding weight
                     call monaco_get( rand, weight)

c     transform the random numbers to a physical phase space point
                     call phasespace(rand, p, x, v, ps_number, dps  )

                     if ( dps .gt. 0d0 ) then

                        if (alllep_flag) call lepton_gen

c     define jets, leptons, photons
                        call defineJets(p, v, 1, jets, n_jets)
                        call DefineLeptons(v, 1, leptons, n_leptons)
                        call DefineInvisible(v,1, invisible, n_invisible)
                        call DefinePhotons(v, 1, photons, n_photons)

c in case of multi-channel: choose phase space:
                        PS_OK = Choose_PS(ps_number,v, 1)

c     apply cuts
                        cut_ok = Cuts(jets, n_jets, leptons, n_leptons, 
     1                       invisible,n_invisible,photons, n_photons, 1)

                        if (cut_ok .and. PS_ok) then
c     calculate fac. and ren. scale for this phase space point
                           call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, 1, 0)
c     call the matrix element code and get an amplitude square
                           m2s(0) = Amplitude(rand, p, x, v, 0, ps_number)
                        else
                           dps = 0d0
                           m2s(0) = 0d0
                        end if 
                        dsig(0) = m2s(0)*dps

                        if (cut_ok .and. PS_ok) then
                           eventweight=dsig(0)*weight

c     in the last iteration : 
                           if (iteration.eq.N_iterations1) then

c     pass event information to the histogram routine
                              call HistogramEvent(weight,dsig(0),dps,
     &                             rand, p(0,1,1), x, v(0,1,1), 
     &                             jets(0,1,1), n_jets(1), 
     &                             leptons(0,1,1), n_leptons(1), 
     &                             invisible(0,1,1),n_invisible(1),
     &                             photons(0,1,1), n_photons(1), 0)

c     the unweighting of events : 
                       
c                              if (unweighting) then 
                                 call Reweight(eventweight,i,Ncall,ps_number) ! reweight
c                              endif

                              if (eventweight.gt.0d0) then ! event is to be kept
c Les Houches interface ----------------
                                 if((lha.or.hepmc).and..not.doNLO) call lha_put(p,v)

                              endif
                           endif
                           
                        endif   ! cutok

                     else       ! dps > 0d0
                        m2s(0) = 0d0
                        dsig(0) = 0d0
                     end if

c     pass weight to monaco
                     call monaco_put(rand, weight, dsig(0))

                  end do
        
                  call monaco_write( file_name_out )

               enddo

               call monaco_result(xsection, sdev, chi2)


               lo_r(ps_number)=xsection
               e_lo_r(ps_number)=sdev     

     
c     print out the result. 
               print *,""
               if(Loops_sub_LO.eq.1) then
                  print *," result (LO): ",xsection," +- ",sdev," fb  "
     &                 ,sdev/xsection*100d0,'%'
               else
                  print *," result sub (LO): ",xsection," +- ",sdev,
     &                 " fb  ",sdev/xsection*100d0,'%'
               end if
               print *,""
                  
               ps_xsec(1)=ps_xsec(1)+xsection
               ps_sdev(1)=ps_sdev(1)+sdev**2
               sub_xsec(1)=sub_xsec(1)+xsection
               sub_sdev(1)=sub_sdev(1)+sdev**2
            end do ! end sub-processes

            if(Loops_sub_LO.gt.1) then
               print *,""
               print *," result (LO): ",sub_xsec(1)," +- "
     &              ,sqrt(sub_sdev(1))," fb  ",sqrt(ps_sdev(1))/
     &              ps_xsec(1)*100d0,'%'
               print *," "
               lo_r(ps_number)=sub_xsec(1)
               e_lo_r(ps_number)=sqrt(sub_sdev(1))
            endif  

            if(.not.doNLO) then
               if (PS_Loops.gt.1) then
                  print*,'number of events in the last iteration in this phasespace =',ncall
               else
                  print*,'number of events in the last iteration =',ncall
               endif
               call reweightinginfo(.false.)
            endif

         enddo                  !multi-channel
         

         if (ps_loops.gt.1) then
            print *,""
            print *," Total (multi-channel) result (LO): ",
     &        ps_xsec(1)," +- ",sqrt(ps_sdev(1))," fb  ",
     &           sqrt(ps_sdev(1))/ps_xsec(1)*100d0,'%'
            print *," "
         endif

      else
         print *," No LO calculation performed."
      endif

      if (ps_xsec(1) .ne. ps_xsec(1)) then
         write(*,*)'Sorry, something has gone wrong with the LO'
         write(*,*)'cross section!'
         stop
      end if

c...Les Houches interface
      if((lha.or.hepmc).and..not.doNLO) then
         xsection = ps_xsec(1)
         sdev = sqrt(ps_sdev(1))
         call lha_file(0)
         call lha_file(1)
      endif

      if(.not.doNLO) call reweightinginfo(.true.)



#ifdef WITH_NLO
c============================
c    The NLO Calculation
c============================     

** FF initialisation
      call initFF

c----------------------------
c  The fermion loops 
c----------------------------
      if ((doFloops).and.hasFLoops.and.(floops.ne.0)) then
cc This bit is here because we need the LO definitions for that
cc and we save reinitializing things again

c should be calculate with LO PDFs
         if (.not.doBorn) call InitPDFs(0)
c and different PS
         infloops = .true.
         call InitPhaseSpace

         call InitFLoop
         gnlo = 5

         do ps_number = 1, PS_Loops !multi-channel
            if (PS_Loops.gt.1) then
               write(*,*)" "
               write(*,*)"PHASE SPACE", ps_number
               write(*,*)" "
            endif
            sub_xsec(5)=0d0
            sub_sdev(5)=0d0


            do sub_number = 1, Loops_sub_LO ! multi-processes


               if (Loops_sub_LO.gt.1) then
                  write(*,*)" "
                  write(*,*)"Subprocess", sub_number
                  write(*,*)" "
               endif

               if (Loops_sub_LO.gt.1) then
                  file_name_in  = path(1:len_trim(path))//"/"//
     &                 gridname4(ps_number)
     &                 (1:INDEX(gridname4(ps_number),'  ')-1)
                  file_name_in = file_name_in(1:len_trim(file_name_in))
     &                 //'_'//digit(sub_number)
               else
                  file_name_in  = path(1:len_trim(path))//"/"//
     &                 gridname4(ps_number)
     &                 (1:INDEX(gridname4(ps_number),'  ')-1)
                  file_name_in = file_name_in(1:len_trim(file_name_in))
               end if

               xsection=0d0
               sdev=0d0

               do iteration = 1, N_iterations1

c Adjust statistics for different phase space 
                  call statistics_proc_FLoops(ps_number)
         
                  if (Loops_sub_LO.gt.1) then
                     file_name_out = 'GRID/'//gridname4(ps_number)
     &                    (1:INDEX(gridname4(ps_number),'  ')-1)//
     &                    '_'//digit(sub_number)//'.out.'//digit(iteration)
                  else
                     file_name_out = gridname4(ps_number)
     &                    (1:INDEX(gridname4(ps_number),'  ')-1)//
     &                    '.out.'//digit(iteration)
                  end if

                  if ( iteration .eq. 1 ) then
                     call monaco_init( PS_dimension, Ncall, seed )
                     call monaco_read(file_name_in)
                  else
                     call monaco_init2( PS_dimension, Ncall )
                  end if       
                  if (iteration.eq.N_iterations1) then
                     call monran_set(1) ! save random number info
                  endif

                  do i = 1, Ncall
c     retrieve a set of random numbers and the corresponding weight
                     call monaco_get( rand, weight)

c     transform the random numbers to a physical phase space point
                     call phasespace(rand, p, x, v, ps_number, dps  )

                     if ( dps .gt. 0d0 ) then

                        if (alllep_flag) call lepton_gen

c     define jets, leptons, photons
                        call defineJets(p, v, 1, jets, n_jets)
                        call DefineLeptons(v, 1, leptons, n_leptons)
                        call DefineInvisible(v,1, invisible, n_invisible)
                        call DefinePhotons(v, 1, photons, n_photons)

c in case of multi-channel: choose phase space:
                        PS_OK = Choose_PS(ps_number,v, 1)
c     apply cuts
                        cut_ok = Cuts(jets, n_jets, leptons, n_leptons, 
     1                       invisible,n_invisible,photons, n_photons, 1)

                        if (cut_ok .and. PS_ok) then
c     calculate fac. and ren. scale for this phase space point (LO scale)
                           call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, 1, 0)
c     call the matrix element code and get an amplitude square
                           m2s(0) = Amplitude(rand, p, x, v, gnlo, ps_number)

                        else
                           dps = 0d0
                           m2s(0) = 0d0
                        end if 
                        dsig(0) = m2s(0)*dps

                        if (cut_ok .and. PS_ok) then
c     in the last iteration : 
                           if (iteration.eq.N_iterations1) then

c     pass event information to the histogram routine
                              call HistogramEvent(weight,dsig(0),dps,
     &                             rand, p(0,1,1), x, v(0,1,1), 
     &                             jets(0,1,1), n_jets(1), 
     &                             leptons(0,1,1), n_leptons(1), 
     &                             invisible(0,1,1),n_invisible(1),
     &                             photons(0,1,1), n_photons(1), -1)
                           endif
                           
                        endif   ! cutok

                     else       ! dps > 0d0
                        m2s(0) = 0d0
                        dsig(0) = 0d0
                     end if

c     pass weight to monaco
                     call monaco_put(rand, weight, dsig(0))

                  end do
        
                  call monaco_write( file_name_out )

               enddo

               call monaco_result(xsection, sdev, chi2)


               floop_r(ps_number)=xsection
               e_floop_r(ps_number)=sdev     

     
c     print out the result. 
               print *,""
               if(Loops_sub_LO.eq.1) then
                  print *," result (NLO fermion loop): ",xsection,
     &                 " +- ",sdev," fb  ",sdev/xsection*100d0,'%'
               else
                  print *," result sub (NLO fermion loop): ",xsection,
     &                 " +- ",sdev," fb  ",sdev/xsection*100d0,'%'
               end if
               print *,""
                  
               ps_xsec(5)=ps_xsec(5)+xsection
               ps_sdev(5)=ps_sdev(5)+sdev**2
               sub_xsec(5)=sub_xsec(5)+xsection
               sub_sdev(5)=sub_sdev(5)+sdev**2
            end do ! end sub-processes

            if(Loops_sub_LO.gt.1) then
               print *,""
               print *," result (NLO fermion loop): ",sub_xsec(5)," +- "
     &              ,sqrt(sub_sdev(5))," fb  ",sqrt(ps_sdev(5))/
     &              ps_xsec(5)*100d0,'%'
               print *," "
               floop_r(ps_number)=sub_xsec(5)
               e_floop_r(ps_number)=sqrt(sub_sdev(5))
            endif  
         enddo                  !multi-channel
         

         if (ps_loops.gt.1) then
            print *,""
            print *," Total (multi-channel) result (NLO fermion loop): ",
     &        ps_xsec(5)," +- ",sqrt(ps_sdev(5))," fb  ",
     &           sqrt(ps_sdev(5))/ps_xsec(5)*100d0,'%'
            print *," "
         endif

         infloops = .false.
         call InitPhaseSpace

      else
         print *," No NLO fermion loop calculation performed."
      endif

      if (ps_xsec(5) .ne. ps_xsec(5)) then
         write(*,*)'Sorry, something has gone wrong with the NLO'
         write(*,*)'fermion loop cross section!'
         stop
      end if


c============================
c    The NLO Calculation
c============================     
c----------------------------
c  The virtual corrections      
c----------------------------
      if ((doVirtuals.and.hasNLO) .or. ewcor_switch) then

c Only one iteration.Think whether we want to put more. Add a bit more sta-
c tistics to the virtual part instead of incresing by 2^n

       iteration=1
       N_iterationsv=1

         call InitPDFs(1)

         do ps_number = 1,PS_Loops !multi-channel

            if (PS_Loops.gt.1) then
               print*," "
               print*,"PHASE SPACE ", ps_number
               print*," "
            endif
c Initialize to ZERO the instabilities and other variables

            call Init_Instabilities(ps_number)

            sub_xsec(2)=0d0
            sub_sdev(2)=0d0

            do sub_number = 1, Loops_sub_LO ! multi-processes
   
               NLOxsection(0) = 0d0
               NLOsdev(0) = 0d0


               if (Loops_sub_LO.gt.1) then
                  write(*,*)" "
                  write(*,*)"Subprocess", sub_number
                  write(*,*)" "
               endif

               if (Loops_sub_LO.gt.1) then
                  if (N_iterations1.eq.1) then
                     file_name_in  = path(1:len_trim(path))//"/"//
     &                    gridname1(ps_number)
     &                    (1:INDEX(gridname1(ps_number),'  ')-1)
                     file_name_in = file_name_in(1:len_trim(file_name_in))
     &                    //'_'//digit(sub_number)
                  else
                     file_name_in  = 'GRID/'//gridname1(ps_number)
     &                    (1:INDEX(gridname1(ps_number),'  ')-1)//
     &                    '_'//digit(sub_number)//
     &                    '.out.'//digit(N_iterations1)
                  endif
               else
                  if (N_iterations1.eq.1) then
                     file_name_in  = path(1:len_trim(path))//"/"//
     &                    gridname1(ps_number)
     &                    (1:INDEX(gridname1(ps_number),'  ')-1)
                     file_name_in = file_name_in(1:len_trim(file_name_in))
                  else
                     file_name_in  = gridname1(ps_number)
     &                    (1:INDEX(gridname1(ps_number),'  ')-1)//
     &                    '.out.'//digit(N_iterations1)
                  endif
               end if

               do gnlo = 1, NLO_Loops ! loop over different virtual contns
               
c Adjust statistics for different phase space regions
                  call statistics_proc_NLO(ps_number)
    
                  if (Loops_sub_LO.gt.1) then
                     file_name_out = 'GRID/'//gridname1(ps_number)
     &                    (1:INDEX(gridname1(ps_number),'  ')-1)//
     &                    '_'//digit(sub_number)//
     &                    '.vout.'//digit(1)
                  else
                     file_name_out = gridname1(ps_number)
     &                    (1:INDEX(gridname1(ps_number),'  ')-1)//
     &                    '.vout.'//digit(1)
                  end if
               
                  call monaco_init( PS_dimension, Ncall, seed )
                  call monaco_read(file_name_in)
!                  call monran_set(2)  
         
                  do i = 1, Ncall
                     call monaco_get( rand, weight)
                  
                     call phasespace(rand, p, x, v, ps_number, dps  )
                  
                     if ( dps .gt. 0d0 ) then 

                        if (alllep_flag) call lepton_gen        
                        
                        call defineJets(p, v, 1, jets, n_jets)
                        call DefineLeptons(v, 1, leptons, n_leptons)
                        call DefinePhotons(v, 1, photons, n_photons)
                        call DefineInvisible(v,1, invisible, n_invisible)
                        
                        PS_OK = Choose_PS(PS_number, v, 1)
                        
                        cut_ok = Cuts(jets, n_jets, leptons, n_leptons, 
     1                       invisible,n_invisible,photons, n_photons, 1)
                        
                        if (cut_ok.and.ps_ok) then
                           call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, 1, 1)
                           m2s(0) = Amplitude(rand, p, x, v, gnlo, ps_number)
#ifdef WITH_LT  
                           call clearLT
#endif
                        else
                           dps = 0d0
                           m2s(0) = 0d0
                        end if 
                     
                        dsig(0) = m2s(0)*dps
                        
                        call HistogramEvent(weight,dsig(0),dps,
     &                       rand, p(0,1,1), x, v(0,1,1), 
     &                       jets(0,1,1), n_jets(1), 
     &                       leptons(0,1,1), n_leptons(1), 
     &                       invisible(0,1,1),n_invisible(1),     
     &                       photons(0,1,1), n_photons(1), -1)

                     else
                        m2s(0) = 0d0
                        dsig(0) = 0d0
                     end if

                     call monaco_put(rand, weight, dsig(0))
                  end do  ! Ncalls

                  call monaco_write(file_name_out)
     
                  call monaco_result(NLOxsection(gnlo), 
     &                 NLOsdev(gnlo), NLOchi2(gnlo))

                  if (gnlo.eq.1) then
                     virt_b(ps_number)=virt_b(ps_number)+NLOxsection(gnlo)
                     e_virt_b(ps_number)=e_virt_b(ps_number)+NLOsdev(gnlo)**2
                  endif
                  if (gnlo.eq.2) then
                     box_v(ps_number)=box_v(ps_number)+NLOxsection(gnlo)
                     e_box_v(ps_number)=e_box_v(ps_number)+NLOsdev(gnlo)**2
                  endif
                  if (gnlo.eq.3) then
                     pent_v(ps_number)=pent_v(ps_number)+NLOxsection(gnlo)
                     e_pent_v(ps_number)=e_pent_v(ps_number)+NLOsdev(gnlo)**2
                  endif
                  if (gnlo.eq.4) then
                     hex_v(ps_number)=hex_v(ps_number)+NLOxsection(gnlo)
                     e_hex_v(ps_number)=e_hex_v(ps_number)+NLOsdev(gnlo)**2
                  endif
                  if (gnlo.eq.5) then
                     box2_v(ps_number)=box2_v(ps_number)+NLOxsection(gnlo)
                     e_box2_v(ps_number)=e_box2_v(ps_number)+NLOsdev(gnlo)**2
                  endif


                  print *," "
                  print *," result for virtual contribution, no. ",gnlo
                  print *," sigma = ",NLOxsection(gnlo)," +- ",
     &                 NLOsdev(gnlo)," fb  "


C printing stabilities
                  CALL  Instabilities(ps_number)
                  

                  print *," "
                  NLOxsection(0) = NLOxsection(0) + NLOxsection(gnlo)
                  NLOsdev(0) = sqrt(NLOsdev(0)**2 + NLOsdev(gnlo)**2)

                  print *,'total result for virtual contns so far '

                  print *,' sigma = ',NLOxsection(0),' +- ',NLOsdev(0),
     -                 ' fb'
                  print *,' '

               enddo   ! different virtual contributions

               ps_xsec(2)=ps_xsec(2)+NLOxsection(0)
               ps_sdev(2)=ps_sdev(2)+NLOsdev(0)**2

               sub_xsec(2)=sub_xsec(2)+NLOxsection(0)
               sub_sdev(2)=sub_sdev(2)+NLOsdev(0)**2
            enddo               ! end subprocesses
  
            e_virt_b(ps_number)=sqrt(e_virt_b(ps_number))
            e_box_v(ps_number)=sqrt(e_box_v(ps_number))
            e_pent_v(ps_number)=sqrt(e_pent_v(ps_number))
            e_hex_v(ps_number)=sqrt(e_hex_v(ps_number))
            e_box2_v(ps_number)=sqrt(e_box2_v(ps_number))

            if(Loops_sub_LO.gt.1)then
               print *,' total result for virtual PS ', ps_number
               print *,' sigma = ',sub_xsec(2),' +- ',sqrt(sub_sdev(2)),
     &              ' fb'
               print *,' '
            endif  

         enddo                  !multi-channel

         if (ps_loops.gt.1) then
            print *,""
            print *," Total (multi-channel) result (NLO virtual): "
     &           ,ps_xsec(2)," +- ",sqrt(ps_sdev(2))," fb  "
     &           ,sqrt(ps_sdev(2))/ps_xsec(2)*100d0,'%'
            print *,""
         endif

      endif


      if (ps_xsec(2) .ne. ps_xsec(2)) then
         write(*,*)'Sorry, something has gone wrong with the NLO'
         write(*,*)'virtual result!'
         stop
      end if

** closing FF
      call closeFF

c----------------------------
c    The real emissions      
c----------------------------
      if (doEmissions.and.hasNLO) then

         if (.not. doVirtuals) call InitPDFs(1)
         call InitRealEmissions

         do ps_number = 1,PS_Loops

            if (ps_loops.gt.1) then
               print*," "
               print*,"PHASE SPACE ", ps_number
               print*," "
            endif


            sub_xsec(3)=0d0
            sub_sdev(3)=0d0

            do sub_number = 1, Loops_sub_NLO ! multi-processes
               
               if (Loops_sub_NLO.gt.1) then
                  write(*,*)" "
                  write(*,*)"Subprocess", sub_number
                  write(*,*)" "
               endif

               file_name_in  = path(1:len_trim(path))//"/"//
     &              gridname2(ps_number)
     &              (1:INDEX(gridname2(ps_number),'  ')-1)
               if (Loops_sub_NLO.gt.1) then
                  file_name_in = file_name_in(1:len_trim(file_name_in))
     &                 //'_'//digit(sub_number)
               else
                  file_name_in = file_name_in(1:len_trim(file_name_in))
               end if

               NLOxsection(-1) = 0d0
               NLOsdev(-1) = 0d0


               ! for histogram error estimation
               call InitRealHist(n_kin)


               DO iteration = 1, N_iterations2

c Control the statistics for different phase space
                  call statistics_proc_Real(ps_number)


                  if (Loops_sub_NLO.gt.1) then
                     file_name_out = 'GRID/'//gridname2(ps_number)
     &                    (1:INDEX(gridname2(ps_number),'  ')-1)//
     &                    '_'//digit(sub_number)//
     &                    '.out.'//digit(iteration)
                  else
                     file_name_out = gridname2(ps_number)
     &                    (1:INDEX(gridname2(ps_number),'  ')-1)//
     &                    '.out.'//digit(iteration)
                  end if

                  if ( iteration .eq. 1 ) then
                     call monaco_init( PS_dimension, Ncall, seed )
                     call monaco_read(file_name_in)
                  else
                     call monaco_init2( PS_dimension, Ncall )
                  end if      
           
                  do i = 1, Ncall
                     call monaco_get( rand, weight)

                     call phasespace(rand, p, x, v, ps_number, dps  )

                     if ( dps .gt. 0d0 ) then 

                        if (alllep_flag) call lepton_gen
     
                        call ptilde(p(0,1,1),xuz(1,2),v(0,1,1))
                        call defineJets(p, v, n_kin, jets, n_jets)
                        call DefineLeptons(v, n_kin, leptons, n_leptons)
                        call DefinePhotons(v, n_kin, photons, n_photons)
                        call DefineInvisible(v,n_kin, invisible, n_invisible)
                        
                        ps_ok = Choose_PS(PS_number, v, 1)

                     
                        lokt(1) = Cuts(jets, n_jets, leptons, n_leptons,
     -                       invisible,n_invisible,photons,n_photons, 1)
     -                       .and.ps_ok
                        
                        cut_ok = lokt(1)

                        do nd = 2, n_kin
                           ps_ok = Choose_PS(PS_number, v, nd)
                           lokt(nd) = Cuts(jets, n_jets, leptons, n_leptons,
     1                          invisible,n_invisible,
     1                          photons, n_photons, nd).and.ps_ok
                           cut_ok = cut_ok .or. lokt(nd)
                        enddo

                        if (cut_ok) then
                           call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, n_kin, 1)

                           call RE_Amplitude(rand, 
     -                          p, x, v, lokt, xuz, m2s,ps_number)
                        else
                           dps = 0d0
                           do L = 0,n_kin
                              m2s(L) = 0d0
                           enddo                     
                        end if 
                      
                        do L = 0,n_kin
                           dsig(L) = m2s(L)*dps
                        enddo   

                        if (iteration.eq.N_iterations2) then

                           realcont=.false.
                           do L = 1,n_kin
                              if (dsig(L).ne.0d0) then
                                 call HistogramEvent(weight,dsig(L),dps,
     &                                rand, p(0,1,L), x, v(0,1,L), 
     &                                jets(0,1,L), n_jets(L), 
     &                                leptons(0,1,L), n_leptons(L),
     &                                invisible(0,1,L),n_invisible(L),        
     &                                photons(0,1,L), n_photons(L), L)
                                 realcont=.true.
                              endif
                           enddo
                           if (realcont) call SaveRealHist

                        endif

                     else
                        do L = 0,n_kin
                           m2s(L) = 0d0
                           dsig(L) = 0d0
                        enddo
                     end if   

                     call monaco_put(rand, weight, dsig(0))
               
                  end do
        
                  call monaco_write( file_name_out )

               END DO

               call monaco_result(NLOxsection(-1), NLOsdev(-1), NLOchi2(-1))
     
c     print out the result. 
               if(Loops_sub_NLO.eq.1) then
                  print *,""
                  print *," result for real emissions "
                  print *," sigma = ",NLOxsection(-1)," +- ",NLOsdev(-1)," fb"
                  print *," "
               else
                  print *,""
                  print *," result subprocess for real emissions "
                  print *," sigma = ",NLOxsection(-1)," +- ",NLOsdev(-1)," fb"
                  print *," "
               endif

               ps_xsec(3)=ps_xsec(3)+NLOxsection(-1)
               ps_sdev(3)=ps_sdev(3)+NLOsdev(-1)**2
               sub_xsec(3)=sub_xsec(3)+NLOxsection(-1)
               sub_sdev(3)=sub_sdev(3)+NLOsdev(-1)**2

               Re_nlo(ps_number)=sub_xsec(3)
               e_Re_nlo(ps_number)=sqrt(sub_sdev(3))

            enddo               ! endsuprocess

            
            if(Loops_sub_NLO.gt.1) then
               print *," result for real emissions "
               print *," sigma = ",sub_xsec(3)," +- ",sqrt(sub_sdev(3))
     &              ," fb"
               print *," "
            endif

         enddo                  !multi-channel

         if (ps_loops.gt.1) then
            print *,""
            print *," Total (multi-channel) result (NLO real): "
     &           ,ps_xsec(3)," +- ",sqrt(ps_sdev(3))," fb  "
     &           ,sqrt(ps_sdev(3))/ps_xsec(3)*100d0,'%'
            print *,""
         endif
         
      endif


*sophy: calculating the real photon emission.  Adapted from code written by
*       Terrance Figy
c----------------------------
c    The real photon emissions      
c----------------------------
      if (ewcor_switch .and. (sector .ge. 3)) then

         print *,""
         print *,"Starting real photon emissions"

*         if (.not. ewcor_switch) call InitPDFs(1)

         call InitQEDEmissions

         do ps_number = 1,PS_Loops

            if (ps_loops.gt.1) then
               print*," "
               print*,"PHASE SPACE ", ps_number
               print*," "
            endif

            file_name_in  = path(1:len_trim(path))//"/"//
     &           gridname3(ps_number)
     &           (1:INDEX(gridname3(ps_number),'  ')-1)
            file_name_in = file_name_in(1:len_trim(file_name_in))

            PHTNxsection=0d0
            PHTNsdev=0d0

            ! for histogram error estimation
            call InitRealHist(n_qed)

            DO iteration = 1, N_iterations2
               Ncall = 2**( N_points(-1) - N_iterations2 + iteration )
              
               file_name_out = gridname3(ps_number)
     &               (1:INDEX(gridname3(ps_number),'  ')-1)//
     &               '.out.'//digit(iteration)

               if ( iteration .eq. 1 ) then
                  call monaco_init( PS_dimension, Ncall, seed )
                  call monaco_read(file_name_in)
               else
                  call monaco_init2( PS_dimension, Ncall )
               end if       
           
               do i = 1, Ncall
                  call monaco_get( rand, weight)

                  call phasespace(rand, p, x, v, ps_number, dps  )

                  if ( dps .gt. 0d0 ) then    

                     if (alllep_flag) call lepton_gen
  
                     call ptilde_qed(p(0,1,1),xuz(1,2),v(0,1,1)) 
                     call defineJets(p, v, n_qed, jets, n_jets)
                     call DefineLeptons(v, n_qed, leptons, n_leptons)
                     call DefinePhotons(v, n_qed, photons, n_photons)
                     call DefineInvisible(v,n_qed, invisible, n_invisible)
             
                     ps_ok = Choose_PS(PS_number, v, 1)
                     
                     lokt(1) = Cuts(jets, n_jets, leptons, n_leptons,
     1                                            invisible,n_invisible,
     1                                            photons,n_photons, 1)
     1                         .and.ps_ok
                     cut_ok = lokt(1)
                     do nd = 2, n_qed
                        ps_ok = Choose_PS(PS_number, v, nd)
                        lokt(nd) = Cuts(jets, n_jets, leptons, n_leptons,
     1                                              invisible,n_invisible,
     1                                              photons, n_photons, nd)
     1                         .and.ps_ok
                        cut_ok = cut_ok .or. lokt(nd)
                     enddo
                     if (cut_ok) then
                        call Scales(p, v, jets, n_jets, leptons, 
     1                             n_leptons, photons, n_photons, n_kin, 1)
                        call QED_Amplitude(rand, p, x, v, lokt, xuz, m2s,
     1                                                          ps_number)
                     else
                        dps = 0d0
                        do L = 0,n_qed
                           m2s(L) = 0d0
                        enddo                     
                     end if 

                     do L = 0,n_qed
                        dsig(L) = m2s(L)*dps
                     enddo   

                     if (iteration.eq.N_iterations2) then

                           realcont=.false.
                           do L = 1,n_qed
                              if (dsig(L).ne.0d0) then
                                 call HistogramEvent(weight,dsig(L),dps,
     &                                rand, p(0,1,L), x, v(0,1,L), 
     &                                jets(0,1,L), n_jets(L), 
     &                                leptons(0,1,L), n_leptons(L),
     &                                invisible(0,1,L),n_invisible(L),        
     &                                photons(0,1,L), n_photons(L), L)
                                 realcont=.true.
                              endif
                           enddo
                           if (realcont) call SaveRealHist

                     endif

                  else
                     do L = 0,n_qed
                        m2s(L) = 0d0
                        dsig(L) = 0d0
                     enddo
                  end if
                  
                  call monaco_put(rand, weight, dsig(0))
                  
               end do
            
               call monaco_write( file_name_out )

            END DO

            call monaco_result(PHTNxsection, PHTNsdev, PHTNchi2)
     
c     print out the result. 
            print *,""
            print *," result for QED real emissions "
            print *," sigma = ",PHTNxsection," +- ",PHTNsdev," fb"
            print *," "

            ps_xsec(4)=ps_xsec(4)+PHTNxsection
            ps_sdev(4)=ps_sdev(4)+PHTNsdev**2
         enddo                  !multi-channel

         if (ps_loops.gt.1) then
            print *,""
            print *," Total (multi-channel) result (QED real): "
     &          ,ps_xsec(4)," +- ",sqrt(ps_sdev(4))," fb  "
     &          ,sqrt(ps_sdev(4))/ps_xsec(4)*100d0,'%'
            print *,""
         endif

      end if
      

** closing LoopTools
#ifdef WITH_LT
      call closeLT
#endif

         if ((dovirtuals.and.doEmissions.and.hasNLO).or.ewcor_switch) then
            NLOxsection(-2) = ps_xsec(2)+ps_xsec(3)+ps_xsec(4)+ps_xsec(5)
            NLOsdev(-2) = sqrt(ps_sdev(2)+ps_sdev(3)+ps_sdev(4)+ps_sdev(5))
            print *," "
            print *," final result at NLO "         
            print *," sigma = ",NLOxsection(-2)," +- ",NLOsdev(-2)," fb"
     &      ,NLOsdev(-2)/NLOxsection(-2)*100d0,'%'
         endif

#endif

      call Final_Instabilities
      ! if(FinalPrinting .and. (NLO_Loops .gt. 1) .and. dovirtuals) 
      if(FinalPrinting) 
     &     call Final_Printing

c...Les Houches / HepMC interface
      if (lha.and..not.doNLO) then 
         print*,''
         print*,'created LHA event file for the LO calculation : ',
     &        lhafile
      endif
      if (hepmc.and..not.doNLO) then 
         print*,''
         print*,'created HepMC event file for the LO calculation : ',
     &        hepmcfile
      endif
      if (noEventOutput) then 
         print*,''
         print*,'Sorry, event output is not available for this process! '
      endif
      if ((lha.or.hepmc).and..not.doNLO) then
         if (unweighting) then
           print*,'number of unweighted events written to file : ', nevacc
         else
           print*,'number of weighted events written to file : ', nevall
         endif
         print*,''
      endif
      if (ps_loops.gt.1 .and. .not.doNLO .and. (hepmc .or. lha)) then
           print*, ""
           print*,'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!' //
     &      '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
           print*, ""
           print*, "Important note:"
           print*, "The events of different phase spaces are written block-wise into the event file."
           print*, "Therefore the event file should always be used completely, otherwise some parts of phase"
           print*, "space are underrepresentated. Using only parts of the event file gives only correct results"
           print*, "if the events are taken randomly from the whole file."
           print*, ""
           print*,'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!' //
     &      '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
           print*, ""
      endif

      call WriteHistograms

* event output is commented out, because it's buggy and not standardized
*      call CloseEventOutput
     
c print out results in a file also
      open (20,
     &      file=xsecfile,
     &      status='REPLACE')
      write(20,*) ps_xsec(1), sqrt(ps_sdev(1))
      write(20,*) NLOxsection(-2), NLOsdev(-2)
      close(20)

c.. Printing summary of results

      write(*,*)' '
      write(*,*)' '
      print*,'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!' //
     &     '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'

      write(*,*)'    VBFNLO 2.7.0 -- BETA 3        '

      call printFinalProc

      if (doBorn) then
         print *," TOTAL result (LO):       ",
     &        ps_xsec(1)," +- ",sqrt(ps_sdev(1))," fb  ",
     &        sqrt(ps_sdev(1))/ps_xsec(1)*100d0,'%'
      end if
      if ((doVirtuals.and.hasNLO) .or. ewcor_switch) then
         print *," NLO virtual result:      "
     &        ,ps_xsec(2)," +- ",sqrt(ps_sdev(2))," fb  "
     &        ,sqrt(ps_sdev(2))/ps_xsec(2)*100d0,'%'
      end if
      if (doFLoops.and.hasFloops.and.(floops.ne.0)) then
         print *," NLO fermion loop result: "
     &        ,ps_xsec(5)," +- ",sqrt(ps_sdev(5))," fb  "
     &        ,sqrt(ps_sdev(5))/abs(ps_xsec(5))*100d0,'%'
      end if
      if (doEmissions.and.hasNLO) then
         print *," QCD real emission:       "
     &        ,ps_xsec(3)," +- ",sqrt(ps_sdev(3))," fb  "
     &        ,sqrt(ps_sdev(3))/abs(ps_xsec(3))*100d0,'%'
      end if
      if (ewcor_switch .and. (sector .ge. 3)) then
         print *," QED real emission:       "
     &        ,ps_xsec(4)," +- ",sqrt(ps_sdev(4))," fb  "
     &        ,sqrt(ps_sdev(4))/abs(ps_xsec(4))*100d0,'%'
      end if
      if ((doEmissions.and.hasNLO) .or. 
     &     (ewcor_switch .and. (sector .ge. 3))) then
         print *," TOTAL result (NLO):      "
     &        ,NLOxsection(-2)," +- ",NLOsdev(-2)," fb  "
     &        ,NLOsdev(-2)/NLOxsection(-2)*100d0,'%'
      end if
      if (doNLO .or. ewcor_switch) then         
         write(*,*) " K-Factor:                ",  NLOxsection(-2)/ps_xsec(1)
      end if
      print*,'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!' //
     &     '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
      write(*,*)' '
      write(*,*)' '


      if (showtiming) call mtime
   
      STOP


      END
c     END OF VBFNLO
    


