dnl most stuff in here is shamelessly adapted from Herwig++ or ThePEG

dnl some switches I think are useful

AC_DEFUN([VBFNLO_CHECK_KK],
[
AC_MSG_CHECKING([whether KK should be included])
AC_ARG_ENABLE(kk,
        AC_HELP_STRING([--enable-kk],[--enable-kk to enable simulation of Kaluza-Klein resonances]),
        [],
        [enable_kk=no]
        )
AC_MSG_RESULT([$enable_kk])
AM_CONDITIONAL(WITH_KK,[test "x$enable_kk" = "xyes"])

if test "x$enable_kk" == "xyes" ; then
VBFNLO_CHECK_GSL
fi

])

AC_DEFUN([VBFNLO_CHECK_SPIN2],
[
AC_MSG_CHECKING([whether Spin2 should be included])
AC_ARG_ENABLE(spin2,
        AC_HELP_STRING([--enable-spin2],[--enable-spin2 to enable simulation of Spin-2 resonances]),
        [],
        [enable_spin2=no]
        )
AC_MSG_RESULT([$enable_spin2])
AM_CONDITIONAL(WITH_SPIN2,[test "x$enable_spin2" = "xyes"])

])

AC_DEFUN([VBFNLO_CHECK_NLO],
[
AC_MSG_CHECKING([whether NLO should be enabled])
AC_ARG_ENABLE(NLO,
        AC_HELP_STRING([--enable-NLO],[--disable-NLO to disable next-to-leading order QCD]),
        [],
        [enable_NLO=yes]
        )
AC_MSG_RESULT([$enable_NLO])
AM_CONDITIONAL(WITH_NLO,[test "x$enable_NLO" = "xyes"])
])

dnl which processes to enable

AC_DEFUN([VBFNLO_ENABLE_PROCESSES],
[
AC_MSG_CHECKING([for processes to include])

AC_ARG_ENABLE(processes,
        AC_HELP_STRING([--enable-processes=LIST],[Comma-separated list of processes to enable. Options are (vbf ggf diboson dibosonjet triboson tribosonjet hjjj all all_except_hexagons)]),
        [],
        [enable_processes=all_except_hexagons]
        )
AC_MSG_RESULT([$enable_processes])

oldIFS="$IFS"
IFS=","
for i in $enable_processes; do
    declare $i=yes
done
IFS="$oldIFS"

AM_CONDITIONAL(WITH_DY,[test "$dy" -o "$all" -o "$all_except_hexagons"])
AM_CONDITIONAL(WITH_DIBOSON,[test "$diboson" -o "$all" -o "$all_except_hexagons"])
AM_CONDITIONAL(WITH_GGF,[test "$ggf" -o "$all" -o "$all_except_hexagons"])
AM_CONDITIONAL(WITH_DIBOSONJET,[test "$dibosonjet" -o "$all" -o "$all_except_hexagons"])
AM_CONDITIONAL(WITH_TRIBOSON,[test "$triboson" -o "$all" -o "$all_except_hexagons"])
AM_CONDITIONAL(WITH_VBF,[test "$vbf" -o "$all" -o "$all_except_hexagons"])
AM_CONDITIONAL(WITH_HJJJ,[test "$hjjj" -o "$all" -o "$all_except_hexagons"])
AM_CONDITIONAL(WITH_TRIBOSONJET,[test "$tribosonjet" -o "$all"])
AM_CONDITIONAL(WITH_HEXAGONS,[test "$tribosonjet" -o "$all"])
AM_CONDITIONAL(WITH_GGFLO,[test "$ggf" -o "$diboson" -o "$all" -o "$all_except_hexagons"])
])

dnl ##### ROOT #####
AC_DEFUN([VBFNLO_CHECK_ROOT],[

ROOTPATH=""
ROOTLIBPATH=""
ROOTLIBS=""
ROOTLDFLAGS=""
ROOTINCLUDE=""

AC_MSG_CHECKING([for ROOT])


AC_ARG_WITH(root,
        AC_HELP_STRING([--with-root=DIR],[location of ROOT installation]),
        [],
        [with_root=no])

if test "x$with_root" = "xno"; then
  AC_MSG_RESULT([not required])
else

  if test \( "x$with_root" = "xyes" \) -o \( "x$with_root" = "x" \); then
    with_root="$ROOTSYS"
  fi
  ROOTPATH="$with_root"

  if test -f $ROOTPATH/bin/root-config; then

    AC_MSG_RESULT([$ROOTPATH])

    AC_MSG_CHECKING([for ROOTLIBS])
    if test -z "$ROOTLIBS"; then
      ROOTLIBS=`$ROOTPATH/bin/root-config --libs`
      ROOTLIBPATH=`$ROOTPATH/bin/root-config --libdir`
      ROOTLDFLAGS="-L$ROOTLIBPATH -Wl,-rpath,$ROOTLIBPATH"
    fi
    AC_MSG_RESULT([$ROOTLIBS])

    AC_MSG_CHECKING([for ROOTINCLUDE])
    if test -z "$ROOTINCLUDE"; then
      ROOTINCLUDE="`$ROOTPATH/bin/root-config --cflags`"
    fi
    AC_MSG_RESULT([$ROOTINCLUDE])


    oldLIBS="$LIBS"
    oldLDFLAGS="$LDFLAGS"
    oldCPPFLAGS="$CPPFLAGS"
    LIBS="$LIBS $ROOTLIBS"
    LDFLAGS="$LDFLAGS $ROOTLDFLAGS"
    CPPFLAGS="$CPPFLAGS $ROOTINCLUDE"

    AC_MSG_CHECKING([that ROOT works])
    AC_LANG_PUSH([C++])
    AC_LINK_IFELSE([AC_LANG_PROGRAM([[#include <TCanvas.h>]],
    	[[TCanvas c("c1", "");]])],[AC_MSG_RESULT([yes])],[AC_MSG_RESULT([no]) 
     	AC_MSG_ERROR([Use '--with-root=' to set the path to your ROOT installation.\
	If it doesn't work anyhow, you eventually have to set the ROOTSYS environment variable.])
    ])
    AC_LANG_POP([C++])

    LIBS="$oldLIBS"
    LDFLAGS="$oldLDFLAGS"
    CPPFLAGS="$oldCPPFLAGS"

    AC_SUBST(ROOTLIBS)
    AC_SUBST(ROOTLIBPATH)
    AC_SUBST(ROOTLDFLAGS)
    AC_SUBST(ROOTPATH)
    AC_SUBST(ROOTINCLUDE)

  else
    AC_MSG_ERROR([root-config not found...aborting])    
  fi

fi
AM_CONDITIONAL(WITH_ROOT, test ! x"$with_root" = "xno")

])

dnl ###### GSL ######
AC_DEFUN([VBFNLO_CHECK_GSL],
[
AC_MSG_CHECKING([for gsl location])
GSLINCLUDE=""
GSLLDFLAGS=""
GSLLIBS=""

AC_ARG_WITH(gsl,
        AC_HELP_STRING([--with-gsl=DIR],[location of gsl installation @<:@default=system libs@:>@]),
        [],
	[with_gsl=system])

if test "x$with_gsl" = "xno"; then
AC_MSG_ERROR([libgsl is required. Please install the GNU scientific library and header files.])
fi

if test "x$with_gsl" = "xyes"; then
  with_gsl="system"
fi

if test "x$with_gsl" = "xsystem"; then
        AC_LANG_PUSH([C])
	AC_MSG_RESULT([in system libraries])
	oldlibs="$LIBS"
	AC_SEARCH_LIBS(gsl_integration_qags,gsl,[],
			[
			AC_MSG_ERROR([Cannot find libgsl. Please install the GNU scientific library and header files or use --with-gsl=.])
			],
                        [-lgslcblas -lm]
		     )
	GSLLIBS="$LIBS -lgslcblas -lm "
	LIBS=$oldlibs
        AC_LANG_POP([C])
else
	if test "`uname -m`" = "x86_64" -a \( -e "$with_gsl/lib64/libgsl.a" -o -e "$with_gsl/lib64/libgsl.so" \) -a -d "$with_gsl/include/gsl"; then
		AC_MSG_RESULT([found in $with_gsl])
		GSLLIBS="-L$with_gsl/lib64 -Wl,-rpath,$with_gsl/lib64 -lgslcblas -lgsl -lm"
                GSLLDFLAGS="-L$with_gsl/lib64 -Wl,-rpath,$with_gsl/lib64"
		GSLINCLUDE="-I$with_gsl/include"
	elif test \( -e "$with_gsl/lib/libgsl.a" -o -e "$with_gsl/lib/libgsl.so" \) -a -d "$with_gsl/include/gsl"; then
		AC_MSG_RESULT([found in $with_gsl])
		GSLLIBS="-L$with_gsl/lib -Wl,-rpath,$with_gsl/lib -lgslcblas -lgsl -lm"
                GSLLDFLAGS="-L$with_gsl/lib64 -Wl,-rpath,$with_gsl/lib64"
		GSLINCLUDE="-I$with_gsl/include"
	else
		AC_MSG_RESULT([not found])
		AC_MSG_ERROR([Can't find $with_gsl/lib/libgsl.a / $with_gsl/lib/libgsl.so or the headers in $with_gsl/include])
	fi
fi
GSLLIBS+="  -lstdc++"

AC_SUBST(GSLINCLUDE)
AC_SUBST(GSLLDFLAGS)
AC_SUBST(GSLLIBS)
])

dnl #### LHAPDF ######
AC_DEFUN([VBFNLO_CHECK_LHAPDF],
[
AC_MSG_CHECKING([for LHAPDF])
HAS_LHAPDF="yes"
LHAPDF_DIR=""
LHAPDF_LIBDIR=""
AC_ARG_WITH(LHAPDF,[  --with-LHAPDF=/path/to/LHAPDF/ or --without-LHAPDF to use internal PDF sets], [if test -n "$with_LHAPDF" -a "x$with_LHAPDF" != "xyes" -a "x$with_LHAPDF" != "xno"; then LHAPDF_DIR="$with_LHAPDF"; elif test "x$with_LHAPDF" = "xno"; then HAS_LHAPDF="no"; fi])

if test -n "$LHAPDF_DIR"; then
	if test "`uname -m`" = "x86_64" -a -d "$LHAPDF_DIR/lib64" -a -e "$LHAPDF_DIR/lib64/libLHAPDF.la" ; then
		LHAPDF_LIBDIR=$LHAPDF_DIR/lib64
	else
		LHAPDF_LIBDIR=$LHAPDF_DIR/lib
	fi
fi

LHAPDF_LDFLAGS=""
if test -n "$LHAPDF_LIBDIR"; then
  if test -e $LHAPDF_LIBDIR/libLHAPDF.so -o -e $LHAPDF_LIBDIR/libLHAPDF.dylib
  then
    LHAPDF_LDFLAGS="-L$LHAPDF_LIBDIR -Wl,-rpath,$LHAPDF_LIBDIR"
  else
    HAS_LHAPDF="no"
  fi
fi

LHAPDF_LIBS="-lLHAPDF"

oldLIB="$LIBS"
oldLDFLAGS="$LDFLAGS"

if test "$HAS_LHAPDF" == "yes"; then
dnl Now lets see if the libraries work properly
  LIBS="$LIBS $LHAPDF_LIBS"
  LDFLAGS="$LDFLAGS $LHAPDF_LDFLAGS"
  AC_LANG_PUSH([C++])
  AC_LINK_IFELSE([AC_LANG_PROGRAM([[extern "C" { void initpdf_(int&); }]],
                                  [[int i = 1; initpdf_(i);]])], ,
                                  HAS_LHAPDF="no")
  AC_LANG_POP([C++])
fi

LIBS="$oldLIB"
LDFLAGS="$oldLDFLAGS"

if test "$HAS_LHAPDF" == "yes"; then
  AC_MSG_RESULT([yes])

  AC_MSG_CHECKING([path to pdf sets])
dnl Try to get PDF path
  LHAPDF_PDFPATH=""
  if test -x "$LHAPDF_DIR/bin/lhapdf_config"; then
dnl   Try lhapdf-config
    LHAPDF_PDFPATH=`$LHAPDF_DIR/bin/lhapdf_config --pdfsets-path`
    if ! test -d "$LHAPDF_PDFPATH"; then
dnl   Try without final PDFsets directory
      LHAPDF_PDFPATH="${LHAPDF_PDFPATH%PDFsets}"
      if ! test -d "$LHAPDF_PDFPATH"; then
        LHAPDF_PDFPATH=""
      fi
    fi
  fi
  if test "x$LHAPDF_PDFPATH" == "x"; then
dnl   Try standard dir
    LHAPDF_PDFPATH="$LHAPDF_DIR/share/lhapdf/PDFsets"
    if ! test -d "$LHAPDF_PDFPATH"; then
dnl   Try without final PDFsets directory
      LHAPDF_PDFPATH="${LHAPDF_PDFPATH%PDFsets}"
      if ! test -d "$LHAPDF_PDFPATH"; then
        LHAPDF_PDFPATH=""
      fi
    fi
  fi

  if test "x$LHAPDF_PDFPATH" == "x"; then
    AC_MSG_RESULT([not found])
    AC_WARN([Could not guess path to pdfs.])
    AC_WARN([Set it explicitly via PDFPATH or in vbfnlo.dat!])
  else
    AC_MSG_RESULT([$LHAPDF_PDFPATH])
  fi
elif test "x$with_LHAPDF" == "xno" -o "x$with_LHAPDF" == "x"; then
  AC_MSG_RESULT([not required])
else
  AC_MSG_ERROR([LHAPDF was requested but the library was not found.])
fi
AC_SUBST(LHAPDF_DIR)
AC_SUBST(LHAPDF_LIBS)
AC_SUBST(LHAPDF_LDFLAGS)
AC_SUBST(LHAPDF_PDFPATH)
AM_CONDITIONAL([WITH_LHAPDF], [test "x$HAS_LHAPDF" == "xyes"])
])


dnl #### HEPMC ######
AC_DEFUN([VBFNLO_CHECK_HEPMC],
[
AC_MSG_CHECKING([for HepMC])
AC_ARG_WITH(hepmc,
        AC_HELP_STRING([--with-hepmc=DIR],[location of HepMC installation]),
        [],
        [with_hepmc=no])

if test "x$with_hepmc" = "xno"; then
  AC_MSG_RESULT([not required])
else

HAS_HEPMC="yes"

if test "x$with_hepmc" != "xyes"; then
	if test "`uname -m`" = "x86_64" -a -d "$with_hepmc/lib64" ; then
		HEPMCLIBS="-L$with_hepmc/lib64 -Wl,-rpath,$with_hepmc/lib64 -lHepMC"
		HEPMCLDFLAGS="-L$with_hepmc/lib64 -Wl,-rpath,$with_hepmc/lib64"
	else
		HEPMCLIBS="-L$with_hepmc/lib -Wl,-rpath,$with_hepmc/lib -lHepMC"
		HEPMCLDFLAGS="-L$with_hepmc/lib -Wl,-rpath,$with_hepmc/lib"
	fi
	HEPMCINCLUDE="-I$with_hepmc/include"
else
	HEPMCLIBS="-lHepMC"
	HEPMCLDFLAGS=""
	HEPMCINCLUDE=""
fi
AC_LANG_PUSH([C++])
oldLIB="$LIBS"
oldLDFLAGS="$LDFLAGS"
oldCPPFLAGS="$CPPFLAGS"
LIBS="$LIBS $HEPMCLIBS"
LDFLAGS="$LDFLAGS $HEPMCLDFLAGS"
CPPFLAGS="$CPPFLAGS $HEPMCINCLUDE"
AC_LINK_IFELSE(
   [AC_LANG_PROGRAM([[#include <HepMC/GenEvent.h>]],
    	            [[HepMC::write_HepMC_IO_block_begin(std::cout);]]
                   )],
   [AC_MSG_RESULT([yes])],
   [AC_MSG_RESULT([no]) 
    AC_MSG_ERROR([Cannot find libHepMC. Please install the HepMC library and header files])]
)
LIBS=$oldLIB
LDFLAGS=$oldLDFLAGS
CPPFLAGS=$oldCPPFLAGS
AC_LANG_POP([C++])
fi
AC_SUBST(HEPMCLIBS)
AC_SUBST(HEPMCLDFLAGS)
AC_SUBST(HEPMCINCLUDE)
AM_CONDITIONAL([WITH_HEPMC], [test "x$HAS_HEPMC" == "xyes"])
])


dnl #### LOOPTOOLS ######

AC_DEFUN([VBFNLO_CHECK_LOOPTOOLS],
[
AC_MSG_CHECKING([for LOOPTOOLS])

HAS_LOOPTOOLS="yes"
LOOPTOOLS_DIR=""
LOOPTOOLS_LIBDIR=""
AC_ARG_WITH(LOOPTOOLS,[  --with-LOOPTOOLS=/path/to/LOOPTOOLS/ to enable the calculation of electroweak corrections], [if test -n "$with_LOOPTOOLS" -a "x$with_LOOPTOOLS" != "xyes" -a "x$with_LOOPTOOLS" != "xno"; then LOOPTOOLS_DIR="$with_LOOPTOOLS"; elif test "x$with_LOOPTOOLS" = "xno"; then HAS_LOOPTOOLS="no"; fi])


if test -n "$LOOPTOOLS_DIR"; then
    HAS_LOOPTOOLS="yes"

  if test -x "$LOOPTOOLS_DIR/lib"; then
    LOOPTOOLS_LIBDIR=$LOOPTOOLS_DIR/lib
  elif test -x "$LOOPTOOLS_DIR/lib64"; then
    LOOPTOOLS_LIBDIR=$LOOPTOOLS_DIR/lib64
  elif test -x "$LOOPTOOLS_DIR/$(tcsh -cf 'echo $HOSTTYPE')"; then
    LOOPTOOLS_DIR=$LOOPTOOLS_DIR/$(tcsh -cf 'echo $HOSTTYPE')
    if test -x "$LOOPTOOLS_DIR/lib"; then
      LOOPTOOLS_LIBDIR=$LOOPTOOLS_DIR/lib
    elif test -x "$LOOPTOOLS_DIR/lib64"; then
      LOOPTOOLS_LIBDIR=$LOOPTOOLS_DIR/lib64
    fi
  elif test -x "$LOOPTOOLS_DIR/$(tcsh -cf 'uname -m')-$(tcsh -cf 'uname -s')"; then
    LOOPTOOLS_DIR=$LOOPTOOLS_DIR/$(tcsh -cf 'uname -m')-$(tcsh -cf 'uname -s')
    if test -x "$LOOPTOOLS_DIR/lib"; then
      LOOPTOOLS_LIBDIR=$LOOPTOOLS_DIR/lib
    elif test -x "$LOOPTOOLS_DIR/lib64"; then
      LOOPTOOLS_LIBDIR=$LOOPTOOLS_DIR/lib64
    fi
  else
    LOOPTOOLS_LIBDIR=$LOOPTOOLS_DIR
    LOOPTOOLS_DIR=$(echo "$LOOPTOOLS_LIBDIR" | sed 's/lib64//g')
    LOOPTOOLS_DIR=$(echo "$LOOPTOOLS_DIR" | sed 's/lib//g')
  fi

  LOOPTOOLS_LDFLAGS="-L$LOOPTOOLS_LIBDIR"
  LOOPTOOLS_LIBS="-looptools"

 AC_MSG_CHECKING([link to LoopTools])
 oldlibs="$LIBS"
 LIBS="$LOOPTOOLS_LDFLAGS $LOOPTOOLS_LIBS"
 AC_LINK_IFELSE(
   [AC_LANG_PROGRAM([],
                    [[        call ffini()
                                test = A0(100D0)]])], 
   [LTVERSION="OLD"],
 [
   AC_LINK_IFELSE(
     [AC_LANG_PROGRAM([],
                      [[        call ltini()
                                test = A0(100D0)]])], 
     [LTVERSION="NEW"],
  AC_MSG_ERROR(["Could not link with LoopTools. Are you sure the path given is correct and that the same compiler (gfortran/g77) is used for VBFNLO and LoopTools?"]))
  ]
 )

 LIBS=$oldlibs

 oldLTDIR=$LOOPTOOLS_DIR
 LOOPTOOLS_DIR=$(echo "$LOOPTOOLS_DIR" | sed "s/$(tcsh -cf 'uname -m')-$(tcsh -cf 'uname -s')//g")
 LOOPTOOLS_DIR=$(echo "$LOOPTOOLS_DIR" | sed "s/$(tcsh -cf 'echo $HOSTTYPE')//g")
 if test -n "$(fgrep 'ff2c' $LOOPTOOLS_DIR/makefile)"; then
    AM_FCFLAGS="$AM_FCFLAGS -ff2c"
 fi
 LOOPTOOLS_DIR=$oldLTDIR


  { $as_echo "$as_me:${as_lineno-$LINENO}: result: yes" >&5
    $as_echo "okay" >&6; }

 if test "${enable_shared}" != "no"; then
  if objdump --reloc $LOOPTOOLS_LIBDIR/libooptools.a | grep -q R_X86_64_32S; then
   AC_MSG_ERROR(["The version of LoopTools you have linked to is statically linked.  Please re-run configure with the option '--enable-shared=no'."])
  fi
 fi

else
    HAS_LOOPTOOLS="no"
  { $as_echo "$as_me:${as_lineno-$LINENO}: result: not required" >&5
    $as_echo "not required" >&6; }
fi


AM_CONDITIONAL([NEW_LT], [test "$LTVERSION" == "NEW"])
AM_CONDITIONAL([OLD_LT], [test "$LTVERSION" == "OLD"])
AC_SUBST(LOOPTOOLS_DIR)
AC_SUBST(LOOPTOOLS_LIBS)
AC_SUBST(LOOPTOOLS_LDFLAGS)
AM_CONDITIONAL([WITH_LOOPTOOLS], [test "$HAS_LOOPTOOLS" == "yes"])
])


dnl #### FEYNHIGGS ######

AC_DEFUN([VBFNLO_CHECK_FEYNHIGGS],
[
AC_MSG_CHECKING([for FEYNHIGGS])

HAS_FEYNHIGGS="yes"
FEYNHIGGS_DIR=""
FEYNHIGGS_LIBDIR=""
AC_ARG_WITH(FEYNHIGGS,[  --with-FEYNHIGGS=/path/to/FEYNHIGGS/ to enable calculation of MSSM Higgs sector by FeynHiggs], [if test -n "$with_FEYNHIGGS" -a "x$with_FEYNHIGGS" != "xyes" -a "x$with_FEYNHIGGS" != "xno"; then FEYNHIGGS_DIR="$with_FEYNHIGGS"; elif test "x$with_FEYNHIGGS" = "xno"; then HAS_FEYNHIGGS="no"; fi])


if test -n "$FEYNHIGGS_DIR"; then

  if test -x "$FEYNHIGGS_DIR/lib"; then
    FEYNHIGGS_LIBDIR=$FEYNHIGGS_DIR/lib
  elif test -x "$FEYNHIGGS_DIR/lib64"; then
    FEYNHIGGS_LIBDIR=$FEYNHIGGS_DIR/lib64
  elif test -x "$FEYNHIGGS_DIR/$(tcsh -cf 'echo $HOSTTYPE')"; then
    FEYNHIGGS_DIR=$FEYNHIGGS_DIR/$(tcsh -cf 'echo $HOSTTYPE')
    if test -x "$FEYNHIGGS_DIR/lib"; then
      FEYNHIGGS_LIBDIR=$FEYNHIGGS_DIR/lib
    elif test -x "$FEYNHIGGS_DIR/lib64"; then
      FEYNHIGGS_LIBDIR=$FEYNHIGGS_DIR/lib64
    fi
  elif test -x "$FEYNHIGGS_DIR/$(tcsh -cf 'uname -m')-$(tcsh -cf 'uname -s')"; then
    FEYNHIGGS_DIR=$FEYNHIGGS_DIR/$(tcsh -cf 'uname -m')-$(tcsh -cf 'uname -s')
    if test -x "$FEYNHIGGS_DIR/lib"; then
      FEYNHIGGS_LIBDIR=$FEYNHIGGS_DIR/lib
    elif test -x "$FEYNHIGGS_DIR/lib64"; then
      FEYNHIGGS_LIBDIR=$FEYNHIGGS_DIR/lib64
    fi
  else
    FEYNHIGGS_LIBDIR=$FEYNHIGGS_DIR 
    FEYNHIGGS_DIR=$(echo "$FEYNHIGGS_LIBDIR" | sed 's/lib64//g')
    FEYNHIGGS_DIR=$(echo "$FEYNHIGGS_DIR" | sed 's/lib//g')
  fi

 FEYNHIGGS_LDFLAGS="-L$FEYNHIGGS_LIBDIR"
 FEYNHIGGS_LIBS="-lFH"

 AC_MSG_CHECKING([link to FeynHiggs])
 oldlibs="$LIBS"
 LIBS="$FEYNHIGGS_LDFLAGS $FEYNHIGGS_LIBS"
AC_LINK_IFELSE(
   [AC_LANG_PROGRAM([],
                    [[           call FHSetFlags(0,1,1,1,1,1,1,1,1,1)]])], 
   [ HAS_FEYNHIGGS="yes"],
 [ AC_MSG_ERROR(["Could not link with FeynHiggs.  Are you sure the path given is correct and that the same compiler (gfortran/g77) is used for VBFNLO and FeynHiggs?"])
  ]
 )
 LIBS=$oldlibs


  { $as_echo "$as_me:${as_lineno-$LINENO}: result: yes" >&5
    $as_echo "okay" >&6; }

 if test -n "$(fgrep 'FeynHiggs 2.6' $FEYNHIGGS_LIBDIR/libFH.a)"; then
    FHVERSION="26"
 elif test -n "$(fgrep 'FeynHiggs 2.7' $FEYNHIGGS_LIBDIR/libFH.a)"; then
    FHVERSION="27"
 elif test -n "$(fgrep 'FeynHiggs 2.8' $FEYNHIGGS_LIBDIR/libFH.a)"; then
    FHVERSION="28"
 else
  AC_MSG_ERROR(["Sorry, we don't recognise the version of FeynHiggs you are using."])
 fi

 if test "${enable_shared}" != "no"; then
  if objdump --reloc $FEYNHIGGS_LIBDIR/libFH.a | grep -q R_X86_64_32S; then
   AC_MSG_ERROR(["The version of FeynHiggs you have linked to is statically linked.  Please re-run configure with the option '--enable-shared=no'."])
  fi
  if test -n "$(fgrep 'FeynHiggs 2.8' $FEYNHIGGS_LIBDIR/libFH.a)"; then
    FHSHARED="NO"
  else
    FHSHARED="YES"
  fi 
 else
  FHSHARED="NO"
 fi

else
    HAS_FEYNHIGGS="no"
  { $as_echo "$as_me:${as_lineno-$LINENO}: result: not required" >&5
    $as_echo "not required" >&6; }
fi


AM_CONDITIONAL([FH26], [test "$FHVERSION" == "26"])
AM_CONDITIONAL([FH27], [test "$FHVERSION" == "27"])
AM_CONDITIONAL([FH28], [test "$FHVERSION" == "28"])
AM_CONDITIONAL([FH_SHARED], [test "$FHSHARED" == "YES"])
AM_CONDITIONAL([FH_28], [test "$FHSHARED" == "NO"])
AC_SUBST(FEYNHIGGS_DIR)
AC_SUBST(FEYNHIGGS_LIBS)
AC_SUBST(FEYNHIGGS_LDFLAGS)
AM_CONDITIONAL([WITH_FEYNHIGGS], [test "x$HAS_FEYNHIGGS" == "xyes"])
])



dnl #### check, which fortran compiler we are using #####
AC_DEFUN([VBFNLO_CHECK_FC],
[
  AC_MSG_CHECKING([which Fortran compiler we are using])
  VBFFC="unrecognized, assuming g77 behaviour"

  check_g77=`echo $FC | grep g77`
  check_gfortran=`echo $FC | grep gfortran`
  check_ifort=`echo $FC | grep ifort`

  if test -n "$check_g77" ; then
     VBFFC="g77"
  AM_FCFLAGS="$AM_FCFLAGS -fno-automatic -ffixed-line-length-none"
     AM_FCFLAGS="$AM_FCFLAGS -finit-local-zero"

  elif test -n "$check_gfortran" ; then
     AM_FCFLAGS="$AM_FCFLAGS -fno-automatic -ffixed-line-length-none"
     VBFFC="gfortran"
  elif test -n "$check_ifort" ; then
     AM_FCFLAGS="$AM_FCFLAGS -save -extend_source"
     VBFFC="ifort"

  fi

  AC_MSG_RESULT([$VBFFC])

])

dnl check for MadGraph comparison mode and set flags
AC_DEFUN([VBFNLO_CHECK_MADGRAPH],
[
AC_MSG_CHECKING([whether MadGraph comparisons should be included])
AC_ARG_ENABLE(madgraph,
        AC_HELP_STRING([--enable-madgraph],[--enable-madgraph to include code for MadGraph comparisons]),
        [],
        [enable_madgraph=no]
        )
AC_MSG_RESULT([$enable_madgraph])
AM_CONDITIONAL(WITH_MADGRAPH,[test "x$enable_madgraph" = "xyes"])
])


dnl check for debugging mode and set compiler flags

AC_DEFUN([VBFNLO_SET_FLAGS],
[
AC_MSG_CHECKING([for debugging mode])
AC_ARG_ENABLE(debug,
        AC_HELP_STRING([--enable-debug],[--enable-debug to compile with debug symbols]),
        [],
        [enable_debug=no]
        )
AC_MSG_RESULT([$enable_debug])
AM_CONDITIONAL(VBFNLO_DEBUG,[test "x$enable_debug" = "xyes"])


if test "x$enable_debug" == "xyes" ; then
  AM_FCFLAGS="$AM_FCFLAGS -g $APPLE_GFORTRAN_FIXES -I\$(top_builddir)/include"
  AM_CPPFLAGS="$AM_CPPFLAGS -I\$(top_builddir)/include \$(GSLINCLUDE) \$(LTINCLUDE)"
  AM_CXXFLAGS="$AM_CXXFLAGS -g -fno-inline"
else
  AM_FCFLAGS="$AM_FCFLAGS $APPLE_GFORTRAN_FIXES -I\$(top_builddir)/include"
  AM_CPPFLAGS="$AM_CPPFLAGS -I\$(top_builddir)/include \$(GSLINCLUDE) \$(LTINCLUDE) -DNDEBUG" 
fi
  AM_LDFLAGS="$AM_LDFLAGS $LDFLAGS $ROOTLDFLAGS $GSLLDFLAGS $LHAPDF_LDFLAGS $HEPMCLDFLAGS $LOOPTOOLS_LDFLAGS $FEYNHIGGS_LDFLAGS" 

  AC_SUBST(AM_CPPFLAGS)
  AC_SUBST(AM_CXXFLAGS)
  AC_SUBST(AM_FCFLAGS)
  AC_SUBST(AM_LDFLAGS)

  F77=$FC
  AM_FFLAGS=$AM_FCFLAGS
  AC_SUBST(F77)
  AC_SUBST(FFLAGS)
  AC_SUBST(AM_FFLAGS)

  AM_CONDITIONAL(WITH_LOOPS,[test "x$enable_NLO" == "xyes" -o "$ggf" -o "$diboson" -o "$all" -o "$all_except_hexagons"])

])

AC_DEFUN([VBFNLO_SUMMARY],
[
if test "x$with_LHAPDF" == "x" ; then
  with_LHAPDF="no"
fi
if test "x$with_LOOPTOOLS" == "x" ; then
  with_LOOPTOOLS="no"
fi
if test "x$with_FEYNHIGGS" == "x" ; then
  with_FEYNHIGGS="no"
fi
if test "x$with_gsl" == "x" ; then
  with_gsl="not required"
fi

if test "x$with_looptools" == "x" ; then
  with_looptools="no"
fi

if test "x$with_hepmc" == "x" ; then
  with_hepmc="no"
fi

echo "=================================================="
echo ""
echo "  $PACKAGE_STRING configuration"
echo ""
echo "  Processes included:"
echo "  $enable_processes"
echo ""
echo "  NLO QCD           $enable_NLO"
echo "  KK resonances     $enable_kk"
echo "  Spin2 resonances  $enable_spin2"
echo "  Debug symbols     $enable_debug"
if test "x$enable_madgraph" = "xyes" ; then
echo "  MadGraph code     included"
fi
echo ""
echo "  LHAPDF            $with_LHAPDF"
echo "  LOOPTOOLS         $with_LOOPTOOLS"
echo "  FEYNHIGGS         $with_FEYNHIGGS"
echo "  ROOT              $with_root"
echo "  GSL               $with_gsl"
echo "  HEPMC             $with_hepmc"

echo ""
echo "=================================================="
])

