c*************************************************************************
      SUBROUTINE InitPhaseSpace
c*************************************************************************
c     Setup phasespace generators.
c*************************************************************************

      implicit none

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/process.inc"

      SELECT CASE(procID)
      CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjjj,
     1     Hjjj_AA, Hjjj_mu, Hjjj_tau, Hjjj_bbar, 
     2     HAjj, HAjj_AA, HAjj_mu, HAjj_tau, 
     3     HAjj_bbar)
         call def_ps_vnj(6)
      CASE(AAjj)
         call def_ps_vnj(39)
c         call def_ps_vnj(6)   !take this line instead of the previous one if 
                               !you want to run this process 
                               !with a Higgs instead of a spin-2 resonance 
                               !(second part of spin2toaa.F)
      CASE(Hjj_WW,Hjjj_WW, HAjj_WW, Hjj_WPhadWM, Hjj_WPWMhad)
         call def_ps_vvnj(6,3,4,0,1,1)
      CASE(Hjj_ZZ_ll, Hjj_ZZ_lnu,Hjjj_ZZ_ll,Hjjj_ZZ_lnu,
     $        HAjj_ZZ_ll, HAjj_ZZ_lnu, Hjj_ZZhad)
         call def_ps_vvnj(6,2,2,0,1,1)
      CASE(Ajj)
         call def_ps_vnj(1)
      CASE(Zjj_l, Zjj_nu)
         call def_ps_vnj(2)
      CASE(WPjj)
         call def_ps_vnj(3)
      CASE(WMjj)
         call def_ps_vnj(4)
      CASE(WPWMjj, WPhadWMjj, WPWMhadjj)
         call def_ps_vvnj(-6,3,4,1,1,1)
      CASE(WPWPjj,WMWMjj, WPhadWPjj, WMhadWMjj)
         call def_ps_vvnj(0,3,4,1,1,1)
      CASE(ZZjj_ll, ZZjj_lnu, ZZhadjj) 
         call def_ps_vvnj(-6,2,2,1,1,1)
      CASE(WPZjj, WPhadZjj, WPZhadjj)
         call def_ps_vvnj(0,3,2,1,1,1)
      CASE(WMZjj, WMhadZjj, WMZhadjj)
         call def_ps_vvnj(0,4,2,1,1,1)
      CASE(GFHjj, GFHjj_AA, GFHjj_mu, GFHjj_tau, GFHjj_bbar)
         call def_ps_vnj(6) 
      CASE(GFHjj_WW)
         call def_ps_vvnj(6,3,4,0,1,1)
      CASE(GFHjj_ZZ_ll, GFHjj_ZZ_lnu)
         call def_ps_vvnj(6,2,2,0,1,1)

#ifdef WITH_DIBOSON
      CASE(WW,GFWW,WPhadWMlep,WPlepWMhad,GFWPhadWMlep,GFWPlepWMhad)
         call def_ps_qqVVj(3,4)
      CASE(WPZ,WPhadZ,WPZhad)
         call def_ps_qqVVj(3,2)
      CASE(WMZ,WMhadZ,WMZhad)
         call def_ps_qqVVj(4,2)
      CASE(ZZ,ZZhad,GFZZ,GFZZhad)
         call def_ps_qqVVj(2,2)
      CASE(WPA)
         call def_ps_qqVAj(3,1)
      CASE(WMA)
         call def_ps_qqVAj(4,1)
      CASE(ZA,GFZA)
         call def_ps_qqVAj(2,1)
      CASE(AA,GFAA)
         call def_ps_qqAAj(1,1)
#endif

#ifdef WITH_DIBOSONJET
      CASE(WMAJ,WPAJ)
         call def_ps_vnj(3)
      CASE(WMZJ, WMhadZJ, WMZhadJ)
         call def_ps_vvnj(0,4,2,1,1,1)
      CASE(WPZJ, WPhadZJ, WPZhadJ)
         call def_ps_vvnj(0,3,2,1,1,1)
!           call def_ps_qqVVj(3,2)         
#endif

#ifdef WITH_TRIBOSON

        CASE(WWZ, WPhadWMZ, WPWMhadZ, WWZhad)
#ifdef WITH_KK
        if (with_kk) then
           call def_ps_WWZ_kk(3,2,4)
        else
#endif
           call def_ps_qqVVVj(3,4,2)
#ifdef WITH_KK
        endif
#endif
        CASE(ZZWP, ZZhadWP, ZZWPhad)
#ifdef WITH_KK
        if (with_kk) then
           call def_ps_ZZW_kk(2,3,2)
        else
#endif
           call def_ps_qqVVVj(2,2,3)
#ifdef WITH_KK
        endif   
#endif
      CASE(ZZWM, ZZhadWM, ZZWMhad)
#ifdef WITH_KK
        if (with_kk) then
         call def_ps_ZZW_kk(2,4,2)
        else
#endif
           call def_ps_qqVVVj(2,2,4)
#ifdef WITH_KK
        endif
#endif
      CASE(WWWP, WPhadWMWP, WPWMhadWP)
#ifdef WITH_KK
        if (with_kk) then
         call def_ps_WWW_kk(3,4,3)
        else
#endif
         call def_ps_WWW(3,4,3)
#ifdef WITH_KK
        endif
#endif
      CASE(WWWM, WMhadWPWM, WMWPhadWM)
#ifdef WITH_KK
        if (with_kk) then
         call def_ps_WWW_kk(4,3,4)
        else
#endif
         call def_ps_WWW(4,3,4)
#ifdef WITH_KK
        endif
#endif
      CASE(ZZZ, ZZZhad)
         call def_ps_ZZZj(2,2,2)
      CASE(WWA)
         call def_ps_qqWWAj(3,4,1)
      CASE(ZZA)
         call def_ps_qqZZAj(2,2,1)
      CASE(WPZA)
         call def_ps_qqWZAj(3,2,1)
      CASE(WMZA)
         call def_ps_qqWZAj(4,2,1)
      CASE(WPAA)
         call def_ps_qqWAAj(3,1,1)
      CASE(WMAA)
         call def_ps_qqWAAj(4,1,1)
      CASE(ZAA,ZnAA)
         call def_ps_qqZAAj(2,1,1)
      CASE(AAA)
         call def_ps_qqAAAj(1,1,1)

#endif
  
#ifdef WITH_TRIBOSONJET
      CASE(WPAAj)
         call def_ps_qqWAAj(3,1,1)
      CASE(WMAAj)
         call def_ps_qqWAAj(4,1,1)
#endif

      CASE DEFAULT
         print*,"No initialization of phase space for this process"
         stop    
      END SELECT

      RETURN
      END


c*************************************************************************
      SUBROUTINE phasespace(rand,
     &                      p, x,
     &                      v, 
     $                      ps_number,  
     &                      dps  )
c*************************************************************************
c     This is the manager of all phasespace generators and the (only) 
c     interface of those to the main program.  Depending on the process ID
c     the right phasespace generator is called with the correct calling 
c     convention, which might well be individually different.
c
c*************************************************************************
c     INPUT
c     rand            : Array with random numbers driven by monaco. 
c     p               : 4-momenta of the partons involved in the basic
c                       QCD 2->2(+1)+X process.
c                       The first 2 entries are the incoming partons.
c     x               : the feynman x of the incoming partons 
c     v               : The 4-momenta of the other particles
c     ps_number       : Number specifying the desired phase space
c     
c     OUTPUT
c     dps             : The jacobi factor of the phasespace point.
c*************************************************************************
      implicit none
      
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/process.inc"

      real*8 x(nx) 
      real*8 rand(max_PS_dim)
      real*8 p(0:3,max_p,max_kin)
      real*8 v(0:3,max_v,max_kin)
      real*8 dps, dps_dec

      real*8 rn
      real*8 Q(0:4), QV(0:4,2)
      integer i,L,mu, ps_number
      common/MB1/ rn
      real*8 RandomNumber,rdec(1:5), mjj2
      external RandomNumber, mjj2
      double precision phirn, cosrn, sinrn, tmpx, tmpy
      real*8 TPI
      parameter (TPI=2D0*PI )

c fill momentum arrays with zero
      do L=1,max_kin
         do i = 1,max_p
            do mu=0,3
               p(mu,i,L) = 0d0
            enddo
         enddo
      enddo
      do L=1,max_kin
         do i=1,max_v
            do mu=0,3
               v(mu,i,L) = 0d0
            enddo
         enddo
      enddo

      rn = 0d0
c switch between different processes

      SELECT CASE(procID)
      CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar) ! pp-> Hjj(j) -> xxjj(j)
         call LIPSN(n_p-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps, 1)
      CASE(AAjj) ! pp-> spin-2 jj(j) -> AAjj(j)
         call LIPSN(n_p-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps, 1)
      CASE(Hjj_WW, Hjj_ZZ_ll, Hjj_ZZ_lnu,
     1     Hjj_WPhadWM, Hjj_WPWMhad, Hjj_ZZhad) ! pp -> Hjj(j) -> VVjj(j)
         call LIPSNVV(n_p-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)
      CASE(Ajj)
         call LIPSN0(n_p-nx, rand, rn,
     1               p(0,1,1), p(0,2,1), 
     2               Q, p(0,3,1), X(1), X(2), dps)
         v(0,1,1) = Q(0)
         v(1,1,1) = Q(1)
         v(2,1,1) = Q(2)
         v(3,1,1) = Q(3)   
      CASE(HAjj, HAjj_AA, HAjj_mu, HAjj_tau,  ! pp-> HAjj(j) -> Axxjj(j)
     3     HAjj_bbar)
         call LIPSN(n_p+1-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps,1)
         v(0,3,1)=p(0,n_p+1,1)
         v(1,3,1)=p(1,n_p+1,1)
         v(2,3,1)=p(2,n_p+1,1)
         v(3,3,1)=p(3,n_p+1,1)
         p(0,n_p+1,1)=0d0
         p(1,n_p+1,1)=0d0
         p(2,n_p+1,1)=0d0
         p(3,n_p+1,1)=0d0
      CASE(HAjj_WW, HAjj_ZZ_ll, HAjj_ZZ_lnu)
         call LIPSNVV(n_p+1-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)
         v(0,5,1)=p(0,n_p+1,1)
         v(1,5,1)=p(1,n_p+1,1)
         v(2,5,1)=p(2,n_p+1,1)
         v(3,5,1)=p(3,n_p+1,1)
         p(0,n_p+1,1)=0d0
         p(1,n_p+1,1)=0d0
         p(2,n_p+1,1)=0d0
         p(3,n_p+1,1)=0d0
      CASE(Zjj_l, Zjj_nu, WPjj, WMjj) ! pp -> Vjj(j) -> lljj(j)
         call LIPSN(n_p-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps, 0)
      CASE(WPWMjj,WPWPjj,WMWMjj,ZZjj_ll,ZZjj_lnu,WPZjj,WMZjj,
     1     WPhadWMjj, WPWMhadjj, WPhadZjj, WPZhadjj, WMhadZjj, WMZhadjj, ZZhadjj,
     2     WPhadWPjj, WMhadWMjj) ! pp -> VVjj(j)
         call LIPSNVV(n_p-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)
      CASE(HJJJ,Hjjj_AA, Hjjj_mu, Hjjj_tau, Hjjj_bbar) ! pp -> H + 3 jets
         call LIPSN(n_p-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps, 1)
      CASE(Hjjj_WW, Hjjj_ZZ_ll, Hjjj_ZZ_lnu) ! pp -> Hjjj(j) -> VVjjj(j)
         call LIPSNVV(n_p-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)  
      CASE(GFHjj) ! pp -> Hjj in GF   
         call LIPSN0(n_p-nx, rand, rn,
     1               p(0,1,1), p(0,2,1), 
     2               Q, p(0,3,1), X(1), X(2), dps)
         v(0,1,1) = Q(0)
         v(1,1,1) = Q(1)
         v(2,1,1) = Q(2)
         v(3,1,1) = Q(3) 
      CASE(GFHjj_AA, GFHjj_mu, GFHjj_tau, GFHjj_bbar)     
         call LIPSN(n_p-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps, 1)  
      CASE(GFHjj_WW, GFHjj_ZZ_ll, GFHjj_ZZ_lnu)           
         call LIPSNVV(n_p-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)

#ifdef WITH_DIBOSON
      CASE(WW,GFWW,WPhadWMlep,WPlepWMhad,GFWPhadWMlep,GFWPlepWMhad)
         call LIPSVVJ(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)

      CASE(WPZ,WPhadZ,WPZhad)
         call LIPSVVJ(n_p-nx,rand,rn,
     1        p(0,1,1), p(0,2,1),
     2        v(0,1,1),p(0,3,1),
     3        X(1),X(2),dps,narrowwidth)

      CASE(WMZ,WMhadZ,WMZhad)
         call LIPSVVJ(n_p-nx,rand,rn,
     1        p(0,1,1), p(0,2,1),
     2        v(0,1,1),p(0,3,1),
     3        X(1),X(2),dps,narrowwidth)
         
      CASE(ZZ,ZZhad,GFZZ,GFZZhad)
         call LIPSVVJ(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)

      CASE(WPA)
         if (ps_number.eq.1) then
           call LIPSVAJ1(n_p-nx,rand,rn,
     1                   p(0,1,1), p(0,2,1),
     2                   v(0,1,1),p(0,3,1),
     3                   X(1),X(2),dps,narrowwidth)
         else if (ps_number.eq.2) then
           call LIPSVAJ2(n_p-nx,rand,rn,
     1                   p(0,1,1), p(0,2,1),
     2                   v(0,1,1),p(0,3,1),
     3                   X(1),X(2),dps,narrowwidth)
         endif

      CASE(WMA)
         if (ps_number.eq.1) then
           call LIPSVAJ1(n_p-nx,rand,rn,
     1                   p(0,1,1), p(0,2,1),
     2                   v(0,1,1),p(0,3,1),
     3                   X(1),X(2),dps,narrowwidth)
         else if (ps_number.eq.2) then
           call LIPSVAJ2(n_p-nx,rand,rn,
     1                   p(0,1,1), p(0,2,1),
     2                   v(0,1,1),p(0,3,1),
     3                   X(1),X(2),dps,narrowwidth)
         endif

      CASE(ZA,GFZA)
c flipped because for GFZA only PS1 relevant
         if (ps_number.eq.1) then
           call LIPSVAJ1(n_p-nx,rand,rn,
     1                   p(0,1,1), p(0,2,1),
     2                   v(0,1,1),p(0,3,1),
     3                   X(1),X(2),dps,narrowwidth)
         else if (ps_number.eq.2) then
           call LIPSVAJ2(n_p-nx,rand,rn,
     1                   p(0,1,1), p(0,2,1),
     2                   v(0,1,1),p(0,3,1),
     3                   X(1),X(2),dps,narrowwidth)
         endif

      CASE(AA,GFAA)
         call LIPSAAJ(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)
#endif

#ifdef WITH_DIBOSONJET
      CASE(WMZJ, WMhadZJ, WMZhadJ, WPZJ, WPhadZJ, WPZhadJ)
         call LIPSNVV(n_p-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)
!          call LIPSVVJ(n_p-nx,rand,rn,
!      1                p(0,1,1), p(0,2,1),
!      2                v(0,1,1),p(0,3,1),
!      3                X(1),X(2),dps,narrowwidth)
        
      CASE(WMAJ,WPAJ)
         if (ps_number.eq.1) then
            call LIPSNthreebod(n_p-nx, rand, rn,
     1           p(0,1,1), p(0,2,1), 
     2           Q, v(0,1,1), v(0,2,1),v(0,3,1),
     3           p(0,3,1), X(1), X(2), dps)
         else if (ps_number.eq.2) then
            call LIPSN(n_p-nx+1, rand, rn,
     1           p(0,1,1), p(0,2,1), 
     2           Q, v(0,1,1), v(0,2,1),
     3           p(0,3,1), X(1), X(2), dps, 0)
            do mu=0,3
               v(mu,3,1) = p(mu,n_p+1,1) !Generate photon momenta as jet momenta
               p(mu,n_p+1,1) = 0d0
            enddo        
         endif

#endif

#ifdef WITH_TRIBOSON

        CASE(WWZ, WPhadWMZ, WPWMhadZ, WWZhad)
#ifdef WITH_KK
        if (with_kk) then
         call LIPSWWZ_kk(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)
        else
#endif
         call LIPSVVVJ(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)
#ifdef WITH_KK
        endif
#endif
      CASE(ZZWP, ZZhadWP, ZZWPhad, ZZWM, ZZhadWM, ZZWMhad)
#ifdef WITH_KK
        if (with_kk) then
        
         call LIPSZZW_kk(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)
        else
#endif
         call LIPSVVVJ(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)
#ifdef WITH_KK
        endif
#endif

      CASE(WWWP, WPhadWMWP, WPWMhadWP, WWWM, WMhadWPWM, WMWPhadWM)
#ifdef WITH_KK
        if (with_kk) then
         call LIPSWWW_kk(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)
        else
#endif
         call LIPSWWW(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)
#ifdef WITH_KK
        endif
#endif

      CASE(ZZZ, ZZZhad)
         call LIPSZZZJ(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth, ps_number)


      CASE(ZZA)
         if(with_anom) then
            if (ps_number.eq.1) then
               call LIPSZZAJ_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.2) then
               call LIPSZZAJ1_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.3) then
               call LIPSZZAJ2_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            endif
         else
            if (ps_number.eq.1) then
               call LIPSZZAJ(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.2) then
               call LIPSZZAJ1(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.3) then
               call LIPSZZAJ2(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            endif
         endif


      CASE(WWA)
         if(with_anom) then
            if (ps_number.eq.1) then
               call LIPSWWAJ_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.2) then
               call LIPSWWAJ1_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.3) then
               call LIPSWWAJ2_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            endif
         else
            if (ps_number.eq.1) then
               call LIPSWWAJ(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.2) then
               call LIPSWWAJ1(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.3) then
               call LIPSWWAJ2(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            endif
         endif


      CASE(WPZA, WMZA)
         if(with_anom) then
            if (ps_number.eq.1) then
               call LIPSWZAJ_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.2) then
               call LIPSWZAJ1_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.3) then
               call LIPSWZAJ2_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            endif
         else
            if (ps_number.eq.1) then
               call LIPSWZAJ(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.2) then
               call LIPSWZAJ1(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.3) then
               call LIPSWZAJ2(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            endif
         endif

      CASE(WPAA, WMAA)
         if(with_anom) then
            if (ps_number.eq.1) then
               call LIPSWAAJ1_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.2) then
               call LIPSWAAJ2_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.3) then
               call LIPSWAAJ3_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.4) then
               call LIPSWAAJ4_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.5) then   !one additional ps for H -> A A contribution
               call LIPSWAAJ5_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            endif
         else
            if (ps_number.eq.1) then
               call LIPSWAAJ1(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.2) then
               call LIPSWAAJ2(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.3) then
               call LIPSWAAJ3(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.4) then
               call LIPSWAAJ4(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            endif
         endif

      CASE(ZAA)
         if(with_anom) then
            if (ps_number.eq.1) then
               call LIPSZAAJ1_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.2) then
               call LIPSZAAJ2_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.3) then
               call LIPSZAAJ3_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.4) then
               call LIPSZAAJ4_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.5) then   !one additional ps for H -> A A contribution
               call LIPSZAAJ5_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            endif
         else
            if (ps_number.eq.1) then
               call LIPSZAAJ1(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.2) then
               call LIPSZAAJ2(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.3) then
               call LIPSZAAJ3(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.4) then
               call LIPSZAAJ4(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            endif
         endif

      CASE(ZnAA)
         if(with_anom) then !multichannel due to new Higgs couplings
            if (ps_number.eq.1) then
               call LIPSZnAAJ1_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.2) then !for H -> Z A1
               call LIPSZnAAJ2_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.3) then !for H -> Z A2
               call LIPSZnAAJ3_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.4) then !for H -> A A
               call LIPSZnAAJ5_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            endif
         else
            call LIPSZnAAJ1(n_p-nx,rand,rn,
     1           p(0,1,1), p(0,2,1),
     2           v(0,1,1),p(0,3,1),
     3           X(1),X(2),dps,narrowwidth)
         endif


      CASE(AAA)
         if(with_anom) then !multichannel due to new Higgs couplings
            if (ps_number.eq.1) then
               call LIPSAAAJ_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.2) then !for H -> A1 A2
               call LIPSAAAJ2_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.3) then !for H -> A1 A3
               call LIPSAAAJ3_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            elseif (ps_number.eq.4) then !for H -> A2 A3
               call LIPSAAAJ4_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps)
            endif
         else
            call LIPSAAAJ(n_p-nx,rand,rn,
     1           p(0,1,1), p(0,2,1),
     2           v(0,1,1),p(0,3,1),
     3           X(1),X(2),dps)
         endif
#endif

#ifdef WITH_TRIBOSONJET
      CASE(WPAAj, WMAAj)
         if(with_anom) then
            if (ps_number.eq.1) then
               call LIPSWAAJ1_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.2) then
               call LIPSWAAJ2_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.3) then
               call LIPSWAAJ3_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.4) then
               call LIPSWAAJ4_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.5) then   !one additional ps for H -> A A contribution
               call LIPSWAAJ5_anomal(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            endif
         else
            if (ps_number.eq.1) then
               call LIPSVAAJ1(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.2) then
               call LIPSVAAJ2(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.3) then
               call LIPSVAAJ3(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            elseif (ps_number.eq.4) then
               call LIPSVAAJ4(n_p-nx,rand,rn,
     1              p(0,1,1), p(0,2,1),
     2              v(0,1,1),p(0,3,1),
     3              X(1),X(2),dps,narrowwidth)
            endif
         endif
#endif

      CASE DEFAULT
         print*," No phase space generator is called"
         stop   
      END SELECT       


c get additional random number not driven by VEGAS    
      rn = RandomNumber()
c globally rotate by angle(rn)
      phirn = TPI*rn
      cosrn = cos(phirn)
      sinrn = sin(phirn)
      do i=nx,n_p
        tmpx =  cosrn*p(1,i,1)+sinrn*p(2,i,1)
        tmpy = -sinrn*p(1,i,1)+cosrn*p(2,i,1)
        p(1,i,1) = tmpx
        p(2,i,1) = tmpy
      enddo
      do i=1,n_v
        tmpx =  cosrn*v(1,i,1)+sinrn*v(2,i,1)
        tmpy = -sinrn*v(1,i,1)+cosrn*v(2,i,1)
        v(1,i,1) = tmpx
        v(2,i,1) = tmpy
      enddo

      ! avoid singularity for small photon virtuality
      ! for semileptonic processes, when Z/gamma decays hadronically
      SELECT CASE(procid)
      CASE(ZZhadjj, ZZhad, GFZZhad, ZZhadWP, ZZhadWM, ZZZhad, Hjj_ZZhad)
         if (mjj2(v(0,1,1),v(0,2,1)).lt.qsqAmin_had) dps=0.d0
      CASE(WPZhad, WMZhad, WPZhadjj, WMZhadjj)
         if (mjj2(v(0,3,1),v(0,4,1)).lt.qsqAmin_had) dps=0.d0
      CASE(WWZhad)
         if (mjj2(v(0,5,1),v(0,6,1)).lt.qsqAmin_had) dps=0.d0
      END SELECT
c      if (dps.gt.0.d0) print*, sqrt(mjj2(v(0,1,1),v(0,2,1)))
c      if ( sqrt((v(1,1,1)+v(1,2,1))**2 + (v(2,1,1)+v(2,2,1))**2) .lt. 20.0) dps=0.d0 

c Multiply with symmetry factor for identical leptons in the final state
      dps = dps*symmetryfactor

      RETURN
      END
      


