

c*****************************************************************************
c
c    begin subroutine LIPSWWAJ_anomal
c
c*****************************************************************************
c
c       giuseppe bozzi, <giuseppe@particle.uni-karlsruhe.de>
c       Initial version: January 2008
c       Last modified by Michael Rauch: April 2011
c
c phase space generator for the production of 3 bosons (+ 1 jet)
c at hadron colliders
c
c  INPUT:   RD         random number array (between 0 and 1)
c           RN         additional random number which will not be driven by
c                      VEGAS
c
c  OUTPUT:  K1,K2      4-vectors for 2 incoming partons
c           X1,X2      corresponding values of Feynman x
c           V(0:3,5)   4-vectors for the 4 decay leptons, assumed massless
c                      Q(*,1) = V(*,1) + V(*,2)
c                      Q(*,2) = V(*,3) + V(*,4)
c                      V(*,5) = photon
c           Pjet(0:3)  4-vector for the jet, assumed massless
c           W          the weight of the phase space point
c
c
c
c  INPUT INFORMATION on the phase space region is transferred via the
c    common block /BLIPSI/
c
c****************************************************************************

      SUBROUTINE LIPSWWAJ_anomal(N,RD,RN,
     1                   K1,K2,V,Pjet,
     2                   X1,X2,W)

      IMPLICIT NONE

#include "VBFNLO/utilities//global.inc"
c
      INTEGER N,NRANDL
      REAL*8 TPI
      PARAMETER (nrandl=15, TPI=2D0*PI )
      REAL*8 RD(nrandl),RN
      REAL*8 K1(0:3), K2(0:3), V(0:3,6), X1, X2, W, K(0:3)
      REAL*8 Pjet(0:3,N)
      REAL*8 Pjets(0:3,N+1)

      REAL*8 RM2, RMG, RM2MIN, RM2MAX, S, PTC
      REAL*8 RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN
      REAL*8 RM2MAX1, RM2MAX2
      COMMON /TRIBOSBLIPSI/ RM2(3), RMG(3), RM2MIN(3), RM2MAX(3),
     1                RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN,
     2                S, PTC, RM2MAX1, RM2MAX2

      DOUBLE PRECISION CLR, XM2, XMG, B, VE, A
      COMMON /BKOPOU/ CLR(4,5,-1:1), XM2(6), XMG(6),
     1                B(6,6,6), VE(4,5), A(4,5)

C  local variables

      INTEGER I, J, II, JJ, MU, ic

      REAL*8 Y, Y2, PT, PHI
      REAL*8 xmin1, xmax1, delx1, tanx1, xval1
      REAL*8 xmin2, xmax2, delx2, tanx2, xval2
      REAL*8 xmin3, xmax3, delx3, tanx3, xval3
      REAL*8 xminx, xmaxx, delxx, tanxx, xvalx
      REAL*8 xminy, xmaxy, delxy, tanxy, xvaly
      REAL*8 coshy2, sinhy2, temp, test
      REAL*8 y_border, tau, ql(0:4,2), q(0:4,3)  
      REAL*8 pX(0:4), phiX, costhetaX, abs_pX, sinthetaX
      REAL*8 pY(0:4), phiY, costhetaY, abs_pY, sinthetaY
      REAL*8 phi2, costheta2, sintheta2
      REAL*8 phi3, costheta3, sintheta3
      REAL*8 phi4, costheta4, sintheta4
      REAL*8 pTmax, pTmin, pL, y2_border, ET
      REAL*8 abs_p, abs_p2, abs_p3, abs_p4
      REAL*8 C, m2, mw
      REAL*8 RM2MAXV1, RM2MAXV2

      DOUBLE PRECISION xxwidth, xxm, V1mg, V2mg
      DOUBLE PRECISION dotrr

      PARAMETER ( xxwidth = 36d1, xxm = 17d1 )      !broader resonance and smaller mass
      PARAMETER ( V1mg = 33d1, V2mg = 33d1 ) !broader V1 and V2 resonance

      EXTERNAL dotrr

      LOGICAL ldebug
      PARAMETER (ldebug=.false.)

      LOGICAL NONRESONANCE, RESONANCE, TWOBODYDECAY, TWOTOJETSPLUSX, RESONANCE2
      EXTERNAL NONRESONANCE, RESONANCE, TWOBODYDECAY, TWOTOJETSPLUSX, RESONANCE2

c--------------------------------------------------------------------
c---------------------------------------------------------------------

      if (ldebug) print*," enter LIPSVVA "

      W = 1d0             !init of weightfactor

      q(4,3)=0d0

c pX^2 of intermediate particle X -> V1 + V2 out of rd(3)
c split up rd(3) for X resonance and Higgs resonance (for the new anomalous Higgs-contributions)

c      if (.not. Resonance(xxm**2, xxm*xxwidth, rm2minx, rm2maxx,
c     1     rd(3), W, pX(4))) return

       if ( rd(3) .le. 0.2d0 ) then
          if (.not. Resonance(xxm**2, xxm*xxwidth, rm2minx, rm2max1,
     1                            rd(3)/0.2d0, W, pX(4))) return
          W = W/0.2d0
       else if ( rd(3) .le.  0.5d0 ) then 
          if (.not. Resonance(xm2(6), xmg(6), rm2max1, rm2max2,
     1                    (rd(3)-0.2d0)/0.3d0, W, pX(4))) return 
          W = W/0.3d0
       else
          if (.not. Resonance(xxm**2, xxm*xxwidth, rm2max2, rm2maxx,
     1                    (rd(3)-0.5d0)/0.5d0, W, pX(4))) return
          W = W/0.5d0
       endif


c q^2 of V1 is generated via tan mapping out of rd(1)
      rm2maxV1 = pX(4)
      if (.not.Resonance(rm2(1), V1mg, rm2min(1), rm2maxV1,  !broader W resonance
     1     rd(1), W, q(4,1))) RETURN 

c q^2 of V2 is generated via tan mapping out of rd(2)
      rm2maxV2 = pX(4)+q(4,1)-2*sqrt(pX(4)*q(4,1))
      if (.not.Resonance(rm2(2), V2mg, rm2min(2), rm2maxV2,  !broader W resonance
     1     rd(2), W, q(4,2))) RETURN 
      
c-------------------------------------------------------------------
c-------------------------------------------------------------------

c generate 2 -> 2 process for k1 k2 -> Pjet q_Y out of rd(10-12-15)
      if (.not.TwoToJetsPlusX(N+1, rd(10), rn, ecm, pTc, pX(4),
     1        k1(0), k2(0), x1, x2, pX(0), pJets(0,1), W)) return

      do mu=0,3
        do i=1,N
          pjet(mu,i) = pjets(mu,i)
        enddo
        q(mu,3) = pjets(mu,N+1)
      enddo

c generate phase space for decay of X -> V1 + V2 out of rd(4-5)
      if (.not. TwoBodyDecay(rd(4), rd(5), pX(0), pX(4), q(4,1), q(4,2),
     1     q(0,1),q(0,2),W)) return

c generate phase space for decay of V1 -> l1 + l2 out of rd(6-7)
      call TwoBodyDecay0(rd(6),rd(7),q(0,1),q(4,1),
     1     v(0,1),v(0,2),W)
      
c generate phase space for decay of V2 -> l3 + l4 out of rd(8-9)
      call TwoBodyDecay0(rd(8),rd(9),q(0,2),q(4,2),
     1     v(0,3),v(0,4),W)

c set q_A=v(5)
      do mu=0,3
         v(mu,5)=q(mu,3)
         v(mu,6)=0d0
      enddo

c -----------------------------------------------------

      if (ldebug) then
         print*, "--------- new event -----------"
         print*, ""
         print*, "random numbers"
         print*, rn
         print*, rd(1), rd(2), rd(3)
         print*, rd(4), rd(5), rd(6)
         print*, rd(7), rd(8), rd(9)
         print*, rd(10), rd(11), rd(12)
         if (N.eq.1) print*, rd(13), rd(14), rd(15)
         print*, "y     = ", y
         print*, "tau   = ", tau
         print*, "x1    = ", x1
         print*, "x2    = ", x2
         print*, "y2    = ", y2
         print*, "wgt   = ", w
         print*, "---------------------------"

         print*, "--- momenta conservation ? ---"
         do mu=0,3
            test = k1(mu)+k2(mu)-Pjet(mu,1)-q(mu,1)-q(mu,2)-q(mu,3)
            if (test.gt.1E-9) print*, test
            test = q(mu,1)-v(mu,1)-v(mu,2) 
            if (test.gt.1E-9) print*, test
            test = q(mu,2)-v(mu,3)-v(mu,4) 
             if (test.gt.1E-9) print*, test
            test = q(mu,3)-v(mu,5)-v(mu,6) 
             if (test.gt.1E-9) print*, test
            test = k1(mu)+k2(mu)-Pjet(mu,1)-v(mu,1)-v(mu,2)
     &             -v(mu,3)-v(mu,4)-v(mu,5)-v(mu,6) 
             if (test.gt.1E-9) print*, test
         enddo
         print*,"tau*s = (k1+k2)^2"
         test = tau*s - dotrr(k1,k1)-dotrr(k2,k2)-2*dotrr(k1,k2)
         print*, test
         print*,"---tau*s = (q1 + q2 + q3 + pjet)^2 ?---"
         test = tau*s - dotrr(q(0,1),q(0,1)) - dotrr(q(0,2),q(0,2))
     &     - dotrr(q(0,3),q(0,3)) - 2*dotrr(q(0,1),q(0,2))
     &     - 2*dotrr(q(0,1),q(0,3)) - 2*dotrr(q(0,2),q(0,3))
     &     - dotrr(Pjet(0,1),Pjet(0,1)) - 2*dotrr(Pjet(0,1),q(0,1))
     &     - 2*dotrr(Pjet(0,1),q(0,2)) - 2*dotrr(Pjet(0,1),q(0,3))
         print*, test
         print*, "--- mass equal to zero ? ---"
         print*,"p1^2 = ",dotrr(k1(0),k1(0))
         print*,"p2^2 = ",dotrr(k2(0),k2(0))
         print*,"Pjet^2 = ",dotrr(Pjet(0,1),Pjet(0,1))
         print*,"v1^2 = ",dotrr(v(0,1),v(0,1))
         print*,"v2^2 = ",dotrr(v(0,2),v(0,2))
         print*,"v3^2 = ",dotrr(v(0,3),v(0,3))
         print*,"v4^2 = ",dotrr(v(0,4),v(0,4))
         print*,"v5^2 = ",dotrr(v(0,5),v(0,5))
         print*,"v6^2 = ",dotrr(v(0,6),v(0,6))
         print*, "-----------------------"
         read(*,*)
      endif

      return
      end

c*****************************************************************************
c
c    end subroutine LIPSWWAJ_anomal
c
c*****************************************************************************

c*****************************************************************************
c
c    begin subroutine LIPSWWAJ1_anomal
c
c*****************************************************************************
c
c       giuseppe bozzi, <giuseppe@particle.uni-karlsruhe.de>
c       Initial version: January 2008
c       Last modified by Vera Hankele: January 2009
c
c Alternative phase-space for pp --> V1 (--> l1 l2 A) V2
c
c****************************************************************************

      SUBROUTINE LIPSWWAJ1_anomal(N,RD,RN,
     1                    K1,K2,V,Pjet,
     2                    X1,X2,W)

      IMPLICIT NONE

#include "VBFNLO/utilities//global.inc"

      INTEGER N,NRANDL
      REAL*8 TPI
      PARAMETER (nrandl=15, TPI=2D0*PI )
      REAL*8 RD(nrandl),RN
      REAL*8 K1(0:3), K2(0:3), V(0:3,6), X1, X2, W, K(0:3)
      REAL*8 Pjet(0:3)
      REAL*8 RM2, RMG, RM2MIN, RM2MAX, S, PTC
      REAL*8 RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN
      REAL*8 RM2MAX1, RM2MAX2
      COMMON /TRIBOSBLIPSI/ RM2(3), RMG(3), RM2MIN(3), RM2MAX(3),
     1                RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN,
     2                S, PTC, RM2MAX1, RM2MAX2
      DOUBLE PRECISION CLR, XM2, XMG, B, VE, A
      COMMON /BKOPOU/ CLR(4,5,-1:1), XM2(6), XMG(6),
     1                B(6,6,6), VE(4,5), A(4,5)
      INTEGER I, J, II, JJ, MU, ic
      REAL*8 Y, Y2, PT, PHI
      REAL*8 xmin1, xmax1, delx1, tanx1, xval1
      REAL*8 xmin2, xmax2, delx2, tanx2, xval2
      REAL*8 xmin3, xmax3, delx3, tanx3, xval3
      REAL*8 xminx, xmaxx, delxx, tanxx, xvalx
      REAL*8 xminy, xmaxy, delxy, tanxy, xvaly
      REAL*8 coshy2, sinhy2, temp, test
      REAL*8 y_border, tau, ql(0:4,2), q(0:4,3)  
      REAL*8 pX(0:4), phiX, costhetaX, abs_pX, sinthetaX
      REAL*8 pY(0:4), phiY, costhetaY, abs_pY, sinthetaY
      REAL*8 phi2, costheta2, sintheta2
      REAL*8 phi3, costheta3, sintheta3
      REAL*8 phi4, costheta4, sintheta4
      REAL*8 pTmax, pTmin, pL, y2_border, ET
      REAL*8 abs_p, abs_p2, abs_p3, abs_p4
      REAL*8 C, m2, mw

      DOUBLE PRECISION xxwidth, xxm, xywidth, xym, V1mg, V2mg
      DOUBLE PRECISION dotrr

      PARAMETER ( xxwidth = 10d1, xxm = 45d1 )
      PARAMETER ( xywidth = 10d1, xym = 45d1 )
      PARAMETER ( V1mg = 110d1, V2mg = 22d1 ) !broader V1 and V2 resonance

      EXTERNAL dotrr

      LOGICAL ldebug
      PARAMETER (ldebug=.false.)

      LOGICAL NONRESONANCE, RESONANCE, TWOBODYDECAY, RESONANCE2
      EXTERNAL NONRESONANCE, RESONANCE, TWOBODYDECAY, RESONANCE2

c--------------------------------------------------------------------
c---------------------------------------------------------------------

      if (ldebug) print*," enter LIPSWWA1 "

      W = 1d0             !init of weightfactor

      q(4,3)=0d0

c q^2 of V1 is generated via tan mapping out of rd(1)
      if (.not.Resonance(rm2(1), V1mg, rm2min(1), rm2max(1), 
     1     rd(1), W, q(4,1))) RETURN 

c q^2 of V2 is generated via tan mapping out of rd(2)
      if (.not.Resonance(rm2(2), V2mg, rm2min(2), rm2max(2), 
     1     rd(2), W, q(4,2))) RETURN 
      
c-------------------------------------------------------------------
c-------------------------------------------------------------------

      if (N.eq.1) then

c pX^2 of intermediate particle X -> V1 + V2 out of rd(13)
         if (.not. Resonance(xxm**2, xxm*xxwidth, rm2minx, rm2maxx,
     1        rd(13), W, pX(4))) return

c generate 2 -> 2 process for k1 k2 -> Pjet q_X out of rd(3-5)
         call TwoToJetPlusX(rd(3), rn, ecm, pTc, pX(4),
     1        k1(0), k2(0), x1, x2, pX(0), pJet(0), W)
         if (w.eq.0) return

c generate phase space for decay of X -> V1 + V2 out of rd(14), rd(15)
         if (.not. TwoBodyDecay(rd(14), rd(15), pX(0), pX(4), q(4,1), q(4,2), 
     1        q(0,1), q(0,2), W)) return

      else if (N.eq.0) then

c generate 2 -> 2 process for k1 k2 -> q_V1 q_V2 out of rd(3-5)
         call TwoToTwo(rd(3), rn, ecm, q(4,1), q(4,2),
     1        k1(0), k2(0), x1, x2, q(0,1), q(0,2), W)
         pjet(0) = 0.0d0
         pjet(1) = 0.0d0
         pjet(2) = 0.0d0
         pjet(3) = 0.0d0
      endif

c generate phase space for decay of V1 -> l1 + l2 + A out of rd(6-10)
      call ThreeBodyDecay0(rd(6),q(0,1),q(4,1),
     1     v(0,1),v(0,2),q(0,3),W)
      
c generate phase space for decay of V2 -> l3 + l4 out of rd(11-12)
      call TwoBodyDecay0(rd(12),rd(11),q(0,2),q(4,2),
     1     v(0,3),v(0,4),W)

c set q_A=v(5)
      do mu=0,3
         v(mu,5)=q(mu,3)
         v(mu,6)=0d0
      enddo

c -----------------------------------------------------

      if (ldebug) then
         print*, "--------- new event -----------"
         print*, ""
         print*, "random numbers"
         print*, rn
         print*, rd(1), rd(2), rd(3)
         print*, rd(4), rd(5), rd(6)
         print*, rd(7), rd(8), rd(9)
         print*, rd(10), rd(11), rd(12)
         if (N.eq.1) print*, rd(13), rd(14), rd(15)
         print*, "y     = ", y
         print*, "tau   = ", tau
         print*, "x1    = ", x1
         print*, "x2    = ", x2
         print*, "y2    = ", y2
         print*, "wgt   = ", w
         print*, "---------------------------"

         print*, "--- momenta conservation ? ---"
         do mu=0,3
            test = k1(mu)+k2(mu)-Pjet(mu)-q(mu,1)-q(mu,2)
            if (test.gt.1E-9) print*,test
            test = q(mu,1)-v(mu,1)-v(mu,2)-v(mu,5) 
            if (test.gt.1E-9) print*,test
            test = q(mu,2)-v(mu,3)-v(mu,4) 
            if (test.gt.1E-9) print*,test
            test = q(mu,3)-v(mu,5)-v(mu,6) 
            if (test.gt.1E-9) print*,test
            test = k1(mu)+k2(mu)-Pjet(mu)-v(mu,1)-v(mu,2)
     &           -v(mu,3)-v(mu,4)-v(mu,5)-v(mu,6) 
            if (test.gt.1E-9) print*,test
         enddo
         tau = x1*x2
         s = ecm**2
         print*,"tau*s = (k1+k2)^2"
         test = tau*s - dotrr(k1,k1)-dotrr(k2,k2)-2*dotrr(k1,k2)
         print*, test
         print*,"---tau*s = (q1 + q2 + pjet)^2 ?---"
         test = tau*s - dotrr(q(0,1),q(0,1)) - dotrr(q(0,2),q(0,2))
     &     - 2*dotrr(q(0,1),q(0,2))
     &     - dotrr(Pjet(0),Pjet(0)) - 2*dotrr(Pjet(0),q(0,1))
     &     - 2*dotrr(Pjet(0),q(0,2))
         print*, test
         print*, "--- mass equal to zero ? ---"
         print*,"p1^2 = ",dotrr(k1(0),k1(0))
         print*,"p2^2 = ",dotrr(k2(0),k2(0))
         print*,"Pjet^2 = ",dotrr(Pjet(0),Pjet(0))
         print*,"v1^2 = ",dotrr(v(0,1),v(0,1))
         print*,"v2^2 = ",dotrr(v(0,2),v(0,2))
         print*,"v3^2 = ",dotrr(v(0,3),v(0,3))
         print*,"v4^2 = ",dotrr(v(0,4),v(0,4))
         print*,"v5^2 = ",dotrr(v(0,5),v(0,5))
         print*,"v6^2 = ",dotrr(v(0,6),v(0,6))
         print*, "-----------------------"
         read(*,*)

      endif

      return
      end

c*****************************************************************************
c
c    end subroutine LIPSWWAJ1_anomal
c
c*****************************************************************************

c*****************************************************************************
c
c    begin subroutine LIPSWWAJ2_anomal
c
c*****************************************************************************
c
c       giuseppe bozzi, <giuseppe@particle.uni-karlsruhe.de>
c       Initial version: January 2008
c       Last modified by Vera Hankele: January 2009
c
c Alternative phase-space for pp --> V1 V2 (--> l1 l2 A)
c
c****************************************************************************

      SUBROUTINE LIPSWWAJ2_anomal(N,RD,RN,
     1                    K1,K2,V,Pjet,
     2                    X1,X2,W)

      IMPLICIT NONE

#include "VBFNLO/utilities//global.inc"
c
      INTEGER N,NRANDL
      REAL*8 TPI
      PARAMETER (nrandl=15, TPI=2D0*PI )
      REAL*8 RD(nrandl),RN
      REAL*8 K1(0:3), K2(0:3), V(0:3,6), X1, X2, W, K(0:3)
      REAL*8 Pjet(0:3)
      REAL*8 RM2, RMG, RM2MIN, RM2MAX, S, PTC
      REAL*8 RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN
      REAL*8 RM2MAX1, RM2MAX2
      COMMON /TRIBOSBLIPSI/ RM2(3), RMG(3), RM2MIN(3), RM2MAX(3),
     1                RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN,
     2                S, PTC, RM2MAX1, RM2MAX2
      DOUBLE PRECISION CLR, XM2, XMG, B, VE, A
      COMMON /BKOPOU/ CLR(4,5,-1:1), XM2(6), XMG(6),
     1                B(6,6,6), VE(4,5), A(4,5)
      INTEGER I, J, II, JJ, MU, ic
      REAL*8 Y, Y2, PT, PHI
      REAL*8 xmin1, xmax1, delx1, tanx1, xval1
      REAL*8 xmin2, xmax2, delx2, tanx2, xval2
      REAL*8 xmin3, xmax3, delx3, tanx3, xval3
      REAL*8 xminx, xmaxx, delxx, tanxx, xvalx
      REAL*8 xminy, xmaxy, delxy, tanxy, xvaly
      REAL*8 coshy2, sinhy2, temp, test
      REAL*8 y_border, tau, ql(0:4,2), q(0:4,3)  
      REAL*8 pX(0:4), phiX, costhetaX, abs_pX, sinthetaX
      REAL*8 pY(0:4), phiY, costhetaY, abs_pY, sinthetaY
      REAL*8 phi2, costheta2, sintheta2
      REAL*8 phi3, costheta3, sintheta3
      REAL*8 phi4, costheta4, sintheta4
      REAL*8 pTmax, pTmin, pL, y2_border, ET
      REAL*8 abs_p, abs_p2, abs_p3, abs_p4
      REAL*8 C, m2, mw

      DOUBLE PRECISION xxwidth, xxm, xywidth, xym, V1mg, V2mg
      DOUBLE PRECISION dotrr

      PARAMETER ( xxwidth = 10d1, xxm = 45d1 )
      PARAMETER ( xywidth = 10d1, xym = 45d1 )
      PARAMETER ( V1mg = 20d1, V2mg = 120d1 ) !broader V1 and V2 resonance

      EXTERNAL dotrr

      LOGICAL ldebug
      PARAMETER (ldebug=.false.)

      LOGICAL NONRESONANCE, RESONANCE, TWOBODYDECAY, RESONANCE2
      EXTERNAL NONRESONANCE, RESONANCE, TWOBODYDECAY, RESONANCE2

c--------------------------------------------------------------------
c---------------------------------------------------------------------

      if (ldebug) print*," enter LIPSWWA2 "

      W = 1d0             !init of weightfactor

      q(4,3)=0d0

c q^2 of V1 is generated via tan mapping out of rd(1)
      if (.not.Resonance(rm2(1), V1mg, rm2min(1), rm2max(1), 
     1     rd(1), W, q(4,1))) RETURN 

c q^2 of V2 is generated via tan mapping out of rd(2)
      if (.not.Resonance(rm2(2), V2mg, rm2min(2), rm2max(2), 
     1     rd(2), W, q(4,2))) RETURN 
      
c-------------------------------------------------------------------
c-------------------------------------------------------------------

      if (N.eq.1) then

c pX^2 of intermediate particle X -> V1 + V2 out of rd(13)
         if (.not. Resonance(xxm**2, xxm*xxwidth, rm2minx, rm2maxx,
     1        rd(13), W, pX(4))) return

c generate 2 -> 2 process for k1 k2 -> Pjet q_X out of rd(3-5)
         call TwoToJetPlusX(rd(3), rn, ecm, pTc, pX(4),
     1        k1(0), k2(0), x1, x2, pX(0), pJet(0), W)
         if (w.eq.0) return

c generate phase space for decay of X -> V1 + V2 out of rd(14), rd(15)
         if (.not. TwoBodyDecay(rd(14), rd(15), pX(0), pX(4), q(4,1), q(4,2), 
     1        q(0,1), q(0,2), W)) return

      else if (N.eq.0) then

c generate 2 -> 2 process for k1 k2 -> q_V1 q_V2 out of rd(3-5)
         call TwoToTwo(rd(3), rn, ecm, q(4,1), q(4,2),
     1        k1(0), k2(0), x1, x2, q(0,1), q(0,2), W)
         pjet(0) = 0.0d0
         pjet(1) = 0.0d0
         pjet(2) = 0.0d0
         pjet(3) = 0.0d0
      endif

c generate phase space for decay of V1 -> l1 + l2 out of rd(6-7)
      call TwoBodyDecay0(rd(6),rd(7),q(0,1),q(4,1),
     1     v(0,1),v(0,2),W)
      
c generate phase space for decay of V2 -> l3 + l4 + A out of rd(8-12)
      call ThreeBodyDecay0(rd(8),q(0,2),q(4,2),
     1     v(0,3),v(0,4),q(0,3),W)

c set q_A=v(5)
      do mu=0,3
         v(mu,5)=q(mu,3)
         v(mu,6)=0d0
      enddo

c -----------------------------------------------------

      if (ldebug) then
         print*, "--------- new event -----------"
         print*, ""
         print*, "random numbers"
         print*, rn
         print*, rd(1), rd(2), rd(3)
         print*, rd(4), rd(5), rd(6)
         print*, rd(7), rd(8), rd(9)
         print*, rd(10), rd(11), rd(12)
         if (N.eq.1) print*, rd(13), rd(14), rd(15)
         print*, "y     = ", y
         print*, "tau   = ", tau
         print*, "x1    = ", x1
         print*, "x2    = ", x2
         print*, "y2    = ", y2
         print*, "wgt   = ", w
         print*, "---------------------------"

         print*, "--- momenta conservation ? ---"
         do mu=0,3
            test = k1(mu)+k2(mu)-Pjet(mu)-q(mu,1)-q(mu,2)
            if (test.gt.1E-9) print*, test
            test = q(mu,1)-v(mu,1)-v(mu,2) 
            if (test.gt.1E-9) print*, test
            test = q(mu,2)-v(mu,3)-v(mu,4)-v(mu,5) 
             if (test.gt.1E-9) print*, test
            test = q(mu,3)-v(mu,5)-v(mu,6) 
             if (test.gt.1E-9) print*, test
            test = k1(mu)+k2(mu)-Pjet(mu)-v(mu,1)-v(mu,2)
     &             -v(mu,3)-v(mu,4)-v(mu,5)-v(mu,6) 
             if (test.gt.1E-9) print*, test
         enddo
         print*,"tau*s = (k1+k2)^2"
         test = tau*s - dotrr(k1,k1)-dotrr(k2,k2)-2*dotrr(k1,k2)
         print*, test
         print*,"---tau*s = (q1 + q2 + pjet)^2 ?---"
         test = tau*s - dotrr(q(0,1),q(0,1)) - dotrr(q(0,2),q(0,2))
     &     - 2*dotrr(q(0,1),q(0,2))
     &     - dotrr(Pjet(0),Pjet(0)) - 2*dotrr(Pjet(0),q(0,1))
     &     - 2*dotrr(Pjet(0),q(0,2))
         print*, test
         print*, "--- mass equal to zero ? ---"
         print*,"p1^2 = ",dotrr(k1(0),k1(0))
         print*,"p2^2 = ",dotrr(k2(0),k2(0))
         print*,"Pjet^2 = ",dotrr(Pjet(0),Pjet(0))
         print*,"v1^2 = ",dotrr(v(0,1),v(0,1))
         print*,"v2^2 = ",dotrr(v(0,2),v(0,2))
         print*,"v3^2 = ",dotrr(v(0,3),v(0,3))
         print*,"v4^2 = ",dotrr(v(0,4),v(0,4))
         print*,"v5^2 = ",dotrr(v(0,5),v(0,5))
         print*,"v6^2 = ",dotrr(v(0,6),v(0,6))
         print*, "-----------------------"
         read(*,*)

      endif

      return
      end

c*****************************************************************************
c
c    end subroutine LIPSWWAJ2_anomal
c
c*****************************************************************************
