 
****************************************************************************
*   This file contains functions and subroutines for the cuts.
c***************************************************************************
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     FUNCTION Cuts(jets, njets, leptons, nl,invisible, ninv, photons, nphotons, L)
c     FUNCTION Basic_Cuts(partons, npartons, jets, njets, leptons, nl, invisible, ninv, photons, nphotons)
c     FUNCTION VBF_Cuts(jets, njets, leptons, nl, photons, nphotons)
c     FUNCTION JetVeto_Cuts(jets, njets, leptons, nl, photons, nphotons)
c     SUBROUTINE DefineLeptons(v, Lmax, leptons, n_leptons)
c     SUBROUTINE DefinePhotons(v, Lmax, photons, n_photons)
c     SUBROUTINE DefineInvisible(v, Lmax, invisible, n_invisible)
c
c***************************************************************************
*   Last modified: 02.02.2011
****************************************************************************
c*************************************************************************
      SUBROUTINE InitCuts
c*************************************************************************
c     Reads in the values for the cuts.
c*************************************************************************
      implicit none
#include "global.inc"
#include "process.inc"
#include "cuts.inc"
#include "lha.inc"

      integer LSP, i
      common /LSP_def/ LSP

      SELECT CASE(procID)
      CASE(WPhadWMjj, WPWMhadjj, WPhadZjj, WPZhadjj, WMhadZjj, WMZhadjj,
     &    ZZhadjj, WPhadWPjj, WMhadWMjj, WPhadWMZ, WPWMhadZ, WWZhad, ZZhadWP,
     &    ZZWPhad, ZZhadWM, ZZWMhad, WPhadWMWP, WPWMhadWP, WMhadWPWM, WMWPhadWM,
     &    ZZZhad, WPhadWMlep, WPlepWMhad, GFWPhadWMlep, GFWPlepWMhad, ZZhad,
     &    GFZZhad, WPhadZ, WPZhad, WMhadZ, WMZhad,
     &    Hjj_WPhadWM, Hjj_WPWMhad, Hjj_ZZhad)
         semileptonic = .true.
      case default
         semileptonic = .false.
      END SELECT

      SELECT CASE(procID)
      CASE(WPZhadjj, WMZhadjj, ZZhadjj, WWZhad, ZZhadWP, ZZhadWM, 
     &     ZZZhad, ZZhad, GFZZhad, WPZhad, WMZhad, Hjj_ZZhad)
         zdecay_had = .true.           ! true for all processes with hadronically decaying Z 
      case default
         zdecay_had = .false.
      end select

      call loadfile("cuts.dat",.false.)

c jet cuts

      do i=1,max_jets
         ptmin_def_default(i)=20.d0
      enddo
      call read_realList("PT_JET_MIN",max_jets,ptmin_def,ptmin_def_default,.true.)
      call read_real("Y_P_MAX",ymax_part,5d0)
      call read_Real("PGENKTJET",jetgenktp,1d0) ! 1:kT, 0:C/A, -1:anti-kT 
      call read_real("Y_JET_MAX",ymax_def,4.5d0)
      call read_real("RJJ_MIN",rjjmin,0.8d0)

c leptons cuts
      call read_real("RLL_MIN",rll_min,0.4d0)
      call read_real("RLL_MAX",rll_max,50.0d0)
      call read_real("RJL_MIN",rjl_min,0.6d0)
      call read_real("Y_L_MAX", ymax_l,2.5d0)
      call read_real("PT_L_MIN", ptlmin, 10d0)
*      call read_real("PT_L_MAX", ptlmax, 14000d0)
      call read_real("MLL_MIN",mllmin,15d0)
      call read_real("MLL_MAX",mllmax,14000d0)
*      call read_real("PT_IN_MIN", ptinmin, 10d0)
      call read_real("PTMISS_MIN", ptmiss_min, 0d0)

c photon cuts
      call read_real("RGG_MIN",rgg_min,0.6d0)
      call read_real("RGG_MAX",rgg_max,50.0d0)
      call read_real("RJG_MIN",rjg_min,0.6d0)
      call read_real("RLG_MIN",rlg_min,0.6d0)
      call read_real("Y_G_MAX",ymax_g,1.5d0)
      call read_real("PT_G_MIN",ptgmin,20d0)
*      call read_real("PT_G_MAX",ptgmax,14000d0)
      call read_real("PHISOLCUT",isol_phopart,0.7d0)
      call read_real("EFISOLCUT",eff_phopart,1d0)
      call read_real("MLG_MIN",mlAmin,0d0)
      call read_real("MLG_MAX",mlAmax,14000d0)
*      call read_real("MT_MIN",MTmin,0d0)
*      call read_real("MT_MAX",MTmax,14000d0)

c VBF cuts - only applied to tagging jets
      call read_real("ETAJJ_MIN",etajjmin, 0d0)
      call read_logical("YSIGN",ysign,.false.)
      call read_logical("LRAPIDGAP",lrapidgap,.false.)
      call read_real("DELY_JL",dely_jlep, 0d0)
      call read_logical("GRAPIDGAP",grapidgap,.false.)
      call read_real("DELY_JG",dely_jphot, 0d0)
      call read_real("MDIJ_MIN",mdijmin, 600d0)
      call read_real("MDIJ_MAX",mdijmax,14000d0)
      ! call read_real("VBFQSQAMIN",qsqAmin,4d0)   ! set in global.inc

c Jet veto cuts
      call read_logical("JVETO",lveto,.false.)
      call read_real("DELY_JVETO",dely_jveto,0d0)
      call read_real("YMAX_VETO",ymax_veto,4.5d0)
      call read_real("PTMIN_VETO",ptmin_veto,10d0)


c semileptonic decay cuts
      if (semileptonic) then
        call read_logical("SINGLE_DECAYJET",semilep_fatjet,.false.)
        if (LOplusjet .and. semilep_fatjet) then
           print*, "CLASH: SINGLE_DECAYJET = true in cuts.dat and LOPROCESS_PLUS_JET = true"
           print*, "in vbfnlo.dat is not allowed to be set at at the same time!"
           STOP
        endif
        if ((procid.eq.WPhadWMjj .or. procid.eq.WPWMhadjj .or. procid.eq.ZZhadjj) .and. semilep_fatjet) then
           print*, "CLASH: SINGLE_DECAYJET = true is temporarily not available"
           print*, "for VBF processes!"
           STOP
        endif
        call read_int("DEF_TAGJET",def_tagjet,1)
        if (def_tagjet.le.0 .or. def_tagjet.gt.3) then
           print*, "ERROR:"
           print*, "For DEF_TAGJET only values 1, 2 or 3 are valid!"
           STOP
        endif
        if (def_tagjet.eq.2) ysign = .true.
        if(ptmin_tag_1.lt.ptmin_tag_2) then
           print*,"Warning: PTMIN_TAG_1 is lower than PTMIN_TAG_2!"
        endif
        if (DEF_TAGJET.eq.2) then
          call read_real("ETA_CENTRAL",eta_central,2d0)
        else
          eta_central=-1d0
        endif
        call read_real("PTMIN_TAG_1",ptmin_tag_1,20d0)
        call read_real("PTMIN_TAG_2",ptmin_tag_2,20d0)
        call read_logical("HARD_CENTRAL",hard_central,.false.)
        call read_real("PTMIN_CENTRAL",ptmin_central,20d0)
        call read_real("QSQAMIN_ZDEC",qsqAmin_had,100d0)
      else
        semilep_fatjet = .false.
        def_tagjet = 1
      endif



      ! minimal number of jets: set in process.F to njet_min
      ! min_jets is used as minimum number of jets within cut routine and defineJets
      ! can be different from njet_min for semileptonic decays
      ! for hadronic z-decays: additional photon cut in phasespace.F
      min_jets=njet_min 
      if (semilep_fatjet) min_jets = min_jets - 1

      call closefile

      do i=1,max_jets-1
         if(ptmin_def(i).lt.ptmin_def(i+1)) then
            print*,"Warning: PT_JET_MIN is not ordered; higher values"
            print*,"should come first"
         endif
      enddo

c print information on cuts
      print *,""
      print *,"         information on cut parameters"
      print *,"-----------------------------------------------"
      print *,""
      ! print max(1, njet_min) jets for LO, njet_min+1 for NLO
      if (donlo) then
        print "(A,5(G9.4e1))"," minimum jet pt        (PT_JET_MIN):    ",(ptmin_def(i), i=1,njet_min+1)
      else
        print "(A,5(G9.4e1))"," minimum jet pt        (PT_JET_MIN):    ",(ptmin_def(i), i=1,max(1,njet_min))
      endif
      print *,"maximum jet rapidity                 (Y_JET_MAX): ",ymax_def
      print *,"maximum parton pseudorapidity          (Y_P_MAX): ",ymax_part
      print *,"minimum jet-jet R-separation           (RJJ_MIN): ",rjjmin
      print *,"minimum number of jets                (NJET_MIN): ",min_jets
      print *,"exponent of generalised kT algorithm (PGENKTJET): ",jetgenktp
      print *,"minimum lepton-lepton R-separation     (RLL_MIN): ",rll_min
      print *,"maximum lepton-lepton R-separation     (RLL_MAX): ",rll_max
      print *,"minimum jet-lepton R-separation        (RJL_MIN): ",rjl_min
      print *,"minimum jet-photon R-separation        (RJG_MIN): ",rjg_min
      print *,"minimum lepton-photon R-separation     (RLG_MIN): ",rlg_min   
      print *,"minimum photon-photon R-separation     (RGG_MIN): ",rgg_min
      print *,"maximum photon-photon R-separation     (RGG_MAX): ",rgg_max
      print *,"maximum lepton rapidity                (Y_L_MAX): ",ymax_l
      print *,"minimum lepton pt                     (PT_L_MIN): ",ptlmin
*      print *,"maximum lepton pt                     (PT_L_MAX): ",ptlmax
*      print *,"minimum invisible pt                  (PT_IN_MIN): ",ptinmin
      print *,"minimum invariant dilepton mass        (MLL_MIN): ",mllmin
      print *,"maximum invariant dilepton mass        (MLL_MAX): ",mllmax
      print *,"maximum photon rapidity                (Y_G_MAX): ",ymax_g
      print *,"minimum photon pt                     (PT_G_MIN): ",ptgmin  
*      print *,"maximum photon pt                     (PT_G_MAX): ",ptgmax 
      print *,"photon isolation cut                 (PHISOLCUT): ",isol_phopart
      print *,"efficiency of photon isolation cut   (EFISOLCUT): ",eff_phopart
      print *,"minimum invariant lepton-photon mass   (MLG_MIN): ",mlAmin
      print *,"maximum invariant lepton-photon mass   (MLG_MAX): ",mlAmax
      print *,"minimal missing transverse momentum (PTMISS_MIN): ",ptmiss_min
*      print *,"minimum Wgamma transverse mass          (MT_MIN): ",MTmin
*      print *,"maximum Wgamma transverse mass          (MT_MAX): ",MTmax
      print *,"minimum jet rapidity separation      (ETAJJ_MIN): ",etajjmin
      print *,"tagging jets in opposite hemispheres     (YSIGN):   ",ysign
      print *,"leptons fall inside rapidity gap     (LRAPIDGAP):   ",lrapidgap
      print *,"min leptons y-dist from tagging jets   (DELY_JL): ",dely_jlep
      print *,"photons fall inside rapidity gap     (GRAPIDGAP):   ",grapidgap
      print *,"min photons y-dist from tagging jets   (DELY_JG): ",dely_jphot
      print *,"dijet min mass cut on tag jets        (MDIJ_MIN): ",mdijmin
      print *,"dijet max mass cut on tag jets        (MDIJ_MAX): ",mdijmax
      print *,"veto criteria for jets                   (JVETO):   ",lveto
      print *,"minimum veto-tag y-dist             (DELY_JVETO): ",dely_jveto
      print *,"maximum |y| for veto jet             (YMAX_VETO): ",ymax_veto
      print *,"minimum pT for veto jet             (PTMIN_VETO): ",ptmin_veto
      if (semileptonic) then
         print *,"definition of tagging jets          (DEF_TAGJET): ",def_tagjet
         if (DEF_TAGJET.eq.2) print *,'max |y| for "central region"       (ETA_CENTRAL): ',eta_central
         print *,'minimal pt for harder tagging jet  (PTMIN_TAG_1): ',ptmin_tag_1
         print *,'minimal pt for softer tagging jet  (PTMIN_TAG_2): ',ptmin_tag_2
         if (hard_central) print *,'min pt for jet in central region (PTMIN_CENTRAL): ',ptmin_central
      endif
      print *,""

      if ((qsqAmin.gt.0d0 .and. photoncut_vbf) .or. 
     &    (qsqAmin_had.gt.0d0 .and. zdecay_had .and. procID.ne.Hjj_ZZhad)) then
         print *,""
         print *,"         information on technical cuts"
         print *,"-----------------------------------------------"
         print*," "
      endif
      if (qsqAmin.gt.0d0 .and. photoncut_vbf) then
         print 171," Photon exchange with virtuality Q^2 < ",qsqAmin,
     1   " GeV^2 is eliminated. "
         print*," Recover by calculating  q gamma ---> q g V (V)   etc."
         print*," "
      endif
      if (qsqAmin_had.gt.0d0 .and. zdecay_had .and. procID.ne.Hjj_ZZhad) then
         print*," "
         print 171," Hadronically decaying Z bosons / photons with Q^2 < ",qsqAmin_had,
     1   " GeV^2 are neglected. "
      endif
 171  format(a,f10.1,a)

c      if (model.eq.2) then
c        print *,""
c        print *,"definition of Lightest Supersymmetric Particle "
c        print *,"-----------------------------------------------"
c        print *,""
c        call loadfile("susy.dat",.true.)
c        call read_Int("LSP",LSP,9999)
c        call closefile
c      else
c        LSP=9999
c      endif
** Lightest susy particle - no effect yet!
      LSP = 9999
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION Cuts(jets, njets, leptons, nl, invisible, ninv, 
     -     photons, nphotons, L)
c*************************************************************************
c     This is the general cut routine. It is intended to be the interface
c     to any basic, specialized or user-defined cut routines. 
c*************************************************************************
c     INPUT 
c     jets      : the array with the jet information as it is returned by the 
c                 jetdefinition routine.
c     njets     : number of visible jets
c     leptons   : the array with the lepton information as it is returned by the 
c                 defineleptons routine.
c     nl        : number of visible leptons
c     invisible : the array with the invisible particle information as it is 
c                  returned by the  defineinvisible routine.
c     ninv      : number of invisible particles
c     photons   : the array with the photon information as it is returned by the 
c                 definephotons routine.
c     nphotons  : number of visible photons
c     L         : id for momentum set in jets, leptons, invisible, photons
c
c     OUTPUT
c     Cuts      : .true. or .false. depending on whether the event passes the cuts.
c*************************************************************************

      IMPLICIT NONE
#include "global.inc"
#include "process.inc"
#include "cuts.inc"

c     input variables
      real*8 partons(0:7,max_jets,max_kin)
      real*8 jets(0:7,max_jets,max_kin) 
      real*8 leptons(0:8,max_v,max_kin), invisible(0:8,max_v,max_kin)
      real*8 photons(0:7,max_v,max_kin)   
      integer npartons(max_kin), ninv(max_kin)
      integer L, njets(max_kin), nl(max_kin), nphotons(max_kin)

      integer jetc

c     external functions
      logical Basic_Cuts, VBF_Cuts, JetVeto_Cuts
      external Basic_Cuts, VBF_Cuts, JetVeto_Cuts

      common/partons/partons,npartons

      ! initialize jetc, tag1, tag2, so it's also set for non-vbf processes
      jetc = 0
      tag1 = 0
      tag2 = 0

c call of function Basic_Cuts for all processes
      Cuts = Basic_Cuts( partons(0,1,L), npartons(L), jets(0,1,L), 
     1                   njets(L), leptons(0,1,L), nl(L), 
     2                   invisible(0,1,L), ninv(L),
     3                   photons(0,1,L), nphotons(L) )


c call of vbf_cuts only for vbf processes
      SELECT CASE(procID)
      CASE(Hjj, Hjj_AA, AAjj, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW,
     &     Hjj_ZZ_ll, Hjj_ZZ_lnu, Ajj, Zjj_l, Zjj_nu, WPjj, WMjj,
     &     WPWMjj, WPWPjj, WMWMjj, ZZjj_ll, ZZjj_lnu, WPZjj, WMZjj, HAjj, 
     &     HAjj_AA, HAjj_mu, HAjj_tau, HAjj_bbar, HAjj_WW, 
     &     HAjj_ZZ_ll, HAjj_ZZ_lnu,
     &     GFHjj, GFHjj_AA, GFHjj_mu, GFHjj_tau, GFHjj_bbar,  
     &     GFHjj_WW, GFHjj_ZZ_ll, GFHjj_ZZ_lnu, 
     &     Hjjj, Hjjj_AA, Hjjj_mu,
     &     Hjjj_tau, Hjjj_bbar, Hjjj_WW, Hjjj_ZZ_ll, Hjjj_ZZ_lnu,
     &     WPhadWMjj, WPWMhadjj, WPhadZjj, WPZhadjj, WMhadZjj, WMZhadjj,
     &     ZZhadjj, WPhadWPjj, WMhadWMjj,
     &     Hjj_WPhadWM, Hjj_WPWMhad, Hjj_ZZhad)
         if (Cuts) then
            Cuts = VBF_Cuts( jets(0,1,L), njets(L), leptons(0,1,L),
     1                         nl(L), photons(0,1,L), nphotons(L), jetc)
         endif
      END SELECT


c call of jet veto for all processes
c has to be called after VBF_Cuts as this routine returns the tagging jet
c IDs for the VBF processes
      if (Cuts) then
         Cuts = JetVeto_Cuts( jets(0,1,L), njets(L), leptons(0,1,L),
     1                          nl(L), photons(0,1,L), nphotons(L), jetc)
      endif


      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION BASIC_CUTS(partons, npartons, jets, njets,
     1                  leptons, nl, invisible, ninv, photons, nphotons)
c*************************************************************************
c     This routine offers basic cuts for all processes.
c*************************************************************************
      IMPLICIT NONE
#include "global.inc"
#include "cuts.inc"

c     input variables
      real*8 partons(0:7,max_jets)
      real*8 jets(0:7,max_jets) 
      real*8 leptons(0:8,max_v)
      real*8 invisible(0:8,max_v)      
      real*8 photons(0:7,max_v)
      integer npartons, njets, nl, nphotons, ninv
      real*8 pt,y,phi
      integer i, j,  k, kk, kmax, mu, nveto, maxnjet
      real*8 ylepmax,ylepmin,m2,mjj2,mll,mlA,yvetomax,yvetomin
      real*8 yphotmax, yphotmin,MT2
      real*8 mom(0:3)

      logical pass_cuts

      double precision RSEPS
      external RSEPS
c     photon isolation stuff
      double precision ptsum,factor
      double precision delta(npartons)
      double precision dlist(npartons,nphotons)
      double precision ptlist(npartons,nphotons)
      double precision flist(npartons,nphotons)
      double precision spt(npartons,nphotons)

  
      pass_cuts = .true.



c   JET CUTS
c=================

c...require minimum min_jets jets in final state
      pass_cuts = pass_cuts .and. (njets.ge.min_jets)

c...jet-jet r-separation
      do i=1,njets-1
         do j = i+1, njets
            pass_cuts=pass_cuts.and.
     &      ( RSEPS(jets(6,i),jets(7,i),jets(6,j),jets(7,j)).gt.rjjmin)
         enddo
      enddo



c   LEPTON CUTS
c=================
c...maximum lepton rapidity and minimum lepton pt
      do i=1, nl
         pass_cuts=pass_cuts.and. 
     &   ( (abs(leptons(6,i)).lt.ymax_l).and.(leptons(5,i).gt.ptlmin))
*     &   .and.(leptons(5,i).le.ptlmax) )  
      enddo

c...cut on minimum pt of invisible particles
c$$$      do i=1, ninv
c$$$         pass_cuts=pass_cuts.and. 
c$$$     &   ( (invisible(5,i).gt.ptinmin) )
c$$$     
c$$$      enddo

c...lepton-lepton r-separation min and max 
      do i=1, nl-1
         do j = i+1, nl
            pass_cuts=pass_cuts.and.
     -           ( RSEPS(leptons(6,i),leptons(7,i),leptons(6,j),
     -           leptons(7,j)).gt.rll_min .and. RSEPS(leptons(6,i),
     -           leptons(7,i),leptons(6,j),leptons(7,j)).lt.rll_max)
         enddo
      enddo

c... cut on invariant dilepton mass
      do i = 1,nl-1
         do j = i+1,nl
            if(leptons(8,i)*leptons(8,j).lt.0) then !opposite sign leptons only
               mll = mjj2(leptons(0,i),leptons(0,j))
               pass_cuts=pass_cuts.and.(mll.gt.mllmin**2).and.
     -              (mll.lt.mllmax**2)
            endif
         enddo
      enddo



c   PHOTON CUTS
c=================
c...maximum photon rapidity and minimum photon pt
      do i=1, nphotons
         pass_cuts=pass_cuts.and. 
     &   ( (abs(photons(6,i)).lt.ymax_g).and.(photons(5,i).gt.ptgmin))
c...maximum photon pt
*     &   .and.(photons(5,i).le.ptgmax) )     
      enddo

c...photon-photon r-separation min and max
      do i=1, nphotons-1
         do j = i+1, nphotons
            pass_cuts=pass_cuts.and.
     -           ( RSEPS(photons(6,i),photons(7,i),photons(6,j),
     -           photons(7,j)).gt.rgg_min .and. RSEPS(photons(6,i),
     -           photons(7,i),photons(6,j),photons(7,j)).lt.rgg_max)
         enddo
      enddo

c...transverse mass from hep-ph/9305314

c     transverse mass only useful for Wgamma production!!!
c$$$      if(nphotons.eq.1)then
c$$$               MT2 = (sqrt(mjj2(leptons(0,1),photons(0,1))+ 
c$$$     &                (photons(1,1)+leptons(1,1))**2 +(photons(2,1)+leptons(2,1))**2)+
c$$$     &                 invisible(5,1))**2     
c$$$     &                -(photons(1,1)+leptons(1,1)+invisible(1,1))**2
c$$$     &               -(photons(2,1)+leptons(2,1)+invisible(2,1))**2
c$$$!               print*,MT2, mla, MTmin, MTmax 
c$$$               pass_cuts=pass_cuts.and.(MT2.gt.MTmin**2).and.(MT2.le.MTmax**2)
c$$$      endif

! c...NEW photon isolation cut according to Frixione,PLB 429(1998),369
       do i=1, nphotons
         k=0
         do j=1, npartons
            delta(j)=RSEPS(photons(6,i),photons(7,i),partons(6,j),
     -           partons(7,j))
            if (delta(j).lt.isol_phopart) then
               k=k+1
               dlist(k,i)=delta(j)
               ptlist(k,i)=partons(5,j)
               flist(k,i)=(1.-dcos(dlist(k,i)))/(1.-dcos(isol_phopart))
            endif
         enddo
         kmax=k
         do k=1,kmax
            spt(k,i)=0d0
            do kk=1,kmax
               if (dlist(kk,i).le.dlist(k,i)) then
                  spt(k,i)=spt(k,i)+ptlist(kk,i)
               endif
            enddo
            pass_cuts=pass_cuts
     &           .and.(spt(k,i).le.eff_phopart*photons(5,i)*flist(k,i))
         enddo
       enddo



c   ADDITIONAL CUTS
c=================================
c...jet-lepton r-separation
      do i=1, nl
         do j=1, njets
            pass_cuts=pass_cuts.and.
     &           ( RSEPS(leptons(6,i),leptons(7,i),jets(6,j),
     -           jets(7,j)).gt.rjl_min )
         enddo
      enddo

c...jet-photon r-separation
      do i=1, nphotons
         do j=1, njets
            pass_cuts=pass_cuts.and.
     &           ( RSEPS(photons(6,i),photons(7,i),jets(6,j),
     -           jets(7,j)).gt.rjg_min )
         enddo
      enddo

c...lepton-photon r-separation
      do i=1, nphotons
         do j=1, nl
            pass_cuts=pass_cuts.and.
     &           ( RSEPS(photons(6,i),photons(7,i),leptons(6,j),
     -           leptons(7,j)).gt.rlg_min )
         enddo
      enddo

c...minimal invariant lepton+photon mass for all combinations of charged leptons and photons
      do i = 1,nl
         do j = 1,nphotons
               mlA = mjj2(leptons(0,i),photons(0,j))
               pass_cuts=pass_cuts.and.(mlA.gt.mlAmin**2).and.
     &             (mlA.le.mlAmax**2)
         enddo
      enddo

c...minimal missing transverse momentum
      do j = 0,3
         mom(j) = 0d0
         do i = 1,nl
            mom(j) = mom(j) + leptons(j,i)
         enddo
         do i = 1,nphotons
            mom(j) = mom(j) + photons(j,i)
         enddo
         do i = 1,njets
            mom(j) = mom(j) + jets(j,i)
         enddo
      enddo
      pass_cuts = pass_cuts .and. (sqrt(mom(1)**2+mom(2)**2).ge.ptmiss_min)



      BASIC_CUTS = pass_cuts
      
      RETURN
      END



c*************************************************************************
      LOGICAL FUNCTION VBF_Cuts(jets, njets, leptons, nl, photons, 
     -     nphotons, jetc)
c*************************************************************************
c     This routine offers vbf specific cuts, that is cuts for processes 
c     with 2 or more jets.
c     jetc returns the number of the additional hard central jet if requested, else 0
c*************************************************************************
      IMPLICIT NONE
#include "global.inc"
#include "cuts.inc"

c     input variables
      real*8 jets(0:7,max_jets) 
      integer njets, nl, nphotons
      real*8 leptons(0:8,max_v)
      real*8 photons(0:7,max_v)
      real*8 pt,y,phi
      integer i, j, mu, nveto
      real*8 ylepmax,ylepmin,m2,mjj2,mll,yvetomax,yvetomin
      real*8 yphotmax, yphotmin

      integer tempjet, jetc

      logical pass_cuts

      double precision RSEPS
      external RSEPS

      logical Basic_Cuts
      external Basic_cuts

      
      pass_cuts = .true.

      ! define tagging jets. Usually it's the two hardest jets, 
      ! except for semileptonic decay processes
      if (semileptonic) then
         select case (def_tagjet)
         case (1)   ! largest pt
            tag1 = 1
            tag2 = 2
         case (2)   ! largest pt outside central region, opposite hemisphere
            tag1 = 0
            tag2 = 0
            do i = 1,njets
               if (jets(6,i).gt.eta_central) then
                  tag1 = i
                  exit
               endif
            enddo
            do i = 1,njets
               if (jets(6,i).lt.(-eta_central)) then
                  tag2 = i
                  exit
               endif
            enddo
            if (tag1*tag2.eq.0) then
               VBF_cuts = .false.
               return
            endif
            if (jets(5,tag2).gt.jets(5,tag1)) then
               tempjet = tag1
               tag1 = tag2
               tag2 = tempjet
            endif
         case (3)   ! largest eta separation, pt_tag1 > pt_tag2
            tag1 = 1
            tag2 = 1
            do i = 2,njets
               if (jets(6,i).gt.jets(6,tag1)) tag1 = i
               if (jets(6,i).lt.jets(6,tag2)) tag2 = i
            enddo
            if (tag1.eq.1 .and. tag2.eq.1) then
               tag1 = 1
               tag2 = 2
            endif
            if (jets(5,tag2).gt.jets(5,tag1)) then
               tempjet = tag1
               tag1 = tag2
               tag2 = tempjet
            endif
         end select
      else
         tag1 = 1
         tag2 = 2
      endif

c...jet-jet rapidity separation
      if (njets.ge.2) then
        pass_cuts=pass_cuts.and.
     &        (abs(jets(6,tag1)-jets(6,tag2)).gt.etajjmin)
      endif

c...jets #1 and #2 must have opposite sign rapidity
      if (ysign) then
         if (njets.ge.2) then
            pass_cuts=pass_cuts.and. ( (jets(6,tag1)*jets(6,tag2)).le.0d0 )
         endif
      endif

c...leptons fall inside rapidity gap
      if(lrapidgap) then
         if (njets.ge.2) then
            ylepmax = max(jets(6,tag1),jets(6,tag2)) - dely_jlep
            ylepmin = min(jets(6,tag1),jets(6,tag2)) + dely_jlep
            do i=1,nl
               pass_cuts=pass_cuts.and.(leptons(6,i).lt.ylepmax.and.
     &              leptons(6,i).gt.ylepmin)      
            enddo
         endif
      endif

c...photons fall inside rapidity gap
      if(grapidgap) then
         if (njets.ge.2) then
            yphotmax = max(jets(6,tag1),jets(6,tag2)) - dely_jphot
            yphotmin = min(jets(6,tag1),jets(6,tag2)) + dely_jphot
            do i=1,nphotons
               pass_cuts=pass_cuts.and.(photons(6,i).lt.yphotmax.and.
     &              photons(6,i).gt.yphotmin)      
            enddo
         endif
      endif

c...  tagging jets invariant dijet mass cut
      if (njets.ge.2) then
         m2=mjj2(jets(0,tag1),jets(0,tag2))
         pass_cuts=pass_cuts.and.(m2.ge.mdijmin**2.and.m2.le.mdijmax**2)
      endif

c...extra vbf cuts for semileptonic processes
      if (semileptonic) then

c...require hard central jet
         jetc=0
         if (hard_central) then
            if (DEF_TAGJET.eq.2) then
               do i = 1,njets
                  if ( abs(jets(6,i)).le.eta_central .and. jets(5,i).ge.ptmin_central
     &                .and. i.ne.tag1 .and. i.ne.tag2 ) then
                     jetc = i
                     exit
                  endif
               enddo
            else
               do i = 1,njets
                  if ( jets(6,i).le.max(jets(6,tag1),jets(6,tag2))
     &                .and. jets(6,i).ge.min(jets(6,tag1),jets(6,tag2))
     &                .and. jets(5,i).ge.ptmin_central
     &                .and. i.ne.tag1 .and. i.ne.tag2 ) then
                     jetc = i
                     exit
                  endif
               enddo
            endif
            pass_cuts=pass_cuts.and.(jetc.gt.0)
         endif

c...minimal pt for tagging jets
         pass_cuts=pass_cuts.and.(jets(5,tag1).gt.ptmin_tag_1)
         pass_cuts=pass_cuts.and.(jets(5,tag2).gt.ptmin_tag_2)

      endif

      VBF_cuts = pass_cuts
      
      RETURN
      END



c*************************************************************************
      LOGICAL FUNCTION JetVeto_Cuts(jets, njets, leptons, nl, photons, 
     -     nphotons, jetc)
c*************************************************************************
c     This routine offers jet veto cuts, that is cuts on additional jets
c     apart from the standard number of the LO process.
c*************************************************************************
      IMPLICIT NONE
#include "global.inc"
#include "process.inc"
#include "cuts.inc"

c     input variables
      integer jetc
      real*8 jets(0:7,max_jets) 
      integer njets, nl, nphotons
      real*8 leptons(0:8,max_v)
      real*8 photons(0:7,max_v)
      real*8 pt,y,phi
      integer i, j, mu, nveto, maxnjet
      real*8 ylepmax,ylepmin,m2,mjj2,mll,yvetomax,yvetomin
      real*8 yphotmax, yphotmin

      logical pass_cuts
      
      pass_cuts = .true.

C... veto jet cut      
      nveto=0
      if(lveto) then
         SELECT CASE(procID)

c vbf processes have maximum two jets 
         CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW,
     &        Hjj_ZZ_ll, Hjj_ZZ_lnu, Ajj, Zjj_l, Zjj_nu, WPjj, WMjj,
     &        WPWMjj, WPWPjj, WMWMjj, ZZjj_ll, ZZjj_lnu, WPZjj, WMZjj, HAjj, 
     &        HAjj_AA, HAjj_mu, HAjj_tau, HAjj_bbar, HAjj_WW, 
     &        HAjj_ZZ_ll, HAjj_ZZ_lnu,
     &        GFHjj, GFHjj_AA, GFHjj_mu, GFHjj_tau, GFHjj_bbar,
     &        GFHjj_WW, GFHjj_ZZ_ll, GFHjj_ZZ_lnu, 
     &        Hjjj, Hjjj_AA, Hjjj_mu,
     &        Hjjj_tau, Hjjj_bbar, Hjjj_WW, Hjjj_ZZ_ll, Hjjj_ZZ_lnu,
     &        WPhadWMjj, WPWMhadjj, WPhadZjj, WPZhadjj, WMhadZjj, WMZhadjj,
     &        ZZhadjj, WPhadWPjj, WMhadWMjj,
     &        Hjj_WPhadWM, Hjj_WPWMhad, Hjj_ZZhad)

            yvetomax = max(jets(6,tag1),jets(6,tag2)) - dely_jveto
            yvetomin = min(jets(6,tag1),jets(6,tag2)) + dely_jveto

            if(njets.gt.min_jets)then
               do i=1,njets
                  if(jets(6,i).le.yvetomax.and.
     &               jets(6,i).ge.yvetomin.and.
     &               abs(jets(6,i)).le.ymax_veto.and.
     &               jets(5,i).gt.ptmin_veto.and.
     &               i.ne.tag1 .and. i.ne.tag2 .and. i.ne.jetc) 
     &                  nveto=nveto+1            
               enddo
               if (hard_central) nveto = nveto + 1
               nveto = nveto - min_jets + 2
            endif

c for all other processes just veto the additional jet
         CASE DEFAULT

            maxnjet = min_jets

            if(njets.gt.min_jets)then
               do i=maxnjet+1,njets
                  if(abs(jets(6,i)).le.ymax_veto.and.
     &               jets(5,i).gt.ptmin_veto) 
     &                  nveto=nveto+1            
               enddo
            endif

         END SELECT

         pass_cuts=pass_cuts.and.(nveto.le.0)

      endif
      
      
      JetVeto_cuts = pass_cuts
      
      RETURN
      END



c*************************************************************************
      SUBROUTINE DefineLeptons(v, Lmax, leptons, n_leptons)
c*************************************************************************
c     Similar to "DefineJets" in jetdefinition.f this routine extracts
c     the leptons out of "v" into "leptons". Only e, mu, tau particles are
c     taken. In addition pt, y and phi information is calculated and stored.
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 v(0:3,max_v,max_kin)
      real*8 leptons(0:8,max_v,max_kin)
      integer n_leptons(max_kin), Lmax
      real*8 pt,y,phi
      integer i, l, mu

      do l=1, Lmax

         do i=1, max_v
            do mu=0,8
               leptons(mu,i,l) = 0d0
            enddo
         enddo

         n_leptons(l)=0
c     get leptons out of decay_particles     
         do i=1, n_v
            if ( (abs(particle_IDs(i)).eq.11) ! check if it is a lepton
     &           .or.(abs(particle_IDs(i)).eq.13)
     &           .or.(abs(particle_IDS(i)).eq.15) ! taus!!
     &           ) then
               n_leptons(l)=n_leptons(l)+1
               do mu = 0,3      ! transfer to "leptons"
                  leptons(mu,n_leptons(l),l) = v(mu,i,l)
               enddo
               leptons(8,n_leptons(l),l) = particle_IDs(i)
            endif 
         enddo
c     calculate rapidity, pt, phi-angle for the leptons
         do i=1, n_leptons(l)
            call lego( leptons(0,i,l), pt, y, phi )
            leptons(4,i,l) = 0.0 ! mass  
            leptons(5,i,l) = pt  ! transverse momentum  
            leptons(6,i,l) = y   ! rapidity 
            leptons(7,i,l) = phi ! azimuthal angle
         end do
         
      enddo

      RETURN
      END
     

c*************************************************************************
      SUBROUTINE DefinePhotons(v, Lmax, photons, n_photons)
c*************************************************************************
c     Similar to "DefineJets" in jetdefinition.f this routine extracts
c     the photons out of "v" into "photons". In addition pt, y and phi 
c     information is calculated and stored.
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 v(0:3,max_v,max_kin)
      real*8 photons(0:7,max_v,max_kin)
      integer n_photons(max_kin), Lmax
      real*8 pt,y,phi
      integer i, l, mu

      do l=1, Lmax

         do i=1, max_v
            do mu=0,7
               photons(mu,i,l) = 0d0
            enddo
         enddo

         n_photons(l)=0
c     get photons out of decay_particles     
         do i=1, n_v
            if ( (abs(particle_IDs(i)).eq.22) ! check if it is a photon
     &           ) then
               n_photons(l)=n_photons(l)+1
               do mu = 0,3      ! transfer to "photons"
                  photons(mu,n_photons(l),l) = v(mu,i,l)
               enddo
            endif 
         enddo
c     calculate rapidity, pt, phi-angle for the leptons
         do i=1, n_photons(l)
            call lego(photons(0,i,l), pt, y, phi )
            photons(4,i,l) = 0.0 ! mass  
            photons(5,i,l) = pt ! transverse momentum  
            photons(6,i,l) = y  ! rapidity 
            photons(7,i,l) = phi ! azimuthal angle
         end do
         
      enddo

      RETURN
      END
     
c*************************************************************************
      SUBROUTINE DefineInvisible(v, Lmax, invisible, n_invisible)
c*************************************************************************
c     Similar to "DefineJets" in jetdefinition.f this routine extracts
c     the invisible particles out of "v" into "invisible". In addition pt,
c     y and phi information is calculated and stored.
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 v(0:3,max_v,max_kin)
      real*8 invisible(0:8,max_v,max_kin)
      integer n_invisible(max_kin), Lmax
      integer LSP
      common /LSP_def/ LSP
      real*8 pt,y,phi
      integer i, l, mu

      do l=1, Lmax
         do i=1, max_v
            do mu=0,8
               invisible(mu,i,l) = 0d0
            enddo
         enddo

         n_invisible(l)=0
c     get invisible particles out of decay_particles     
         do i=1, n_v
            if ( (abs(particle_IDs(i)).eq.12) ! check if it is a neutrino
     &           .or.(abs(particle_IDs(i)).eq.14)
     &           .or.(abs(particle_IDS(i)).eq.16)
     &           .or.(abs(particle_IDS(i)).eq.LSP)) then
               n_invisible(l)=n_invisible(l)+1
               do mu = 0,3      ! transfer to "invisible"
                  invisible(mu,n_invisible(l),l) = v(mu,i,l)
               enddo
               invisible(8,n_invisible(l),l) = particle_IDs(i)
            endif 
         enddo
	 

c     calculate rapidity, pt, phi-angle for the invisible particles
         do i=1, n_invisible(l)
            call lego( invisible(0,i,l), pt, y, phi )
            invisible(4,i,l) = 0.0 ! mass  
            invisible(5,i,l) = pt  ! transverse momentum  
            invisible(6,i,l) = y   ! rapidity 
            invisible(7,i,l) = phi ! azimuthal angle
         end do
      enddo    
      

      RETURN
      END
 
