****************************************************************************
*  This is an include-file of the VBFNLO main program.
*  It contains declarations of global variables.
c***************************************************************************
c  Author(s) : G. Klaemke
c***************************************************************************
*  Last modified: 15.09.2006
****************************************************************************

c the process identification number
      integer procID

c the number of iterations of the integration
      integer N_iterations1, N_iterations2
c the number of points/shots in the last iteration = 2**N_points
c an array is used because there are several loops (LO, virtual contrib., real emissions)
c -1       : real emissions loop
c  0       : LO loop
c  1,2,... : the various virtual contributions
      integer N_points(-1:5)	

c the center of mass energy of the collider
      double precision ecm	

c phasespace dimensionality - this is process dependend!!

      integer*4 PS_dimension

c length of the monaco random number array. (Must always be larger than PS_dimension.)

      integer max_PS_dim
      parameter (max_PS_dim = 30)

c the maximum number of 4-momenta stored in 'partons' 
c here, the partons are the incoming quarks/gluons + the 2 tagging jets
c + the real emission (for NLO only)

      integer max_p
      parameter (max_p=6)

c the maximum number of additional particles like a higgs boson, or any decay products.
c additional/decay particles means: everything which is not a parton involved in 
c the basic QCD 2->2(+1)+X process.

      integer max_v
      parameter (max_v=10)

c number of internal x parameters
      integer nx
      parameter( nx=2 )	

c the maximum number of jets 
      integer max_jets
      parameter (max_jets=max_p+max_v-nx)

c max number of momentum configurations for real emission processes
      integer max_kin
      parameter (max_kin=25)

c max number of different phase spaces
      integer max_ps
      parameter (max_ps = 5)

c maximal number of resonances that can be put into one random number 
c (see MultiResonances in phasespace/ps_tools.F)
      integer max_res, max_multires
      parameter (max_res=20, max_multires=10)

c actual number of entries in partons
      integer n_p

c actual number of entries in decay_particles
      integer n_v

c actual number of kinematics
      integer n_kin, n_qed


c     The particle identification code according to the Monte Carlo particle numbering scheme.
c     reminder :  e- =  11, nue    =  12, mu- =  13, numu    =  14, tau- =  15, nutau  =  16,
c                 e+ = -11, nuebar = -12, mu+ = -13, numubar = -14, tau+ = -15, nutaub = -16,
c             photon = 22,      Z0 = 23,   W+ = 24,       h0 = 25,    A0 = 36,      H+ = 37 
c     in addition, we identify a general parton (quark/gluon) with id=93
      integer particle_IDs(max_v)
c mass of the corresponding particle
      double precision particle_mass(max_v)
		
c file names for input and output of monaco grid information
      character*50 gridname1(1:max_ps), gridname2(1:max_ps), 
     &             gridname3(1:max_ps), gridname4(1:max_ps)

c     proton/antiproton beams
      integer sign1, sign2

c     is NLO implemented for the process?
      logical hasNLO, hasFLoops

c     switches for born/virtual contribution/real emission
      logical doBorn, doVirtuals, doEmissions, doFLoops, inFLoops


** Flags, switches, parameters for running in SUSY and with ew
** corrections
* switches: 
*  loopSqr_switch = whether |M_{loop}|^{2} is calculated
*  ewcor_switch = whether ew corns are included
*  FeynH_switch =  whether FeynHiggs is used
*  slha_switch = whether a SLHA file is used
      logical loopSqr_switch, ewcor_switch, FeynH_switch, slha_switch
      logical loopSqr_switchIN
* flags: 
*  model = SM (1) and MSSM (2)
*  Higgs type  = SM (0), h0 (1), HH (2), A0 (3)
*  ewapprox = which approximations the user has selected for the ew corns
*             (0 = no approximations)
*  sector = which particles are included in the ew corns  
*  ewscheme = method by which LO em coupling is calculated       
      integer model, HiggsType, ewapprox, sector, ewscheme, floops, MHloops
     

c     number of loops for different virtual contributions
      integer NLO_Loops

c     number of loops for different phase space generators
      integer PS_Loops, Loops_sub_LO,Loops_sub_NLO,sub_number
c      commom /subprocnu/sub_number

c put everything in global common blocks: split into integer, logical etc.
      common /cglobalD/ ecm
      common /cglobalI/ ewapprox, model, HiggsType, ewscheme, floops
      common /cglobalI/ sector, MHloops, procID
      common /cglobalI/ N_iterations1, N_iterations2, N_points, n_p, n_v
      common /cglobalI/ sign1, sign2, NLO_Loops, PS_dimension, particle_IDs
      common /cglobalI/ PS_Loops, sub_number,Loops_sub_LO,Loops_sub_NLO
      common /cglobalL/ FeynH_switch, slha_switch, ewcor_switch 
      common /cglobalL/ loopSqr_switch, loopSqr_switchIN
      common /cglobalL/ hasNLO, hasFLoops
      common /cglobalL/ doBorn, doVirtuals, doEmissions, doFLoops
      common /cglobalL/ inFLoops
      common /cglobalC/ gridname1, gridname2, gridname3, gridname4

	
      common /cglobal2/ n_kin, n_qed

      integer N_iterationsv
      common/N_varia/N_iterationsv


c distribute finite terms for Born*virtual+real emission = c*|Born|**2 + ....
c with c = -9/2 - pi^2/3 = creal + cvirt
      double precision creal, cvirt, pi, pi2o3p9h
      parameter (pi = 3.141592653589793d0, pi2o3p9h = pi**2/3d0+9d0/2d0)
      parameter (cvirt = pi**2/3d0-7d0, creal = -pi2o3p9h-cvirt) ! for carlo

c and for pp -> H jjj.
      double precision crealH3j, cvirtH3j
      parameter (cvirtH3j = pi**2/3d0-8d0, crealH3j = 2d0*pi**2/3d0-13d0/2d0)

c and for pp -> VV (V)
      double precision crealVV, cvirtVV, pi2o3m4
      parameter (pi2o3m4=2d0*(pi**2/3d0-4d0))
      parameter (cvirtVV = 4/3d0*pi**2-8d0,crealVV = pi2o3m4-cvirtVV)

c fix number of external quark flavors to sum over; for CC processes both 
c up- and down-type quarks of the highest generation need to be allowed
c (i.e. the largest even number less than or equal to nfl is used)
c For vbf processes that involve both CC and NC channels, by switching vbfNFLb
c  to .true. and nfl=5 external bottom quarks will be included in the NCs but
c  not in the CCs
c For other processes nfl=5 allows external bottom quarks if the process has
c  no CC components
c Set nfl=4 to have no external bottom quarks anywhere
      integer nfl, nflVBF
      parameter (nfl=5)
      logical vbfNFLb
      parameter (vbfNFLb=.false.)
      common /vbfNFL/ nflVBF


c qsqAmin is the minimum virtuality for t-channel photon exchange. 
c Subamplitudes will be set to zero if photon virtuality falls below
c qsqAmin
      double precision qsqAmin
      parameter (qsqAmin=4d0)
!      common /photondivVBF/qsqAmin


c qsqAmin_had is the minimum virtuality for a hadronically decaying photon
c in the semileptonic processes.
c Amplitude will be set to zero if Q^2_gamma < qsqAmin_had
      double precision qsqAmin_had
      common /photondiv/qsqAmin_had
c      parameter (qsqAmin_had=100d0)


c flag for warnings in matrixelement routines
      logical lwarn
      parameter (lwarn = .false.)

      integer igauge
      parameter (igauge=2)
c igauge allows to select different ways to insure EW gauge invariance
c in the face of finite W and Z width effects for EW W and Z production:
c  
c   igauge = 1       overall factor scheme
c            2       complex mass scheme, m**2-->m**2-i*m*Gamma
c                    in all propagators
c            3       for ibos=2 only: naive Z Breit Wigner
c           -1       width = 0, for testing purposes only

      logical with_anom 	
      common /anom_switch/ with_anom
      logical with_anomHiggs
      common /anomHiggs/ with_anomHiggs
      logical with_kk 	
      common /kk_switch/ with_kk
      logical with_spin2
      common /spin2_switch/ with_spin2 


c instabilities for pentagons
      integer pentcountm, pentcount2m,pentcountps(max_ps),pentcount2ps(max_ps)
      common /instpent/ pentcountm, pentcount2m, pentcountps, pentcount2ps
      save /instpent/


c instabilities for hexagons
      integer hexcountm, hexcount2m,hexcountps(max_ps),hexcount2ps(max_ps)
      common /insthex/ hexcountm, hexcount2m, hexcountps, hexcount2ps
      save /insthex/


c instabilities for boxes
      integer boxcountm, boxcount2m,boxcountps(max_ps),boxcount2ps(max_ps)
      common /instbox/ boxcountm, boxcount2m, boxcountps, boxcount2ps
      save /instbox/


c instabilities for boxes
      integer box2countm, box2count2m,box2countps(max_ps),box2count2ps(max_ps)
      common /instbox2/ box2countm, box2count2m, box2countps, box2count2ps
      save /instbox2/

c Additional Variables for Instabilities
      Real*8 lo_r(max_ps),virt_b(max_ps),box_v(max_ps),pent_v(max_ps)
      Real*8 hex_v(max_ps),Re_nlo(max_ps),box2_v(max_ps)
      Real*8 floop_r(max_ps)
      Real*8 e_lo_r(max_ps),e_virt_b(max_ps),e_box_v(max_ps)
      Real*8 e_pent_v(max_ps),e_hex_v(max_ps),e_Re_nlo(max_ps)
      Real*8 e_box2_v(max_ps)
      Real*8 e_floop_r(max_ps)

       common/instabilities_v/lo_r,virt_b,box_v,pent_v,hex_v,Re_nlo,box2_v,
     & floop_r,
     & e_lo_r,e_virt_b,e_box_v,e_pent_v,e_hex_v,e_Re_nlo,e_box2_v,
     & e_floop_r

c default values from GramDets (GGFLO)
      real*8 GRAMCTO0,GRAMDTO0

      common/GRAMTO/GRAMCTO0,GRAMDTO0

c output file for cross section data
      character*50 XSECfile
      common/xsecfile/ XSECfile
