****************************************************************************
* This file contains the subroutines for the histogram handling.
* There is an internal representation of the histograms, which is filled
* by calls of "FillHist". In the end of the program, this data is transferred
* to other histogram representations (e.g. hbook, top-drawer).
* Each histogram has an ID, which is used for the filling.
c***************************************************************************
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     SUBROUTINE InitHistograms
c     SUBROUTINE HistogramEvent(weight,dsig,dps, rand, p, x, v, jets, n_jets, 
c                                   NLO)
c     SUBROUTINE FillHist(ID, value, weight)
c     SUBROUTINE FillHist2d(ID, X, Y, weight, NLO)
c     SUBROUTINE CreateHist(ID, title, bins, min, max)
c     SUBROUTINE Create2dHist(ID, title, xlabel, ylabel, xbins, ybins, xmin,  
c                                  xmax, ymin, ymax)
c     SUBROUTINE WriteHistograms
c     SUBROUTINE WriteRootHists
c     SUBROUTINE WritePawHists
c     SUBROUTINE WriteTopHists
c     SUBROUTINE WriteGnuHists
c     SUBROUTINE WriteDataHists
c     SUBROUTINE SelectFile
c
c***************************************************************************
c
c      to add a histogram
c     --------------------
c     1: put a "CreateHist" in "InitHistograms". Use unique IDs for each 
c         histogram
c     2: put a proper "FillHist" in "HistogramEvents"
c
c***************************************************************************

c*************************************************************************
      SUBROUTINE InitHistograms
c*************************************************************************
c     Set up histograms
c*************************************************************************

      implicit none

#include "hist.inc"
#include "global.inc"
#include "lha.inc"

      integer id, j, k
      real*8 hwid
      character*5 ext

      NLOoffset = HISTMAX/3
      NLOoffset2d = HIST2dMAX/3
c
c reset histogram contents to zero     
c
      do id = 1,HISTMAX
         nbin(ID) = 100
         hlo(ID) = 0d0
         hup(ID) = 1d0
         htitles(ID) = ""
         do j = 1,BINMAX
            sum(id,j) = 0d0
            summ(id,j) = 0d0
            square(id,j) = 0d0
            count(id,j) = 0
         enddo
         hstored(ID) = .false.
      enddo

c
c do the same for the two-dimensional histograms
c
      do id = 1, HIST2dMAX
         nbinX(ID) = 100
         nbinY(ID) = 100
         hloX(ID) = 0d0
         hloY(ID) = 0d0
         hupX(ID) = 1d0
         hupY(ID) = 1d0
         h2dtitles(ID) = ""
         xlabels(ID) = ""
         ylabels(ID) = ""
         do j = 1, BINMAXX
            do k = 1, BINMAXY
               sum2d(id,j,k) = 0d0
               summ2d(id,j,k) = 0d0
               square2d(id,j,k) = 0d0
               count2d(id,j,k) = 0
            enddo
         enddo
         h2dstored(ID) = .false.
      enddo

****************************
****************************
**                        **
**    HISTOGRAM SETUP     **
**                        **
****************************
****************************

c     loadfile has to be called before all CreateHist calls
      print *,""
      print *,"         information on histogram config"
      print *,"-----------------------------------------------"
      print *,""
      call loadfile("histograms.dat",.false.)
      call read_logical("SMEARING",smearbins,.false.)
      write(*,*) "SMEARING           =     ", smearbins
      if (smearbins) then
        call read_Real("SMEAR_VALUE",smearamount,0.49d0)
        if (smearamount.gt.0.499 .or. smearamount.lt.1d-6) then
           print*, "SMEAR_VALUE is outside of allowed range"//
     &          " (10^-6 to 0.499)!"
           print*, "Default is used."
           smearamount = 0.49
        endif
        write(*,*) "SMEAR_VALUE        =     ", smearamount
      endif
      call read_logical("CALC_ERROR_GNUPLOT",calcerrorGnuplot,.false.)
      write(*,*) "CALC_ERROR_GNUPLOT =     ", calcerrorGnuplot
      call read_logical("CALC_ERROR_1D",calcerror1d,.true.)
      write(*,*) "CALC_ERROR_1D      =     ", calcerror1d
      call read_logical("CALC_ERROR_2D",calcerror2d,.false.)
      write(*,*) "CALC_ERROR_2D      =     ", calcerror2d
      print *,""

*     call CreateHist(ID, title, #bins, min, max)
      call CreateHist(1, "dS/dpT_j (fb/GeV)",100,0d0,250d0)    !pT of tagging jets
      call CreateHist(2, "dS/dpTmax_j (fb/GeV)",100,0d0,500d0) !pT of tagging jet with higher pT
      call CreateHist(3, "dS/dpTmin_j (fb/GeV)",100,0d0,200d0) !pT of tagging jet with lower pT
      call CreateHist(4, "dS/deta_j (fb)",100,-5d0,5d0)        !y of tagging jets
      call CreateHist(5, "dS/deta_j1 (fb)",100,-5d0,5d0)       !y of tagging jet with higher pT
      call CreateHist(6, "dS/deta_j2 (fb)",100,-5d0,5d0)       !y of tagging jet with lower pT
      call CreateHist(7, "dS/dpTmax_l (fb/GeV)",100,0d0,500d0)     !pT_max of all leptons
      call CreateHist(8, "dS/dpTmin_l (fb/GeV)",100,0d0,200d0)     !pT_min of all leptons
      call CreateHist(9, "dS/d|eta|max_l (fb)",100,0d0,5d0)  !|eta|_max of all leptons
      call CreateHist(10, "dS/d|eta|min_l (fb)",100,0d0,5d0)  !|eta|_min of all leptons
      call CreateHist(11, "dS/dPhi_jj (fb)", 100, -180d0, 180d0)  !azimuthal angle of tagging jets

c     example for 2d-hist
c     (ID, title, x-axis label, y-axis label, x #bins, y #bins, xmin, xmax, ymin, ymax)
      call Create2dHist(1, "d2S/deta_jj dm_jj (fb/GeV)", 
     -     "eta_jj", "m_jj", 25, 25, 0d0, 6d0, 0d0, 800d0)


* If you do NOT wish a histogram to be saved to file, switch it off here.
c      hstored( 1) = .false.  
c      hstored( 2) = .false.
c      hstored( 3) = .false.
c      hstored( 4) = .false.
c      hstored( 5) = .false.
c      hstored( 6) = .false.
c      hstored( 7) = .false.
c      hstored( 8) = .false.
c      hstored( 9) = .false.
c      hstored(10) = .false.
c      hstored(11) = .false.
c      h2dstored( 1) = .false.

c     closefile has to be called after all CreateHist calls
      call closefile

      print *,""
      print *,"-----------------------------------------------"
      print *,""
      print *,""
      print *,""

***********************************************************************
c     
c select also the NLO histograms to be stored in a file
c
      if ((hasNLO.and.doNLO) .or. ewcor_switch) then
         do id = 1, NLOoffset
            hstored(id+NLOoffset)=hstored(id)
            hstored(id+2*NLOoffset)=hstored(id)
         enddo

         do id = 1, NLOoffset2d
            h2dstored(id+NLOoffset2d)=h2dstored(id)
            h2dstored(id+2*NLOoffset2d)=h2dstored(id)
         enddo
       endif
c   
c calculate inverse bin widths for each histogram
c
      do id = 1, histmax
         hwid = hup(id) - hlo(id)
         
         if ( nbin(id).gt.0 .and. hwid.ne.0d0 ) then
            rwid(id) = real( nbin(id) ) / hwid
            dbin(id) = 1/rwid(id)
         else
            dbin(id) = 0d0
            rwid(id) = 0d0
         end if
      end do
c
c the same for the 2d histograms
c
      do id = 1, hist2dmax
         hwid = hupX(id) - hloX(id)
         if ( nbinX(id).gt.0 .and. hwid.ne.0d0 ) then
            rwidX(id) = real( nbinX(id) ) / hwid
            dbinX(id) = 1/rwidX(id)
         else
            dbinX(id) = 0d0
            rwidX(id) = 0d0
         endif
         hwid = hupY(id) - hloY(id)
         if ( nbinY(id).gt.0 .and. hwid.ne.0d0 ) then
            rwidY(id) = real( nbinY(id) ) / hwid
            dbinY(id) = 1/rwidY(id)
         else
            dbinY(id) = 0d0
            rwidY(id) = 0d0
         endif         
      end do
c
c read input file from "vbfnlo.dat"
c
      call loadfile("vbfnlo.dat",.false.)
      call read_string("XSECFILE",XSECfile,"xsection")
      call read_logical("ROOT",writeROOT,.false.)
*      call read_logical("HBOOK",writePAW,.false.)
      call read_logical("TOP",writeTOP,.false.)
      call read_logical("GNU",writeGNU,.true.)
      call read_logical("DATA",writeDATA,.true.)
      call read_logical("REPLACE",replaceFiles,.true.)
      call read_string("ROOTFILE",rootfile,"histograms")
*      call read_string("PAWFILE",PAWfile,"histograms")
      call read_string("TOPFILE",TOPfile,"histograms")
      call read_string("GNUFILE",GNUfile,"histograms")
      call read_string("DATAFILE",DATAfile,"histograms")
      call closefile
      if (.not.replacefiles) then
         ext = ".out"
         call selectfile(xsecfile,ext)
         ext = ".root"
         call selectfile(rootfile,ext)
*         ext = ".paw"
*         call selectfile(pawfile,ext)
         ext = ".top"
         call selectfile(topfile,ext)
         ext = ".gp"
         call selectfile(gnufile,ext)
         ext = ".dir"
         call selectfile(datafile,ext)
      else
        xsecfile= xsecfile(1:len_trim(xsecfile))//".out"
        rootfile= rootfile(1:len_trim(rootfile))//".root"
*        pawfile= pawfile(1:index(pawfile," ")-1)//".paw"
        topfile= topfile(1:len_trim(topfile))//".top"        
        gnufile= gnufile(1:len_trim(gnufile))//".gp"        
        datafile= datafile(1:len_trim(datafile))//".dir"        
      endif
#ifndef WITH_ROOT
      if (writeROOT) then
        write(*,*)'Sorry, ROOT output not included.'
        write(*,*)'Please reconfigure with --with-root or'//
     &       ' switch off ROOT histogram output!'
        writeROOT = .false.
        stop
      endif
#endif

c
c initialize variables for real emission error estimation
c
      call InitRealHist(1)

#ifdef WITH_ROOT
      if (writeRoot) then 
        call openrootsession(rootfile)
        call InitRootHists()
      endif
#endif

      RETURN
      END


c*************************************************************************
      SUBROUTINE HistogramEvent(weight,dsig,dps, rand, p, x, v,
     &                           jets, n_jets, leptons, nl,
     &                           invisible,ninv, photons, nphotons, NLO)
c*************************************************************************
c     Fill histograms
c RECALL: jets/leptons/invisible labelling (j,i)                           
c             i = jet/lepton label number (1->max_jets/max_v)              
c             j = 0-3 = 4-momentum                                         
c                     = 4   = mass                                         
c                     = 5   = pT                                           
c                     = 6   = rapidity y                                   
c                     = 7   = azimuthal angle phi                          
c                     = 8   = (leptons only) particle ID                   
c*************************************************************************

      implicit none

#include "global.inc"
#include "hist.inc"
#include "cuts.inc"

      real*8 weight, dsig, dps
      integer NLO 
      real*8 p(0:3,max_p), v(0:3,max_v), x(nx),inv(0:3)
      real*8 rand(max_PS_dim)
      real*8 jets(0:7,max_jets), leptons(0:8,max_v), photons(0:7,max_v)
      real*8 invisible(0:8,max_v)
      integer n_jets, nl, nphotons,i,ninv
      real*8 pTmaxlep, pTminlep, ymaxlep, yminlep

      real*8 dw
      real*8 deltaR, deltaY, deltaPhi, mjj2, mjj,minv

      double precision RSEPS
      
      external RSEPS
      external mjj2

* variables for real emission error estimation

#include "hist_real.inc"

      if (NLO.lt.-1) return

c bookkeeping for real emission histograms
      if (nlo .ge. 1) then
        do i = NLOoffset+1,2*NLOoffset
           HIST_CALLS(i)=0
        enddo
        do i = NLOoffset2d+1,2*NLOoffset2d
           HIST_CALLS_2D(i)=0
        enddo
      endif

      if (dsig.eq.0d0) return


** W-W-jj is obtained from W+W+jj:  Momenta should be flipped
      if (procID .eq. 260) call momswitch(p,v,jets,leptons,invisible)


      dw = dsig * weight
c      print*, dsig, weight, jets(5,1)

**********************
* pT of tagging jets *
**********************
      if (tag1.gt.0 .and. tag2.gt.0) then
         call FillHist( 1, jets(5,tag1), dw*0.5d0, NLO)
         call FillHist( 1, jets(5,tag2), dw*0.5d0, NLO)
         call FillHist( 2, jets(5,tag1), dw, NLO)
         call FillHist( 3, jets(5,tag2), dw, NLO)
      else
         if (n_jets.ge.1) then
            call FillHist( 1, jets(5,1), dw*0.5d0, NLO)
            call FillHist( 2, jets(5,1), dw, NLO)
         endif
         if (n_jets.ge.2) then   
            call FillHist( 1, jets(5,2), dw*0.5d0, NLO)
            call FillHist( 3, jets(5,2), dw, NLO)
         end if
      endif

***********************
* eta of tagging jets *
***********************
      if (tag1.gt.0 .and. tag2.gt.0) then
         call FillHist( 4, jets(6,tag1), dw*0.5d0, NLO)
         call FillHist( 4, jets(6,tag2), dw*0.5d0, NLO)
         call FillHist( 5, jets(6,tag1), dw, NLO)
         call FillHist( 6, jets(6,tag2), dw, NLO)
      else
         if (n_jets.ge.1) then
            call FillHist( 4, jets(6,1), dw*0.5d0, NLO)
            call FillHist( 5, jets(6,1), dw, NLO)
         endif
         if (n_jets.ge.2) then   
            call FillHist( 4, jets(6,2), dw*0.5d0, NLO)
            call FillHist( 6, jets(6,2), dw, NLO)
         end if
      endif

**********************
* pT of leptons      *
**********************
      pTmaxlep = 0
      do i = 1, nl
         pTmaxlep = max(pTmaxlep,leptons(5,i))
      enddo
      pTminlep = 1.0d6
      do i = 1, nl
         pTminlep = min(pTminlep,leptons(5,i))
      enddo
      if (nl.gt.0) then  
         call FillHist( 7, pTmaxlep, dw, NLO)
         call FillHist( 8, pTminlep, dw, NLO)
      endif

*************************
* |eta| of leptons      *
*************************
      ymaxlep = 0
      do i = 1, nl
         ymaxlep = max(ymaxlep,abs(leptons(6,i)))
      enddo
      yminlep = 1.0d6
      do i = 1, nl
         yminlep = min(yminlep,abs(leptons(6,i)))
      enddo
      if (nl.gt.0) then  
         call FillHist( 9, ymaxlep, dw, NLO)
         call FillHist(10, yminlep, dw, NLO)
      endif

******************
*  Delta Phi_jj  *
******************
      if (n_jets.ge.2) then
         if (tag1.gt.0 .and. tag2.gt.0) then
            if (jets(6,tag1).gt.jets(6,tag2)) then
               deltaPhi = jets(7,tag1) - jets(7,tag2)
            else
               deltaPhi = jets(7,tag2) - jets(7,tag1)
            endif
         else
            if (jets(6,1).gt.jets(6,2)) then
               deltaPhi = jets(7,1) - jets(7,2)
            else
               deltaPhi = jets(7,2) - jets(7,1)
            endif
         endif
         if (deltaPhi.gt.Pi) then
            deltaPhi = deltaPhi - 2d0*pi
         else if (deltaPhi.lt.-Pi) then
            deltaPhi = deltaPhi + 2d0*pi
         endif
         deltaPhi = deltaPhi * 180d0 / Pi
         call FillHist(11, deltaPhi, dw, NLO)
      endif

****************************
* example for 2d-histogram *
****************************
      if (n_jets.ge.2) then
         deltay=abs(jets(6,1)-jets(6,2))
         mjj=sqrt(mjj2(jets(0,1),jets(0,2)))
         call FillHist2d( 1,deltay,mjj, dw, NLO)
      endif

#ifdef WITH_ROOT
      if (writeRoot) then 
        call FillRootHists(dw,jets,n_jets,leptons,nl,
     &         invisible,ninv,photons,nphotons,NLO)
      endif
#endif

      RETURN
      END


c*************************************************************************
      SUBROUTINE FillHist(ID, value, weight, NLO)
c*************************************************************************
c     Fill value in histogram. 
c     Smear over 2 bins if value is within fraction drb of bin boundary
c*************************************************************************

      implicit none

#include "hist.inc"
#include "global.inc"

      Integer ID, NLO
      real*8 value, weight
      Integer hist_in, BIN, binl, binh
      real*8 X,Y 
      real*8 xbin, yl, yh
      real*8 drb, tdrbi, rat

c variables for real emission error estimation

#include "hist_real.inc"

      integer i
      integer mem, mem_L, mem_H

      if (value .ne. value) return

      if (smearbins) then
        drb   = smearamount
        tdrbi = 0.5d0/drb
      else
        drb   = 0.0d0
        tdrbi = 1.0d0
      endif

      hist_in = ID
      if (NLO.ne.0) hist_in=hist_in+NLOoffset
      x = value
      y = weight*rwid(hist_in)

      if (.not.hstored(HIST_in)) return
      
      xbin = (X-HLO(hist_in)) * RWID(hist_in)
      if (xbin.gt.drb) then
         bin = min( INT(xbin), nbin(hist_in) ) 
         if ( (xbin-bin).lt.drb ) then ! x in low end of bin
            binl = bin+1
            binh = bin+2
            rat = (xbin-bin)*tdrbi + 0.5
            yh = y*rat
            yl = y-yh
            bin = 0
         elseif( (xbin-bin).lt.(1.0-drb) ) then ! x in main part of bin
            bin = bin+2
C            elseif( (xbin-bin).lt.1.0 ) then   ! x in high part of bin
         elseif( (xbin-bin).lt.1.0.and.xbin.lt.nbin(hist_in) ) then ! x in high part of bin
            binl = bin+2
            binh = bin+3
            rat = (xbin-bin-1.0)*tdrbi + 0.5
            yh = y*rat
            yl = y-yh
            bin = 0
         else                   ! x more than drp above upper bin
            bin = nbin(hist_in)+2
         endif
      elseif(xbin.lt.-drb) then
         bin = 1
      else                      ! xbin in [-drb,drb]
         binl = 1
         binh = 2
         rat = xbin * tdrbi + 0.5 ! in [0,1]
         yh = y*rat
         yl = y-yh
         bin = 0
      endif
      if (hist_in.eq.-1) then
         print*," "
         print*," input to hf1dp: x,y = ",value,weight
         print*," bin = ",bin
         if (bin.eq.0) then
            print*," bin_or. = ",min( INT(xbin), nbin(hist_in) ),
     1           " binl,h = ",binl,binh
            print*," xbin = ",xbin," rat = ",rat
            print*," yl = ",yl," yh = ",yh
         endif
      endif

      if (nlo .ge. 1 .and. (calcerror1d.or.calcerrorGnuplot)) then
        HIST_CALLS(hist_in) = HIST_CALLS(hist_in) + 1
        HIST_CALLS_MAX(hist_in) = max(HIST_CALLS_MAX(hist_in),HIST_CALLS(hist_in))
        if (HIST_CALLS(hist_in).gt.max_calls) then
           print*, "***************************************" //
     &          "***********************"
           print*, "The same histogram was called too often per"//
     &          " phase space point!"
           print*, "Please increase max_calls in "//
     &          "utilities/hist_real.inc!"
           print*, "***************************************" //
     &          "***********************"
           STOP
        endif
      endif

      if (bin.eq.0) then
        if (nlo .ge. -1) then 
         SUM(HIST_IN,BINL) = SUM(HIST_IN,BINL) + YL
         SUM(HIST_IN,BINH) = SUM(HIST_IN,BINH) + YH
        endif
        if (calcerror1d.or.calcerrorGnuplot) then
         if (nlo .le. 0) then 
          SUMM(HIST_IN,BINL) = SUMM(HIST_IN,BINL) + YL
          SQUARE(HIST_in,BINL) = SQUARE(HIST_in,BINL) + YL*YL
          COUNT(HIST_in,BINL) = COUNT(HIST_in,BINL) + 1d0 !-rat
          SUMM(HIST_IN,BINH) = SUMM(HIST_IN,BINH) + YH
          SQUARE(HIST_in,BINH) = SQUARE(HIST_in,BINH) + YH*YH
          COUNT(HIST_in,BINH) = COUNT(HIST_in,BINH) + 1d0 ! rat
         endif
         if (nlo .ge. 1) then
           mem_L=0
           mem_H=0
           do i=1,2*(nlo-1)
             if (SAVE_BIN(HIST_in,i,HIST_CALLS(HIST_in)) .eq. BINL) mem_L = i
             if (SAVE_BIN(HIST_in,i,HIST_CALLS(HIST_in)) .eq. BINH) mem_H = i
           enddo
           if (mem_L.eq.0) then
             mem_L = 2*nlo - 1
             SAVE_BIN(HIST_in,mem_L,HIST_CALLS(HIST_in)) = BINL
           endif
           if (mem_H.eq.0) then
             mem_H = 2*nlo
             SAVE_BIN(HIST_in,mem_H,HIST_CALLS(HIST_in)) = BINH
           endif
           SAVE_Y(HIST_in,mem_L,HIST_CALLS(HIST_in)) = SAVE_Y(HIST_in,mem_L,HIST_CALLS(HIST_in)) + YL
           SAVE_Y(HIST_in,mem_H,HIST_CALLS(HIST_in)) = SAVE_Y(HIST_in,mem_H,HIST_CALLS(HIST_in)) + YH
         endif
        endif
      else
        if (nlo .ge. -1) then 
         SUM(HIST_IN,BIN) = SUM(HIST_IN,BIN) + Y
        endif
        if (calcerror1d.or.calcerrorGnuplot) then
         if (nlo .le. 0) then 
           SUMM(HIST_IN,BIN) = SUMM(HIST_IN,BIN) + Y
           SQUARE(HIST_in,BIN) = SQUARE(HIST_in,BIN) + Y*Y
           COUNT(HIST_in,BIN) = COUNT(HIST_in,BIN) + 1
         endif
         if (nlo .ge. 1) then
           mem=0
           do i=1,2*(nlo-1)
             if (SAVE_BIN(HIST_in,i,HIST_CALLS(HIST_in)) .eq. BIN) mem = i
           enddo
           if (mem.eq.0) then
             mem = 2*nlo
             SAVE_BIN(HIST_in,mem,HIST_CALLS(HIST_in)) = BIN
           endif
           SAVE_Y(HIST_in,mem,HIST_CALLS(HIST_in)) = SAVE_Y(HIST_in,mem,HIST_CALLS(HIST_in)) + Y
         endif
        endif
      endif
      
      RETURN
      END


c*************************************************************************
      SUBROUTINE FillHist2d(ID, X, Y, weight, NLO)
c*************************************************************************
c     Fill value in 2d-histogram. 
c     Smear over bins if value is within fraction drb of bin boundary
c*************************************************************************

      implicit none

#include "hist.inc"
#include "global.inc"

      Integer ID, NLO
      real*8 X, Y, weight
      Integer hist_in, BINX, binlx, binhx, BINY, binly, binhy
      real*8 xb, yb, wll, whl, wlh, whh, w, wh, wl, w_h, w_l
      real*8 drb, tdrbi, ratx, raty

c variables for real emission error estimation

#include "hist_real.inc"

      integer i,j
      integer mem_x, mem_y
      integer mem_x_L, mem_x_H, mem_y_L, mem_y_H

      if (weight .ne. weight) return

      if (smearbins) then
        drb   = smearamount
        tdrbi = 0.5d0/drb
      else
        drb   = 0.0d0
        tdrbi = 1.0d0
      endif

      hist_in = ID
      if (NLO.ne.0) hist_in=hist_in+NLOoffset2d
      w=weight*rwidX(hist_in)*rwidY(hist_in)
      xb = (X-HLOX(hist_in)) * RWIDX(hist_in)
      yb = (Y-HLOY(hist_in)) * RWIDY(hist_in)

      if (.not.h2dstored(HIST_in)) return

c     X-direction
      if (xb.gt.drb) then
         binx = min( INT(xb), nbinX(hist_in) ) 
         if ( (xb-binx).lt.drb ) then ! x in low end of bin    
            binlx = binx+1
            binhx = binx+2
            ratx = (xb-binx)*tdrbi + 0.5
            wh = w*ratx
            wl = w-wh
            binx = 0
         elseif( (xb-binx).lt.(1.0-drb) ) then ! x in main part of bin
            binx = binx+2
         elseif( (xb-binx).lt.1.0.and.xb.lt.nbinX(hist_in) ) then ! x in high part of bin
            binlx = binx+2
            binhx = binx+3
            ratx = (xb-binx-1.0)*tdrbi + 0.5
            wh = w*ratx
            wl = w-wh
            binx = 0
         else                   ! x more than drp above upper bin
            binx = nbinX(hist_in)+2
         endif
      elseif(xb.lt.-drb) then
         binx = 1
      else
         binlx = 1
         binhx = 2
         ratx = xb * tdrbi + 0.5 ! in [0,1]
         wh = w*ratx
         wl = w-wh
         binx = 0
      endif
c     Y-direction
      if (yb.gt.drb) then
         biny = min( INT(yb), nbinY(hist_in) ) 
         if ( (yb-biny).lt.drb ) then ! y in low end of bin    
            binly = biny+1
            binhy = biny+2
            raty = (yb-biny)*tdrbi + 0.5
            whh = wh*raty
            whl = wh-whh
            wlh = wl*raty
            wll = wl-wlh
            w_h = w*raty
            w_l = w-w_h
            biny = 0
         elseif( (yb-biny).lt.(1.0-drb) ) then ! y in main part of bin
            biny = biny+2
         elseif( (yb-biny).lt.1.0.and.yb.lt.nbinY(hist_in) ) then ! y in high part of bin
            binly = biny+2
            binhy = biny+3
            raty = (yb-biny-1.0)*tdrbi + 0.5
            whh = wh*raty
            whl = wh-whh
            wlh = wl*raty
            wll = wl-wlh
            w_h = w*raty
            w_l = w-w_h
            biny = 0
         else                   ! y more than drp above upper bin
            biny = nbinY(hist_in)+2
         endif
      elseif(yb.lt.-drb) then
         biny = 1
      else
         binly = 1
         binhy = 2
         raty = yb * tdrbi + 0.5 ! in [0,1]
         whh = wh*raty
         whl = wh-whh
         wlh = wl*raty
         wll = wl-wlh
         w_h = w*raty
         w_l = w-w_h
         biny = 0
      endif

      if (nlo .ge. 1 .and. calcerror2d) then
        HIST_CALLS_2D(hist_in) = HIST_CALLS_2D(hist_in) + 1
        HIST_CALLS_2D_MAX(hist_in) = max(HIST_CALLS_2D_MAX(hist_in),HIST_CALLS_2D(hist_in))
        if (HIST_CALLS_2D(hist_in).gt.max_calls) then
           print*, "**************************************************************"
           print*, "The same histogram was called too often per phase space point!"
           print*, "Please increase max_calls in utilities/hist_real.inc!"
           print*, "**************************************************************"
           STOP
        endif
      endif

      if (binx.eq.0) then
         if (biny.eq.0) then
           if (nlo .ge. -1) then 
            SUM2d(HIST_IN,BINLX,BINLY) = SUM2d(HIST_IN,BINLX,BINLY) + WLL
            SUM2d(HIST_IN,BINHX,BINLY) = SUM2d(HIST_IN,BINHX,BINLY) + WHL
            SUM2d(HIST_IN,BINLX,BINHY) = SUM2d(HIST_IN,BINLX,BINHY) + WLH
            SUM2d(HIST_IN,BINHX,BINHY) = SUM2d(HIST_IN,BINHX,BINHY) + WHH
           endif
           if (calcerror2d) then
            if (nlo .le. 0) then 
             SUMM2d(HIST_IN,BINLX,BINLY) = SUMM2d(HIST_IN,BINLX,BINLY) + WLL
             SQUARE2d(HIST_in,BINLX,BINLY) = SQUARE2d(HIST_in,BINLX,BINLY) + WLL*WLL
             COUNT2d(HIST_in,BINLX,BINLY) = COUNT2d(HIST_in,BINLX,BINLY) + 1

             SUMM2d(HIST_IN,BINHX,BINLY) = SUMM2d(HIST_IN,BINHX,BINLY) + WHL
             SQUARE2d(HIST_in,BINHX,BINLY) = SQUARE2d(HIST_in,BINHX,BINLY) + WHL*WHL
             COUNT2d(HIST_in,BINHX,BINLY) = COUNT2d(HIST_in,BINHX,BINLY) + 1

             SUMM2d(HIST_IN,BINLX,BINHY) = SUMM2d(HIST_IN,BINLX,BINHY) + WLH
             SQUARE2d(HIST_in,BINLX,BINHY) = SQUARE2d(HIST_in,BINLX,BINHY) + WLH*WLH
             COUNT2d(HIST_in,BINLX,BINHY) = COUNT2d(HIST_in,BINLX,BINHY) + 1

             SUMM2d(HIST_IN,BINHX,BINHY) = SUMM2d(HIST_IN,BINHX,BINHY) + WHH
             SQUARE2d(HIST_in,BINHX,BINHY) = SQUARE2d(HIST_in,BINHX,BINHY) + WHH*WHH
             COUNT2d(HIST_in,BINHX,BINHY) = COUNT2d(HIST_in,BINHX,BINHY) + 1
            endif
            if (nlo .ge. 1) then
              mem_x_L = 0
              mem_x_H = 0
              mem_y_L = 0
              mem_y_H = 0
              do i=1,2*(nlo-1)
                if (SAVE_BIN_X(i,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINLX) mem_x_L = i
                if (SAVE_BIN_X(i,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINHX) mem_x_H = i
              enddo
              do j=1,2*(nlo-1)
                if (SAVE_BIN_Y(j,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINLY) mem_y_L = j
                if (SAVE_BIN_Y(j,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINHY) mem_y_H = j
              enddo
              if (mem_x_L.eq.0) then
                mem_x_L=2*nlo -1
                SAVE_BIN_X(mem_x_L,HIST_in,HIST_CALLS_2D(HIST_in)) = BINLX
              endif
              if (mem_x_H.eq.0) then
                mem_x_H=2*nlo
                SAVE_BIN_X(mem_x_H,HIST_in,HIST_CALLS_2D(HIST_in)) = BINHX
              endif
              if (mem_y_L.eq.0) then
                mem_y_L=2*nlo - 1
                SAVE_BIN_Y(mem_y_L,HIST_in,HIST_CALLS_2D(HIST_in)) = BINLY
              endif
              if (mem_y_H.eq.0) then
                mem_y_H=2*nlo
                SAVE_BIN_Y(mem_y_H,HIST_in,HIST_CALLS_2D(HIST_in)) = BINHY
              endif
              SAVE_Y_2D(mem_x_L,mem_y_L,HIST_in,HIST_CALLS_2D(HIST_in)) 
     &            = SAVE_Y_2D(mem_x_L,mem_y_L,HIST_in,HIST_CALLS_2D(HIST_in)) + WLL
              SAVE_Y_2D(mem_x_L,mem_y_H,HIST_in,HIST_CALLS_2D(HIST_in)) 
     &            = SAVE_Y_2D(mem_x_L,mem_y_H,HIST_in,HIST_CALLS_2D(HIST_in)) + WLH
              SAVE_Y_2D(mem_x_H,mem_y_L,HIST_in,HIST_CALLS_2D(HIST_in)) 
     &            = SAVE_Y_2D(mem_x_H,mem_y_L,HIST_in,HIST_CALLS_2D(HIST_in)) + WHL
              SAVE_Y_2D(mem_x_H,mem_y_H,HIST_in,HIST_CALLS_2D(HIST_in)) 
     &            = SAVE_Y_2D(mem_x_H,mem_y_H,HIST_in,HIST_CALLS_2D(HIST_in)) + WHH
            endif
           endif
         else
           if (nlo .ge. -1) then 
            SUM2d(HIST_IN,BINLX,BINY) = SUM2d(HIST_IN,BINLX,BINY) + WL
            SUM2d(HIST_IN,BINHX,BINY) = SUM2d(HIST_IN,BINHX,BINY) + WH
           endif
           if (calcerror2d) then
            if (nlo .le. 0) then 
             SUMM2d(HIST_IN,BINLX,BINY) = SUMM2d(HIST_IN,BINLX,BINY) + WL
             SQUARE2d(HIST_in,BINLX,BINY) = SQUARE2d(HIST_in,BINLX,BINY) + WL*WL
             COUNT2d(HIST_in,BINLX,BINY) = COUNT2d(HIST_in,BINLX,BINY) + 1

             SUMM2d(HIST_IN,BINHX,BINY) = SUMM2d(HIST_IN,BINHX,BINY) + WH
             SQUARE2d(HIST_in,BINHX,BINY) = SQUARE2d(HIST_in,BINHX,BINY) + WH*WH
             COUNT2d(HIST_in,BINHX,BINY) = COUNT2d(HIST_in,BINHX,BINY) + 1
            endif
            if (nlo .ge. 1) then
              mem_x_L = 0
              mem_x_H = 0
              mem_y = 0
              do i=1,2*(nlo-1)
                if (SAVE_BIN_X(i,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINLX) mem_x_L = i
                if (SAVE_BIN_X(i,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINHX) mem_x_H = i
              enddo
              do j=1,2*(nlo-1)
                if (SAVE_BIN_Y(j,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINY) mem_y = j
              enddo
              if (mem_x_L.eq.0) then
                mem_x_L=2*nlo -1
                SAVE_BIN_X(mem_x_L,HIST_in,HIST_CALLS_2D(HIST_in)) = BINLX
              endif
              if (mem_x_H.eq.0) then
                mem_x_H=2*nlo
                SAVE_BIN_X(mem_x_H,HIST_in,HIST_CALLS_2D(HIST_in)) = BINHX
              endif
              if (mem_y.eq.0) then
                mem_y=2*nlo
                SAVE_BIN_Y(mem_y,HIST_in,HIST_CALLS_2D(HIST_in)) = BINY
              endif
              SAVE_Y_2D(mem_x_L,mem_y,HIST_in,HIST_CALLS_2D(HIST_in)) 
     &            = SAVE_Y_2D(mem_x_L,mem_y,HIST_in,HIST_CALLS_2D(HIST_in)) + WL
              SAVE_Y_2D(mem_x_H,mem_y,HIST_in,HIST_CALLS_2D(HIST_in)) 
     &            = SAVE_Y_2D(mem_x_H,mem_y,HIST_in,HIST_CALLS_2D(HIST_in)) + WH
            endif
           endif
         endif
      else
         if (biny.eq.0) then
           if (nlo .ge. -1) then 
            SUM2d(HIST_IN,BINX,BINLY) = SUM2d(HIST_IN,BINX,BINLY) + W_L
            SUM2d(HIST_IN,BINX,BINHY) = SUM2d(HIST_IN,BINX,BINHY) + W_H
           endif
           if (calcerror2d) then
            if (nlo .le. 0) then 
             SUMM2d(HIST_IN,BINX,BINLY) = SUMM2d(HIST_IN,BINX,BINLY) + W_L
             SQUARE2d(HIST_in,BINX,BINLY) = SQUARE2d(HIST_in,BINX,BINLY) + W_L*W_L
             COUNT2d(HIST_in,BINX,BINLY) = COUNT2d(HIST_in,BINX,BINLY) + 1

             SUMM2d(HIST_IN,BINX,BINHY) = SUMM2d(HIST_IN,BINX,BINHY) + W_H
             SQUARE2d(HIST_in,BINX,BINHY) = SQUARE2d(HIST_in,BINX,BINHY) + W_H*W_H
             COUNT2d(HIST_in,BINX,BINHY) = COUNT2d(HIST_in,BINX,BINHY) + 1
            endif
            if (nlo .ge. 1) then
              mem_x = 0
              mem_y_L = 0
              mem_y_H = 0
              do i=1,2*(nlo-1)
                if (SAVE_BIN_X(i,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINX) mem_x = i
              enddo
              do j=1,2*(nlo-1)
                if (SAVE_BIN_Y(j,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINLY) mem_y_L = j
                if (SAVE_BIN_Y(j,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINHY) mem_y_H = j
              enddo
              if (mem_x.eq.0) then
                mem_x=2*nlo
                SAVE_BIN_X(mem_x,HIST_in,HIST_CALLS_2D(HIST_in)) = BINX
              endif
              if (mem_y_L.eq.0) then
                mem_y_L=2*nlo - 1
                SAVE_BIN_Y(mem_y_L,HIST_in,HIST_CALLS_2D(HIST_in)) = BINLY
              endif
              if (mem_y_H.eq.0) then
                mem_y_H=2*nlo
                SAVE_BIN_Y(mem_y_H,HIST_in,HIST_CALLS_2D(HIST_in)) = BINHY
              endif
              SAVE_Y_2D(mem_x,mem_y_L,HIST_in,HIST_CALLS_2D(HIST_in)) 
     &            = SAVE_Y_2D(mem_x,mem_y_L,HIST_in,HIST_CALLS_2D(HIST_in)) + W_L
              SAVE_Y_2D(mem_x,mem_y_H,HIST_in,HIST_CALLS_2D(HIST_in)) 
     &            = SAVE_Y_2D(mem_x,mem_y_H,HIST_in,HIST_CALLS_2D(HIST_in)) + W_H
            endif
           endif
         else
           if (nlo .ge. -1) then 
            SUM2d(HIST_IN,BINX,BINY) = SUM2d(HIST_IN,BINX,BINY) + W
           endif
           if (calcerror2d) then
            if (nlo .le. 0) then 
              SUMM2d(HIST_IN,BINX,BINY) = SUMM2d(HIST_IN,BINX,BINY) + W
              SQUARE2d(HIST_in,BINX,BINY) = SQUARE2d(HIST_in,BINX,BINY) + W*W
              COUNT2d(HIST_in,BINX,BINY) = COUNT2d(HIST_in,BINX,BINY) + 1
            endif
            if (nlo .ge. 1) then
              mem_x = 0
              mem_y = 0
              do i=1,2*(nlo-1)
                if (SAVE_BIN_X(i,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINX) mem_x = i
              enddo
              do j=1,2*(nlo-1)
                if (SAVE_BIN_Y(j,HIST_in,HIST_CALLS_2D(HIST_in)) .eq. BINY) mem_y = j
              enddo
              if (mem_x.eq.0) then
                mem_x=2*nlo
                SAVE_BIN_X(mem_x,HIST_in,HIST_CALLS_2D(HIST_in)) = BINX
              endif
              if (mem_y.eq.0) then
                mem_y=2*nlo
                SAVE_BIN_Y(mem_y,HIST_in,HIST_CALLS_2D(HIST_in)) = BINY
              endif
              SAVE_Y_2D(mem_x,mem_y,HIST_in,HIST_CALLS_2D(HIST_in)) = SAVE_Y_2D(mem_x,mem_y,HIST_in,HIST_CALLS_2D(HIST_in)) + W
            endif
           endif
         endif
      endif

      RETURN
      END



      SUBROUTINE InitRealHist(kinematics)

      implicit none

#include "hist.inc"
#include "global.inc"
#include "hist_real.inc"

      integer id,j,k,kk, kinematics

      real_kinematics = kinematics

      do id = 1, histmax
        do j = 1, 2*max_kin
          do k = 1, max_calls
            SAVE_Y(id,j,k) = 0.
            SAVE_BIN(id,j,k) = 0
            HIST_CALLS(id) = 0
            HIST_CALLS_MAX(id) = 0
          enddo
        enddo
      enddo

      do id = 1, hist2dmax
        do j = 1, 2*max_kin
          do k = 1, max_calls
            do kk = 1, 2*max_kin
              SAVE_Y_2D(j,kk,id,k) = 0.
            enddo
            SAVE_BIN_X(j,id,k) = 0
            SAVE_BIN_X(j,id,k) = 0
            HIST_CALLS_2D(id) = 0
            HIST_CALLS_2D_MAX(id) = 0
          enddo
        enddo
      enddo

      RETURN
      END



      SUBROUTINE SaveRealHist()

      implicit none

#include "hist.inc"
#include "global.inc"
#include "hist_real.inc"

      integer calls,HIST_in,i,j

      if (calcerror1d.or.calcerrorGnuplot) then

      do HIST_in = NLOoffset+1,2*NLOoffset
        if (hstored(HIST_in)) then
          do calls=1,HIST_CALLS_MAX(HIST_in)
            do i=1,2*real_kinematics
              if (SAVE_BIN(HIST_in,i,calls).ne.0 .and. SAVE_Y(HIST_in,i,calls).ne.0.d0) then
                SUMM(HIST_IN,SAVE_BIN(HIST_in,i,calls)) = SUMM(HIST_IN,SAVE_BIN(HIST_in,i,calls)) + SAVE_Y(HIST_in,i,calls)
                SQUARE(HIST_in,SAVE_BIN(HIST_in,i,calls)) = SQUARE(HIST_in,SAVE_BIN(HIST_in,i,calls)) + SAVE_Y(HIST_in,i,calls)**2
                COUNT(HIST_in,SAVE_BIN(HIST_in,i,calls)) = COUNT(HIST_in,SAVE_BIN(HIST_in,i,calls)) + 1
              endif
              SAVE_BIN(HIST_in,i,calls)=0
              SAVE_Y(HIST_in,i,calls)=0.
            enddo
          enddo
        endif
        HIST_CALLS(HIST_in)=0
        HIST_CALLS_MAX(HIST_in)=0
      enddo

      endif

      if (calcerror2d) then

      do HIST_in = NLOoffset2d+1,2*NLOoffset2d
        if (h2dstored(HIST_in)) then
          do calls=1,HIST_CALLS_2D_MAX(HIST_in)
            do i=1,2*real_kinematics
             do j=1,2*real_kinematics
              if (SAVE_BIN_X(i,HIST_in,calls).ne.0 .and. 
     &               SAVE_BIN_Y(j,HIST_in,calls).ne.0 .and. 
     &               SAVE_Y_2D(i,j,HIST_in,calls).ne.0.d0) then
                 SUMM2d(HIST_IN,SAVE_BIN_X(i,HIST_in,calls),
     &                SAVE_BIN_Y(j,HIST_in,calls)) 
     &                = SUMM2d(HIST_IN,SAVE_BIN_X(i,HIST_in,calls),
     &                SAVE_BIN_Y(j,HIST_in,calls)) + 
     &                SAVE_Y_2D(i,j,HIST_in,calls)
                 SQUARE2d(HIST_in,SAVE_BIN_X(i,HIST_in,calls),
     &                SAVE_BIN_Y(j,HIST_in,calls)) 
     &                = SQUARE2d(HIST_in,SAVE_BIN_X(i,HIST_in,calls),
     &                SAVE_BIN_Y(j,HIST_in,calls)) + 
     &                SAVE_Y_2D(i,j,HIST_in,calls)**2
                 COUNT2d(HIST_in,SAVE_BIN_X(i,HIST_in,calls),
     &                SAVE_BIN_Y(j,HIST_in,calls)) 
     &                = COUNT2d(HIST_in,SAVE_BIN_X(i,HIST_in,calls),
     &                SAVE_BIN_Y(j,HIST_in,calls)) + 1
              endif
              SAVE_Y_2D(i,j,HIST_in,calls)=0.
             enddo
             SAVE_BIN_X(i,HIST_in,calls)=0
            enddo
            do j=1,2*real_kinematics
             SAVE_BIN_Y(j,HIST_in,calls)=0
            enddo
          enddo
        endif
        HIST_CALLS_2D(HIST_in)=0
        HIST_CALLS_2D_MAX(HIST_in)=0
      enddo

      endif

      RETURN
      END



c*************************************************************************
      SUBROUTINE CreateHist(ID, title, bins, min, max)
c*************************************************************************
c     Set up the histogram for the internal representation.
c*************************************************************************

      implicit none

#include "hist.inc"

      integer ID
      character*(*) title
      integer bins
      real*8 min, max
      character*14 idstring
      character*7 idstringnr
      real*8 histconf(2), histconfdefault(2)
      character*7 myadjustl
      external myadjustl

      histconfdefault(1) = min
      histconfdefault(2) = max

      write( idstringnr, '(i3)' )  ID
      idstringnr = myadjustl(idstringnr(1:len_trim(idstringnr)))
      idstring = "HIST_ID"//idstringnr

      
c check if internal arrays are large enough
      if (ID .gt. NLOoffset) then
        print*,"Error: Histogram id ",ID," larger than maximum:",HISTMAX,"/3"
        print*,"       Increase HISTMAX in utilities/hist.inc"
        stop
      endif
      if (bins .gt. BINMAX) then
        print*,"Error: Number of bins ",bins," for histogram ",ID,
     -        "larger than maximum:",BINMAX
        print*,"       Increase BINMAX in utilities/hist.inc"
        stop
      endif

** Reading in user-defined histogram limits
      call read_HistList(idstring,histconf,histconfdefault)
      write(*,'(2A,2F10.2,2A)'),idstring, "=", histconf, "  # ", title
      
      nbin(ID) = bins
      hlo(ID) = histconf(1)!min
      hup(ID) = histconf(2)!max
      htitles(ID) = title

      nbin(ID+NLOoffset) = bins
      hlo(ID+NLOoffset) = histconf(1)!min
      hup(ID+NLOoffset) = histconf(2)!max
c      htitles(ID+NLOoffset) = title(1:index(title,"  ")-1)//" (NLO)"
      write(htitles(ID+NLOoffset),*) title," (NLO)"
      htitles(ID+NLOoffset) = htitles(ID+NLOoffset)(1:len_trim(htitles(ID+NLOoffset)))
     
      nbin(ID+2*NLOoffset) = bins
      hlo(ID+2*NLOoffset) = histconf(1)!min
      hup(ID+2*NLOoffset) = histconf(2)!max
c      htitles(ID+2*NLOoffset) = title(1:index(title,"  ")-1)//" (K)"
      write(htitles(ID+2*NLOoffset),*) title," (K)"
      htitles(ID+2*NLOoffset) = htitles(ID+2*NLOoffset)(1:len_trim(htitles(ID+2*NLOoffset)))

      hstored(ID) = .true.
     
      RETURN
      END


c*************************************************************************
      SUBROUTINE Create2dHist(ID, title, xlabel, ylabel, xbins, ybins, 
     -     xmin, xmax, ymin, ymax)
c*************************************************************************
c     Set up a 2d-histogram for the internal representation.
c*************************************************************************

      implicit none

#include "hist.inc"

      integer ID
      character*(*) title, xlabel, ylabel
      integer xbins, ybins
      real*8 xmin, xmax, ymin, ymax
      
      character*14 idstring
      character*5 idstringnr
      real*8 histconf(4), histconfdefault(4)
      character*5 myadjustl
      external myadjustl
      
      histconfdefault(1) = xmin
      histconfdefault(2) = xmax
      histconfdefault(3) = ymin
      histconfdefault(4) = ymax
      
      write( idstringnr, '(i3)' )  ID
      idstringnr = myadjustl(idstringnr(1:len_trim(idstringnr)))
      idstring = "2DHIST_ID"//idstringnr
      
c check if internal arrays are large enough
      if (ID .gt. NLOoffset2d) then
        print*,"Error: 2d-Histogram id ",ID," larger than maximum:",HIST2dMAX,"/3"
        print*,"       Increase HIST2dMAX in utilities/hist.inc"
        stop
      endif
      if (xbins .gt. BINMAXX) then
        print*,"Error: Number of x-bins ",xbins," for 2d-histogram ",ID,"larger than maximum:",BINMAXX
        print*,"       Increase BINMAXX in utilities/hist.inc"
        stop
      endif
      if (ybins .gt. BINMAXY) then
        print*,"Error: Number of y-bins ",ybins," for 2d-histogram ",ID,"larger than maximum:",BINMAXY
        print*,"       Increase BINMAXY in utilities/hist.inc"
        stop
      endif

** Reading in user-defined axis limits
      call read_Hist2dList(idstring,histconf,histconfdefault)
      write(*,'(2A,4F10.2,2A)'),idstring, "=", histconf, "  # ", title
      
      nbinX(ID) = xbins
      nbinY(ID) = ybins
      hloX(ID) = histconf(1) !xmin
      hupX(ID) = histconf(2) !xmax
      hloY(ID) = histconf(3) !ymin
      hupY(ID) = histconf(4) !ymax      
      h2dtitles(ID) = title
      xlabels(ID) = xlabel
      ylabels(ID) = ylabel

      nbinX(ID+NLOoffset2d) = xbins
      hloX(ID+NLOoffset2d) = histconf(1) !xmin
      hupX(ID+NLOoffset2d) = histconf(2) !xmax
      nbinY(ID+NLOoffset2d) = ybins
      hloY(ID+NLOoffset2d) = histconf(3) !ymin
      hupY(ID+NLOoffset2d) = histconf(4) !ymax           
      write(h2dtitles(ID+NLOoffset2d),*) title," (NLO)"
      h2dtitles(ID+NLOoffset2d) = h2dtitles(ID+NLOoffset2d)(1:len_trim(h2dtitles(ID+NLOoffset2d)))
     
      nbinX(ID+2*NLOoffset2d) = xbins
      hloX(ID+2*NLOoffset2d) = histconf(1) !xmin
      hupX(ID+2*NLOoffset2d) = histconf(2) !xmax
      nbinY(ID+2*NLOoffset2d) = ybins
      hloY(ID+2*NLOoffset2d) = histconf(3) !ymin
      hupY(ID+2*NLOoffset2d) = histconf(4) !ymax      
      write(h2dtitles(ID+2*NLOoffset2d),*) title," (K)"
      h2dtitles(ID+2*NLOoffset2d) = h2dtitles(ID+2*NLOoffset2d)(1:len_trim(h2dtitles(ID+2*NLOoffset2d)))

      h2dstored(ID) = .true.
     
      RETURN
      END


c*************************************************************************
      SUBROUTINE WriteHistograms
c*************************************************************************
c     write out histograms in various formats
c*************************************************************************

      implicit none

#include "hist.inc"    

      print*,' '

#ifdef WITH_ROOT
      if (writeRoot) then 
        call CloseRootHists
        call WriteRootHists
      endif
#endif

* #ifdef WITH_CERN
*      if (writePaw) call WritePawHists
* #endif

      if (writeTop) call WriteTopHists

      if (writegnu) call WriteGnuHists

      if (writedata) call WriteDataHists

      RETURN
      END


#ifdef WITH_ROOT
c*************************************************************************
      SUBROUTINE WriteRootHists
c*************************************************************************
c     Transfers the histogram data to the Root format
c*************************************************************************

      implicit none

#include "hist.inc"

      integer i,ID,TID,histdp,bin,binx,biny
      double precision x,y,z
      character*4 htype
      
      DO TID = 1,NLOoffset
         IF (hstored(TID)) THEN
           
            do i=0,1
              ID = TID+i*NLOoffset
              if (i.eq.0) then
                htype = "LO"
              else if (i.eq.1) then
                htype = "NLO"
              endif
              call createroothisto(TID,ID,htitles(ID),
     &               nbin(ID),hlo(ID),hup(ID),htype)
              histdp = ID 
              do bin = 1,nbin(ID)+2
                 ! if (count(histdp,bin).gt.0d0) then
                 if (sum(histdp,bin).ne.0d0) then
                    x = hlo(ID) + dbin(ID)*(bin-1.5d0)
                    y = sum(histdp,bin)
                    call fillroothisto(ID,x,y)
                 endif
              enddo
            enddo
         ENDIF
      ENDDO

      DO TID = 1,NLOoffset2d
         IF (h2dstored(TID)) THEN

            do i=0,1
              ID = TID+i*NLOoffset2d
              if (i.eq.0) then
                htype = "LO"
              else if (i.eq.1) then
                htype = "NLO"
              endif
              call createroothisto2d(TID,ID,htitles(ID),
     &               nbinx(ID),hlox(ID),hupx(ID),
     &               nbiny(ID),hloy(ID),hupy(ID),htype)
              histdp = ID 
              do binx = 1,nbinx(ID)+2
                 do biny = 1,nbiny(ID)+2
                    ! if (count2d(histdp,binx,biny).gt.1d-3) then
                    if (sum2d(histdp,binx,biny).ne.0d0) then
                       x = hlox(ID) + dbinx(ID)*(binx-1.5d0)
                       y = hloy(ID) + dbiny(ID)*(biny-1.5d0)
                       z = sum2d(histdp,binx,biny)
                       call fillroothisto2d(ID,x,y,z)
                    endif
                 enddo
              enddo
            enddo
         ENDIF
      ENDDO

      call closerootsession
      print *," created root file for histograms : ", rootfile
      
      RETURN
      END
#endif


c$$$#ifdef WITH_CERN
c$$$c*************************************************************************
c$$$      SUBROUTINE WritePawHists
c$$$c*************************************************************************
c$$$c     Transfers the histogram data to the HBOOK format
c$$$c*************************************************************************
c$$$      implicit none
c$$$#include "hist.inc"
c$$$      integer ID,histdp,bin
c$$$      real*4 x,y, lo, up
c$$$      integer  nhmem
c$$$      parameter( nhmem=262144 )
c$$$      real  hmem(nhmem)
c$$$      common /pawc/ hmem
c$$$
c$$$      call hlimit( nhmem )
c$$$
c$$$      DO ID = 1,HISTMAX
c$$$         IF (hstored(ID)) THEN
c$$$            lo = hlo(id)
c$$$            up = hup(id)
c$$$            call hbook1( ID,htitles(ID), nbin(id), lo, up, 0.0)
c$$$            call hidopt( ID, "inte" )
c$$$            histdp = ID 
c$$$            do bin = 1,nbin(ID)+2
c$$$               if (count(histdp,bin).gt.1d-3) then
c$$$                  x = lo + dbin(ID)*(real(bin)-1.5)
c$$$                  y = sum(histdp,bin)
c$$$                  call hf1(ID,x,y)
c$$$               endif
c$$$            enddo
c$$$         ENDIF
c$$$      ENDDO
c$$$
c$$$      call hrput( 0, pawfile, "N" )
c$$$       print *," created paw file for histograms : ", pawfile
c$$$       
c$$$
c$$$      RETURN
c$$$      END
c$$$#endif      


c*************************************************************************
      SUBROUTINE WriteTopHists
c*************************************************************************
c     Transfers the histogram data to the top-drawer format
c*************************************************************************

      implicit none

#include "hist.inc"

      integer ID,histdp,bin, ibin, i1
      double precision x,y
      double precision delx,xres(0:1000,2), xmin,xmax,ymin,ymax,ymaxK,yminK

      OPEN(UNIT=20,FILE=topfile,STATUS="REPLACE")

      write (20,"(A)") " set size 11 by 8.5 "

      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
            write (20,*)
            write (20,"(a)") " new frame "

            write (20,"(A)") " set intensity 4 "
            write (20,"(A)") " set font duplex " 
            write (20,"(A)") " set tick size .05 "
            write (20,"(A)") " set title size 2.0 "
            
            delx = (hup(id)-hlo(id))/(real(nbin(id))+1e-10)
            XRES(0,1) = hlo(id) - DELX/2. ! stores underflow
            XRES(1,1) = hlo(id) + DELX/2.
            DO  IBIN = 2,nbin(id)+1
               XRES(IBIN,1) = XRES(IBIN-1,1) + DELX
            END DO
            xmin = hlo(id)
            xmax = hup(id)
            ymin = 1d33
            ymax = -1d33
            yminK = 0
            ymaxK = 2
            do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
               xres(ibin,2) = sum(id,ibin+1)               
               ymin = min( ymin, xres(ibin,2) )
               ymax = max( ymax, xres(ibin,2) )              
            enddo
            do i1 = 0,nbin(id)+1
               if (xres(i1,2).eq.0.) xres(i1,2) = 1e-20
            end do
            write (20,"(A,i3,a)") " title 2.0 7.5 ""Fig.",
     1           id,": "//htitles(id)
            write (20,"(A)") " set window  x 1.2 to 5.2 y 3 to 7.0 "
            write (20,202) " set limits x ",xmin," to ",xmax,
     1                             " y ",ymin," to ",ymax
 201        format(2(a,f8.2),2(a,f12.2))
 202        format(2(a,f8.2),2(a,f12.5)) 
            write(20,*) " ( "
            DO  IBIN = 1,NBIN(ID)
               WRITE (20,"(2g20.6)")  XRES(ibin,1), xres(ibin,2)
            END DO
            write (20,"(A)") " join solid "
                       
         ENDIF
      ENDDO

      CLOSE(20)

      print *," created top-drawer file for histograms : ", topfile

      RETURN
      END


c*************************************************************************
      SUBROUTINE WriteGnuHists
c*************************************************************************
c     Transfers the histogram data to the gnuplot format
c*************************************************************************

      implicit none

#include "hist.inc"

      integer ID,histdp,bin, ibin, i1, jbin
      double precision x,y
      double precision delx,xres(0:1000,3)
      double precision dely,xres2d(0:1000,0:1000,2)

      OPEN(UNIT=20,FILE=gnufile,STATUS="REPLACE")

      write (20,"(A)") 'set output "'//
     &gnufile(1:len_trim(gnufile)-3)//'.ps"'
      write (20,"(A)") "set terminal postscript color"
      write (20,"(A)") "set style data histep"
      write (20,"(A)") "# set style data lines"
      write (20,"(A)") "set key off"

c     the 1d-histograms
      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
            write (20,*)
            write (20,*) "#",ID,": ",
     &           htitles(id)(1:len_trim(htitles(id)))
            write (20,"(A)") 'set title "'//
     &           htitles(id)(1:len_trim(htitles(id)))//'"'
            if (calcerrorGnuplot) then
              write (20,"(A)") 'plot "-", "-" with yerrorbars lt 1 pt 0'
            else
              write (20,"(A)") 'plot "-" '
            endif
            
            delx = (hup(id)-hlo(id))/(real(nbin(id))+1e-10)
            XRES(0,1) = hlo(id) - DELX/2. ! stores underflow
            DO  IBIN = 1,nbin(id)+1
               XRES(IBIN,1) = XRES(IBIN-1,1) + DELX
            END DO

            if (id .le. 2*NLOoffset) then
c LO and NLO
              do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
                 xres(ibin,2) = sum(id,ibin+1)               
                 if (calcerrorGnuplot .and. count(id,ibin+1) .gt. 0d0) then
                   xres(ibin,3) = square(id,ibin+1) - 
     &                             summ(id,ibin+1)**2/count(id,ibin+1)
                 else
                   xres(ibin,3) = 0d0
                 endif
              enddo
            else
c K-factor
              do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
                 if (sum(id-2*NLOoffset,ibin+1).ne.0) then
                   xres(ibin,2) = 
     &               sum(id-NLOoffset,ibin+1)/sum(id-2*NLOoffset,ibin+1)
                   if (calcerrorGnuplot .and. summ(id-2*NLOoffset,ibin+1).ne.0) then
                     xres(ibin,3) = 
     &                square(id-2*NLOoffset,ibin+1)
     &                /summ(id-2*NLOoffset,ibin+1)**2 
     &                - 1d0/count(id-2*NLOoffset,ibin+1)
                   else
                     xres(ibin,3) = 0
                   endif
                   if (calcerrorGnuplot .and. summ(id-NLOoffset,ibin+1).ne.0) then
                     xres(ibin,3) = xres(ibin,3) + 
     &                  square(id-NLOoffset,ibin+1)
     &                  /summ(id-NLOoffset,ibin+1)**2 
     &                  - 1d0/count(id-NLOoffset,ibin+1)
                   endif
                   xres(ibin,3) = xres(ibin,3) * xres(ibin,2)**2 
                 else
                   xres(ibin,2) = -1
                   xres(ibin,3) = 0
                 endif
              enddo
            endif

            DO  IBIN = 1,NBIN(ID)
               WRITE (20,"(2g20.6)")  XRES(ibin,1), xres(ibin,2)
            END DO
            write (20,"(A)") "e"
            if (calcerrorGnuplot) then
              DO  IBIN = 1,NBIN(ID)
                 WRITE (20,"(3g20.6)")  XRES(ibin,1), xres(ibin,2), sqrt(xres(ibin,3))
              END DO
              write (20,"(A)") "e"
            endif
          ENDIF
      ENDDO

c     the 2d-histograms
      DO ID = 1, HIST2dMAX
         IF (h2dstored(ID)) THEN
            write (20,*)
            write (20,*) "#",ID,": ",
     &           h2dtitles(id)(1:len_trim(h2dtitles(id)))
            write (20,"(A)") "set view map"
            write (20,"(A)") "set size 3.9/5.0,3.4/3.0"
            write (20,"(A)") "unset surface"
            write (20,"(A)") "set style data pm3d"
            write (20,"(A)") "set style function pm3d"
            write (20,"(A)") "set pm3d implicit at b"            
            write (20,"(A)") "set pm3d map corners2color c1"
            write (20,"(A)") 'set title "'//
     &           h2dtitles(id)(1:len_trim(h2dtitles(id)))//'"'
            write (20,"(A)") 'set xlabel "'//
     &           xlabels(id)(1:len_trim(xlabels(id)))//'"'
            write (20,"(A)") 'set ylabel "'//
     &           ylabels(id)(1:len_trim(ylabels(id)))//'"'
c            write (20,"(A)") "set palette rgbformulae 7, 5, 15 negative"
            write (20,"(A)") "set palette rgbformulae 30,31,32 negative"

            write (20,"(A)") 'splot "-" '
            
            delx = (hupx(id)-hlox(id))/(real(nbinx(id))+1e-10)
            dely = (hupy(id)-hloy(id))/(real(nbiny(id))+1e-10)
            
            if (id .le. 2*NLOoffset2d) then
              DO IBIN = 1, NBINX(ID)+1
                 DO JBIN = 1, NBINY(ID)+1
                   xres2d(ibin,jbin,2) = sum2d(id,ibin,jbin)
                 ENDDO
              ENDDO
            else
              DO IBIN = 1, NBINX(ID)+1
                 DO JBIN = 1, NBINY(ID)+1
                   if (sum2d(id-2*NLOoffset2d,ibin,jbin).ne.0) then
                     xres2d(ibin,jbin,2) = 
     &           sum2d(id-NLOoffset2d,ibin,jbin)/sum2d(id-2*NLOoffset2d,ibin,jbin)
                   else
                     xres2d(ibin,jbin,2) = -1
                   endif
                 ENDDO
              ENDDO
            endif
                  
            DO IBIN = 1, NBINX(ID)+1
               DO JBIN = 1, NBINY(ID)+1
                 WRITE (20,"(3g20.6)")  hlox(id)+delx*(IBIN-1d0), 
     &                  hloy(id)+dely*(JBIN-1d0), xres2d(ibin,jbin,2)
               ENDDO
               write (20,"(A)") " "
            ENDDO
            write (20,"(A)") "e"
         ENDIF
      ENDDO

      CLOSE(20)
      print *," created GNUplot file for histograms : ", gnufile
      
      RETURN
      END


c*************************************************************************
      SUBROUTINE WriteDataHists
c*************************************************************************
c     Transfers the histogram data to a general data format
c*************************************************************************

      implicit none

#include "hist.inc"

      integer ID,histdp,bin, ibin, i1, jbin
      double precision x,y
      double precision delx,xres(0:1000,3)
      double precision dely,xres2d(0:1000,0:1000,3)
      character*255 tmpstr
      character*50  charid
      character*50 myadjustl
      external myadjustl

      write(tmpstr,*) 'mkdir -p '//datafile(1:len_trim(datafile))//"/LO"
      CALL system(tmpstr)
      write(tmpstr,*) 'mkdir -p '//datafile(1:len_trim(datafile))//"/NLO"
      CALL system(tmpstr)
      write(tmpstr,*) 'mkdir -p '//datafile(1:len_trim(datafile))//"/Kfac"
      CALL system(tmpstr)

c     the 1d-histograms
      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN

            if (id .le. NLOoffset) then
               write(charid,*) ID
               charid = myadjustl(charid)
               write(tmpstr,'(A)') datafile(1:len_trim(datafile))//
     &                             "/LO/hist."//
     &                             charid(1:len_trim(charid))//".dat"
            else if (id .le. 2*NLOoffset) then
               write(charid,*) ID-NLOoffset
               charid = myadjustl(charid)
               write(tmpstr,'(A)') datafile(1:len_trim(datafile))//
     &                             "/NLO/hist."//
     &                             charid(1:len_trim(charid))//".dat"
            else
               write(charid,*) ID-2*NLOoffset
               charid = myadjustl(charid)
               write(tmpstr,'(A)') datafile(1:len_trim(datafile))//
     &                             "/Kfac/hist."//
     &                             charid(1:len_trim(charid))//".dat"
            endif
            OPEN(UNIT=20,FILE=tmpstr,STATUS="REPLACE")
            write (20,*) "#",ID,": ",
     &           htitles(id)(1:len_trim(htitles(id)))

            delx = (hup(id)-hlo(id))/(real(nbin(id))+1e-10)
            XRES(0,1) = hlo(id) - DELX/2. ! stores underflow
            DO  IBIN = 1,nbin(id)+1
               XRES(IBIN,1) = XRES(IBIN-1,1) + DELX
            END DO

            if (id .le. 2*NLOoffset) then
c LO and NLO
              do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
                 xres(ibin,2) = sum(id,ibin+1)               
                 if (calcerror1d .and. count(id,ibin+1) .gt. 0d0) then
                   xres(ibin,3) = square(id,ibin+1) - 
     &                             summ(id,ibin+1)**2/count(id,ibin+1)
                 else
                   xres(ibin,3) = 0d0
                 endif
              enddo
            else
c K-factor
              do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
                 if (sum(id-2*NLOoffset,ibin+1).ne.0) then
                   xres(ibin,2) = 
     &               sum(id-NLOoffset,ibin+1)/sum(id-2*NLOoffset,ibin+1)
                   if (calcerror1d .and. summ(id-2*NLOoffset,ibin+1).ne.0) then
                     xres(ibin,3) = 
     &                square(id-2*NLOoffset,ibin+1)
     &                /summ(id-2*NLOoffset,ibin+1)**2 
     &                - 1d0/count(id-2*NLOoffset,ibin+1)
                   else
                     xres(ibin,3) = 0
                   endif
                   if (calcerror1d .and. summ(id-NLOoffset,ibin+1).ne.0) then
                     xres(ibin,3) = xres(ibin,3) + 
     &                  square(id-NLOoffset,ibin+1)
     &                  /summ(id-NLOoffset,ibin+1)**2 
     &                  - 1d0/count(id-NLOoffset,ibin+1)
                   endif
                   xres(ibin,3) = xres(ibin,3) * xres(ibin,2)**2 
                 else
                   xres(ibin,2) = -1
                   xres(ibin,3) = 0
                 endif
              enddo
            endif
            do ibin = 0,nbin(id)+1
               if (xres(ibin,3).lt.0d0) xres(ibin,3) = 0d0
            end do

            if (calcerror1d) then
              DO  IBIN = 1,NBIN(ID)
                 WRITE (20,"(3g20.6)")  XRES(ibin,1), xres(ibin,2), sqrt(xres(ibin,3))
              END DO
            else
              DO  IBIN = 1,NBIN(ID)
                 WRITE (20,"(2g20.6)")  XRES(ibin,1), xres(ibin,2)
              END DO
            endif

            close(20)
            
          ENDIF
      ENDDO

c     the 2d-histograms
      DO ID = 1, HIST2dMAX
         IF (h2dstored(ID)) THEN
            if (id .le. NLOoffset2d) then
               write(charid,*) ID
               charid = myadjustl(charid)
               write(tmpstr,'(A)') datafile(1:len_trim(datafile))//
     &                             "/LO/hist2."//
     &                             charid(1:len_trim(charid))//".dat"
            else if (id .le. 2*NLOoffset2d) then
               write(charid,*) ID-NLOoffset2d
               charid = myadjustl(charid)
               write(tmpstr,'(A)') datafile(1:len_trim(datafile))//
     &                             "/NLO/hist2."//
     &                             charid(1:len_trim(charid))//".dat"
            else
               write(charid,*) ID-2*NLOoffset2d
               charid = myadjustl(charid)
               write(tmpstr,'(A)') datafile(1:len_trim(datafile))//
     &                             "/Kfac/hist2."//
     &                             charid(1:len_trim(charid))//".dat"
            endif
            OPEN(UNIT=20,FILE=tmpstr,STATUS="REPLACE")
            write (20,*) "#",ID,": ",
     &           h2dtitles(id)(1:len_trim(h2dtitles(id)))
            
            delx = (hupx(id)-hlox(id))/(real(nbinx(id))+1e-10)
            dely = (hupy(id)-hloy(id))/(real(nbiny(id))+1e-10)
            
            if (id .le. 2*NLOoffset2d) then
              DO IBIN = 1, NBINX(ID)+1
                 DO JBIN = 1, NBINY(ID)+1
                   xres2d(ibin,jbin,2) = sum2d(id,ibin,jbin)
                   if (calcerror2d .and. count2d(id,ibin,jbin) .gt. 0d0) then
                     xres2d(ibin,jbin,3) = 
     &                 square2d(id,ibin,jbin) - 
     &                 summ2d(id,ibin,jbin)**2/count2d(id,ibin,jbin)
                   else
                     xres2d(ibin,jbin,3) = 0d0
                   endif
                 ENDDO
              ENDDO
            else
              DO IBIN = 1, NBINX(ID)+1
                 DO JBIN = 1, NBINY(ID)+1
                  if (sum2d(id-2*NLOoffset2d,ibin,jbin).ne.0) then
                    xres2d(ibin,jbin,2) = 
     &        sum2d(id-NLOoffset2d,ibin,jbin)/sum2d(id-2*NLOoffset2d,ibin,jbin)
                    if (calcerror2d .and. summ2d(id-2*NLOoffset,ibin,jbin).ne.0) then
                      xres2d(ibin,jbin,3) = 
     &                  square2d(id-2*NLOoffset,ibin,jbin)
     &                  /summ2d(id-2*NLOoffset,ibin,jbin)**2
     &                  - 1d0/count2d(id-2*NLOoffset,ibin,jbin)
                    else
                      xres2d(ibin,jbin,3) = 0
                    endif
                    if (calcerror2d .and. summ2d(id-NLOoffset,ibin,jbin).ne.0) then
                      xres2d(ibin,jbin,3) = xres2d(ibin,jbin,3) + 
     &                  square2d(id-NLOoffset,ibin,jbin)
     &                  /summ2d(id-NLOoffset,ibin,jbin)**2
     &                  - 1d0/count2d(id-NLOoffset,ibin,jbin)
                    endif
                    xres2d(ibin,jbin,3) = xres2d(ibin,jbin,3) * 
     &                                    xres2d(ibin,jbin,2)**2 
                  else
                    xres2d(ibin,jbin,2) = -1
                    xres2d(ibin,jbin,3) = 0
                  endif
                 ENDDO
              ENDDO
            endif
            do ibin = 0,nbinx(id)+1
              do jbin = 0,nbiny(id)+1
               if (xres2d(ibin,jbin,3).lt.0d0) xres2d(ibin,jbin,3) = 0d0
              end do
            end do
                  
            if (calcerror2d) then
              DO IBIN = 1, NBINX(ID)+1
                 DO JBIN = 1, NBINY(ID)+1
                   WRITE (20,"(4g20.6)")  hlox(id)+delx*(IBIN-1d0), 
     &                    hloy(id)+dely*(JBIN-1d0), xres2d(ibin,jbin,2),
     &                    sqrt(xres2d(ibin,jbin,3))
                 ENDDO
                 write (20,"(A)") " "
              ENDDO
            else
              DO IBIN = 1, NBINX(ID)+1
                 DO JBIN = 1, NBINY(ID)+1
                   WRITE (20,"(3g20.6)")  hlox(id)+delx*(IBIN-1d0), 
     &                    hloy(id)+dely*(JBIN-1d0), xres2d(ibin,jbin,2)
                 ENDDO
                 write (20,"(A)") " "
              ENDDO
            endif
            CLOSE(20)
         ENDIF
      ENDDO

      print *," created data files for histograms : ", datafile
      
      RETURN
      END


ccccccccccccccccccccc begin subroutine SelectFile ccccccccccccccc
c
c  This subroutine selects a file name which does not exist yet
c  Terrance Figy <terrance@pheno.physics.wisc.edu
c  August 17, 2001  
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine SelectFile(name,extens)

      implicit none

      character*50 name,lname,lname1
      character*1 digit(0:9)
      character*5 extens
      integer ii,i,j
      logical FileAlreadyExists
      data DIGIT /"0","1","2","3","4","5","6","7","8","9"/
 
      lname1 = name
      do 20 ii = 1,99
     
         if((ii.ge.1).and.(ii.le.9)) then
            
            lname = 
     &           lname1(1:len_trim(lname1))//"."//DIGIT(ii)//extens
            inquire(file=lname,exist=FileAlreadyExists)
            if(FileAlreadyExists) then
               goto 20
            else
               name = lname(1:index(lname," ")-1)
               return
            endif
         elseif(ii.gt.9) then
            i = (ii/10)
            j = mod(ii,10)
            lname = lname1(1:len_trim(lname1))//"."//
     &              DIGIT(i)//DIGIT(j)//extens
            inquire(file=lname,exist=FileAlreadyExists)
            if(FileAlreadyExists) then
               goto 20
            else
               name = lname(1:len_trim(lname))
               return
            endif
         endif

         name = name(1:len_trim(name))
         
 20      continue
         write(*,*) "-------- fatal crash in SelectFile ----------"
         stop
         end


ccccccccccccccccccccc function myadjustl ccccccccccccccc
c
c implementation of FORTRAN90 intrinsic adjustl
c returns character string left-aligned
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      function myadjustl(instr)
      implicit none
      character(*) instr, myadjustl

      myadjustl = instr
      do while (myadjustl(1:1) == ' ') 
        myadjustl = myadjustl(2:len_trim(myadjustl))
      enddo

      return
      end
